package db

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectBlockGreenChainFlows 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `BlockGreenChainFlowFindByParams` 方法
func BatchSelectBlockGreenChainFlows(Db *xorm.Engine, params map[string]interface{}) (*[]model.BlockGreenChainFlow, error) {
	var BlockGreenChainFlowData []model.BlockGreenChainFlow
	if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).
		Find(&BlockGreenChainFlowData); err != nil {
		return nil, zhios_order_relate_logx.Warn(err)
	}
	return &BlockGreenChainFlowData, nil
}

// BlockGreenChainFlowInsert 插入单条数据
func BlockGreenChainFlowInsert(session *xorm.Session, BlockGreenChainFlow *model.BlockGreenChainFlow) (int64, error) {
	_, err := session.InsertOne(BlockGreenChainFlow)
	if err != nil {
		return 0, err
	}
	return BlockGreenChainFlow.Id, nil
}

// BatchAddBlockGreenChainFlows 批量新增数据
func BatchAddBlockGreenChainFlows(Db *xorm.Engine, BlockGreenChainFlowData []*model.BlockGreenChainFlow) (int64, error) {
	affected, err := Db.Insert(BlockGreenChainFlowData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetBlockGreenChainFlowCount(Db *xorm.Engine) int {
	var BlockGreenChainFlow model.BlockGreenChainFlow
	session := Db.Where("")
	count, err := session.Count(&BlockGreenChainFlow)
	if err != nil {
		return 0
	}
	return int(count)
}

// BlockGreenChainFlowDelete 删除记录
func BlockGreenChainFlowDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.BlockGreenChainFlow{})
	} else {
		return Db.Where("id = ?", id).Delete(model.BlockGreenChainFlow{})
	}
}

// BlockGreenChainFlowUpdate 更新记录
func BlockGreenChainFlowUpdate(session *xorm.Session, id interface{}, BlockGreenChainFlow *model.BlockGreenChainFlow, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = session.Where("id=?", id).Cols(forceColums...).Update(BlockGreenChainFlow)
	} else {
		affected, err = session.Where("id=?", id).Update(BlockGreenChainFlow)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// BlockGreenChainFlowGetOneByParams 通过传入的参数查询数据(单条)
func BlockGreenChainFlowGetOneByParams(session *xorm.Session, params map[string]interface{}) (*model.BlockGreenChainFlow, error) {
	var m model.BlockGreenChainFlow
	var query = fmt.Sprintf("%s =?", params["key"])
	has, err := session.Where(query, params["value"]).Get(&m)
	if err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	if has == false {
		return nil, errors.New("未查询到相应的 block_star_chain 记录")
	}
	return &m, nil
}

// BlockGreenChainFlowFindByParams 通过传入的参数查询数据(多条)
func BlockGreenChainFlowFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.BlockGreenChainFlow, error) {
	var m []model.BlockGreenChainFlow
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}

func BlockGreenChainFlowFindByParamsByPage(Db *xorm.Engine, params map[string]interface{}, page, pageSize int) (*[]model.BlockGreenChainFlow, error) {
	var m []model.BlockGreenChainFlow
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if page == 0 && pageSize == 0 {
		page = 1
		pageSize = 10
	}

	if params["key"] == nil {
		//查询全部数据
		err := Db.Limit(pageSize, (page-1)*pageSize).Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}
