package db

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectBlockStarChains 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `BlockStarChainFindByParams` 方法
func BatchSelectBlockStarChains(Db *xorm.Engine, params map[string]interface{}) (*[]model.BlockStarChain, error) {
	var BlockStarChainData []model.BlockStarChain
	if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).
		Find(&BlockStarChainData); err != nil {
		return nil, zhios_order_relate_logx.Warn(err)
	}
	return &BlockStarChainData, nil
}

// BlockStarChainInsert 插入单条数据
func BlockStarChainInsert(Db *xorm.Engine, BlockStarChain *model.BlockStarChain) (int, error) {
	_, err := Db.InsertOne(BlockStarChain)
	if err != nil {
		return 0, err
	}
	return BlockStarChain.Id, nil
}

// BatchAddBlockStarChains 批量新增数据
func BatchAddBlockStarChains(Db *xorm.Engine, BlockStarChainData []*model.BlockStarChain) (int64, error) {
	affected, err := Db.Insert(BlockStarChainData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetBlockStarChainCount(Db *xorm.Engine) int {
	var BlockStarChain model.BlockStarChain
	session := Db.Where("")
	count, err := session.Count(&BlockStarChain)
	if err != nil {
		return 0
	}
	return int(count)
}

// BlockStarChainDelete 删除记录
func BlockStarChainDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.BlockStarChain{})
	} else {
		return Db.Where("id = ?", id).Delete(model.BlockStarChain{})
	}
}

// BlockStarChainUpdate 更新记录
func BlockStarChainUpdate(session *xorm.Session, id interface{}, BlockStarChain *model.BlockStarChain, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = session.Where("id=?", id).Cols(forceColums...).Update(BlockStarChain)
	} else {
		affected, err = session.Where("id=?", id).Update(BlockStarChain)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// BlockStarChainGetOneByParams 通过传入的参数查询数据(单条)
func BlockStarChainGetOneByParams(session *xorm.Session, params map[string]interface{}) (*model.BlockStarChain, error) {
	var m model.BlockStarChain
	var query = fmt.Sprintf("%s =?", params["key"])
	has, err := session.Where(query, params["value"]).Get(&m)
	if err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	if has == false {
		return nil, errors.New("未查询到相应的 block_star_chain 记录")
	}
	return &m, nil
}

// BlockStarChainFindByParams 通过传入的参数查询数据(多条)
func BlockStarChainFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.BlockStarChain, error) {
	var m []model.BlockStarChain
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}

func BlockStarChainFindByParamsByPage(Db *xorm.Engine, params map[string]interface{}, page, pageSize int) (*[]model.BlockStarChain, error) {
	var m []model.BlockStarChain
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if page == 0 && pageSize == 0 {
		page = 1
		pageSize = 10
	}

	if params["key"] == nil {
		//查询全部数据
		err := Db.Limit(pageSize, (page-1)*pageSize).Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}
