package db

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectBlockStarChainFlows 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `BlockStarChainFlowFindByParams` 方法
func BatchSelectBlockStarChainFlows(Db *xorm.Engine, params map[string]interface{}) (*[]model.BlockStarChainFlow, error) {
	var BlockStarChainFlowData []model.BlockStarChainFlow
	if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).
		Find(&BlockStarChainFlowData); err != nil {
		return nil, zhios_order_relate_logx.Warn(err)
	}
	return &BlockStarChainFlowData, nil
}

// BlockStarChainFlowInsert 插入单条数据
func BlockStarChainFlowInsert(session *xorm.Session, BlockStarChainFlow *model.BlockStarChainFlow) (int64, error) {
	_, err := session.InsertOne(BlockStarChainFlow)
	if err != nil {
		return 0, err
	}
	return BlockStarChainFlow.Id, nil
}

// BatchAddBlockStarChainFlows 批量新增数据
func BatchAddBlockStarChainFlows(Db *xorm.Engine, BlockStarChainFlowData []*model.BlockStarChainFlow) (int64, error) {
	affected, err := Db.Insert(BlockStarChainFlowData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetBlockStarChainFlowCount(Db *xorm.Engine) int {
	var BlockStarChainFlow model.BlockStarChainFlow
	session := Db.Where("")
	count, err := session.Count(&BlockStarChainFlow)
	if err != nil {
		return 0
	}
	return int(count)
}

// BlockStarChainFlowDelete 删除记录
func BlockStarChainFlowDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.BlockStarChainFlow{})
	} else {
		return Db.Where("id = ?", id).Delete(model.BlockStarChainFlow{})
	}
}

// BlockStarChainFlowUpdate 更新记录
func BlockStarChainFlowUpdate(session *xorm.Session, id interface{}, BlockStarChainFlow *model.BlockStarChainFlow, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = session.Where("id=?", id).Cols(forceColums...).Update(BlockStarChainFlow)
	} else {
		affected, err = session.Where("id=?", id).Update(BlockStarChainFlow)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// BlockStarChainFlowGetOneByParams 通过传入的参数查询数据(单条)
func BlockStarChainFlowGetOneByParams(Db *xorm.Engine, params map[string]interface{}) (*model.BlockStarChainFlow, error) {
	var m model.BlockStarChainFlow
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := Db.Where(query, params["value"]).Get(&m); err != nil || has == false {
		return nil, zhios_order_relate_logx.Error(err)
	}
	return &m, nil
}

// BlockStarChainFlowFindByParams 通过传入的参数查询数据(多条)
func BlockStarChainFlowFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.BlockStarChainFlow, error) {
	var m []model.BlockStarChainFlow
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}

func BlockStarChainFlowFindByParamsByPage(Db *xorm.Engine, params map[string]interface{}, page, pageSize int) (*[]model.BlockStarChainFlow, error) {
	var m []model.BlockStarChainFlow
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if page == 0 && pageSize == 0 {
		page = 1
		pageSize = 10
	}

	if params["key"] == nil {
		//查询全部数据
		err := Db.Limit(pageSize, (page-1)*pageSize).Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}
