package db

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"errors"
	"time"
	"xorm.io/xorm"
)

func InsertCapitalPoolLossMoney(sess *xorm.Session, uid, money, title, types, formType string) error {
	if zhios_order_relate_utils.StrToFloat64(money) <= 0 {
		return nil
	}
	moneyData, err := GetCapitalPoolLossMoney(sess, uid)
	if err != nil {
		return err
	}
	if moneyData == nil {
		return errors.New("查不到损失记录")
	}
	beforeMoney := moneyData.Money
	moneyData.Money = zhios_order_relate_utils.Float64ToStrByPrec(zhios_order_relate_utils.StrToFloat64(beforeMoney)+zhios_order_relate_utils.StrToFloat64(money), 3)
	afterMoney := moneyData.Money
	update, err := sess.Where("uid=?", uid).Update(moneyData)
	if update == 0 || err != nil {
		return errors.New("失败")
	}
	var flow = &model.CapitalPoolLossMoneyFlow{
		Uid:         zhios_order_relate_utils.StrToInt(uid),
		Money:       money,
		Time:        time.Now(),
		BeforeMoney: beforeMoney,
		AfterMoney:  afterMoney,
		Type:        zhios_order_relate_utils.StrToInt(types),
		FormType:    formType,
		Title:       title,
	}
	insert, err := sess.Insert(flow)
	if insert == 0 || err != nil {
		return errors.New("失败")
	}
	return nil
}
func GetCapitalPoolLossMoney(sess *xorm.Session, uid string) (*model.CapitalPoolLossMoney, error) {
	var data model.CapitalPoolLossMoney
	get, err := sess.Where("uid=?", uid).Get(&data)
	if err != nil {
		return nil, err
	}
	if get == false {
		data = model.CapitalPoolLossMoney{Uid: zhios_order_relate_utils.StrToInt(uid)}
		sess.Insert(&data)
	}
	return &data, nil

}
func SelfBuyStatisticalBonusValue(sess *xorm.Session) (poolDepositSum, poolBonusSum, nowDepositSum, poolBonusSumOut float64) {
	var (
		capitalPool     model.NewCapitalPoolOrd
		bonusCapital    model.NewCapitalPoolBonus
		bonusCapitalOut model.NewCapitalPoolBonus
		err             error
	)
	poolDepositSum, err = sess.Where("commission_type = ?", "cny").Sum(capitalPool, "deposit_value")
	poolBonusSum, err = sess.Sum(bonusCapital, "bonus_value")
	poolBonusSumOut, err = sess.Sum(bonusCapitalOut, "out_bonus_value")
	if err != nil {
		return
	}
	nowDepositSum = poolDepositSum - (poolBonusSum - poolBonusSumOut)
	return
}
func GetBonusValueAndCoin(sess *xorm.Session, capitalPoolCfg *model.NewCapitalPool, money string) *md.PoolBonus {
	var (
		bonus md.PoolBonus
	)
	_, _, nowDepositSum, _ := SelfBuyStatisticalBonusValue(sess)
	bonus.OldBonusValue = nowDepositSum
	if zhios_order_relate_utils.StrToFloat64(money) > 0 {
		nowDepositSum = zhios_order_relate_utils.StrToFloat64(money)
	}
	if nowDepositSum == 0 {
		return nil
	}
	if capitalPoolCfg != nil {
		bonus.SecuritiesId = capitalPoolCfg.SecuritiesId
		bonus.BonusValue = nowDepositSum
		bonus.BonusLevelType = capitalPoolCfg.BonusLevelType
	}
	return &bonus
}
