package db

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectGreenCoinDoubleChains 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `GreenCoinDoubleChainFindByParams` 方法
func BatchSelectGreenCoinDoubleChains(Db *xorm.Engine, params map[string]interface{}) (*[]model.GreenCoinDoubleChain, error) {
	var GreenCoinDoubleChainData []model.GreenCoinDoubleChain
	if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).
		Find(&GreenCoinDoubleChainData); err != nil {
		return nil, zhios_order_relate_logx.Warn(err)
	}
	return &GreenCoinDoubleChainData, nil
}

// GreenCoinDoubleChainInsert 插入单条数据
func GreenCoinDoubleChainInsert(Db *xorm.Engine, GreenCoinDoubleChain *model.GreenCoinDoubleChain) (int, error) {
	_, err := Db.InsertOne(GreenCoinDoubleChain)
	if err != nil {
		return 0, err
	}
	return GreenCoinDoubleChain.Id, nil
}

// BatchAddGreenCoinDoubleChains 批量新增数据
func BatchAddGreenCoinDoubleChains(Db *xorm.Engine, GreenCoinDoubleChainData []*model.GreenCoinDoubleChain) (int64, error) {
	affected, err := Db.Insert(GreenCoinDoubleChainData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetGreenCoinDoubleChainCount(Db *xorm.Engine) int {
	var GreenCoinDoubleChain model.GreenCoinDoubleChain
	session := Db.Where("")
	count, err := session.Count(&GreenCoinDoubleChain)
	if err != nil {
		return 0
	}
	return int(count)
}

// GreenCoinDoubleChainDelete 删除记录
func GreenCoinDoubleChainDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.GreenCoinDoubleChain{})
	} else {
		return Db.Where("id = ?", id).Delete(model.GreenCoinDoubleChain{})
	}
}

// GreenCoinDoubleChainUpdate 更新记录
func GreenCoinDoubleChainUpdate(session *xorm.Session, id interface{}, GreenCoinDoubleChain *model.GreenCoinDoubleChain, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = session.Where("id=?", id).Cols(forceColums...).Update(GreenCoinDoubleChain)
	} else {
		affected, err = session.Where("id=?", id).Update(GreenCoinDoubleChain)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// GreenCoinDoubleChainGetOneByParams 通过传入的参数查询数据(单条)
func GreenCoinDoubleChainGetOneByParams(session *xorm.Engine, params map[string]interface{}) (*model.GreenCoinDoubleChain, error) {
	var m model.GreenCoinDoubleChain
	var query = fmt.Sprintf("%s =?", params["key"])
	has, err := session.Where(query, params["value"]).Get(&m)
	if err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	if has == false {
		return nil, errors.New("未查询到相应的 block_star_chain 记录")
	}
	return &m, nil
}

// GreenCoinDoubleChainFindByParams 通过传入的参数查询数据(多条)
func GreenCoinDoubleChainFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.GreenCoinDoubleChain, error) {
	var m []model.GreenCoinDoubleChain
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}

func GreenCoinDoubleChainFindByParamsByPage(Db *xorm.Engine, params map[string]interface{}, page, pageSize int) (*[]model.GreenCoinDoubleChain, error) {
	var m []model.GreenCoinDoubleChain
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if page == 0 && pageSize == 0 {
		page = 1
		pageSize = 10
	}

	if params["key"] == nil {
		//查询全部数据
		err := Db.Limit(pageSize, (page-1)*pageSize).Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}
