package db

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectIntegralReleases 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `IntegralReleaseFindByParams` 方法
func BatchSelectIntegralReleases(Db *xorm.Engine, params map[string]interface{}) (*[]model.IntegralReleaseBasic, error) {
	var IntegralReleaseData []model.IntegralReleaseBasic
	if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).
		Find(&IntegralReleaseData); err != nil {
		return nil, zhios_order_relate_logx.Warn(err)
	}
	return &IntegralReleaseData, nil
}

// IntegralReleaseInsert 插入单条数据
func IntegralReleaseInsert(Db *xorm.Engine, IntegralRelease *model.IntegralReleaseBasic) (int, error) {
	_, err := Db.InsertOne(IntegralRelease)
	if err != nil {
		return 0, err
	}
	return IntegralRelease.Id, nil
}

// BatchAddIntegralReleases 批量新增数据
func BatchAddIntegralReleases(Db *xorm.Engine, IntegralReleaseData []*model.IntegralReleaseBasic) (int64, error) {
	affected, err := Db.Insert(IntegralReleaseData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetIntegralReleaseCount(Db *xorm.Engine) int {
	var IntegralRelease model.IntegralReleaseBasic
	session := Db.Where("")
	count, err := session.Count(&IntegralRelease)
	if err != nil {
		return 0
	}
	return int(count)
}

// IntegralReleaseDelete 删除记录
func IntegralReleaseDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.IntegralReleaseBasic{})
	} else {
		return Db.Where("id = ?", id).Delete(model.IntegralReleaseBasic{})
	}
}

// IntegralReleaseUpdate 更新记录
func IntegralReleaseUpdate(session *xorm.Session, id interface{}, IntegralRelease *model.IntegralReleaseBasic, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = session.Where("id=?", id).Cols(forceColums...).Update(IntegralRelease)
	} else {
		affected, err = session.Where("id=?", id).Update(IntegralRelease)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// IntegralReleaseGetOneByParams 通过传入的参数查询数据(单条)
func IntegralReleaseGetOneByParams(session *xorm.Session, params map[string]interface{}) (*model.IntegralReleaseBasic, error) {
	var m model.IntegralReleaseBasic
	var query = fmt.Sprintf("%s =?", params["key"])
	has, err := session.Where(query, params["value"]).Get(&m)
	if err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	if has == false {
		return nil, errors.New("未查询到相应的 block_star_chain 记录")
	}
	return &m, nil
}
func IntegralReleaseGetOneByParamsEg(eg *xorm.Engine, params map[string]interface{}) (*model.IntegralReleaseBasic, error) {
	var m model.IntegralReleaseBasic
	var query = fmt.Sprintf("%s =?", params["key"])
	has, err := eg.Where(query, params["value"]).Get(&m)
	if err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	if has == false {
		return nil, errors.New("未查询到相应的 block_star_chain 记录")
	}
	return &m, nil
}

// IntegralReleaseFindByParams 通过传入的参数查询数据(多条)
func IntegralReleaseFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.IntegralReleaseBasic, error) {
	var m []model.IntegralReleaseBasic
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}

func IntegralReleaseFindByParamsByPage(Db *xorm.Engine, params map[string]interface{}, page, pageSize int) (*[]model.IntegralReleaseBasic, error) {
	var m []model.IntegralReleaseBasic
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if page == 0 && pageSize == 0 {
		page = 1
		pageSize = 10
	}

	if params["key"] == nil {
		//查询全部数据
		err := Db.Limit(pageSize, (page-1)*pageSize).Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}
