package db

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// SubsidyBaseInsert 插入单条数据
func SubsidyBaseInsert(Db *xorm.Engine, SubsidyBase *model.SubsidyBase) (int, error) {
	_, err := Db.InsertOne(SubsidyBase)
	if err != nil {
		return 0, err
	}
	return SubsidyBase.Id, nil
}

// BatchAddSubsidyBases 批量新增数据
func BatchAddSubsidyBases(Db *xorm.Engine, SubsidyBaseData []*model.SubsidyBase) (int64, error) {
	affected, err := Db.Insert(SubsidyBaseData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetSubsidyBaseCount(Db *xorm.Engine) int {
	var SubsidyBase model.SubsidyBase
	session := Db.Where("")
	count, err := session.Count(&SubsidyBase)
	if err != nil {
		return 0
	}
	return int(count)
}

// SubsidyBaseDelete 删除记录
func SubsidyBaseDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.SubsidyBase{})
	} else {
		return Db.Where("id = ?", id).Delete(model.SubsidyBase{})
	}
}

// SubsidyBaseUpdate 更新记录
func SubsidyBaseUpdate(session *xorm.Session, id interface{}, SubsidyBase *model.SubsidyBase, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = session.Where("id=?", id).Cols(forceColums...).Update(SubsidyBase)
	} else {
		affected, err = session.Where("id=?", id).Update(SubsidyBase)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// SubsidyBaseGetOneByParams 通过传入的参数查询数据(单条)
func SubsidyBaseGetOneByParams(session *xorm.Session, params map[string]interface{}) (*model.SubsidyBase, error) {
	var m model.SubsidyBase
	var query = fmt.Sprintf("%s =?", params["key"])
	has, err := session.Where(query, params["value"]).Get(&m)
	if err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	if has == false {
		return nil, errors.New("未查询到相应的 block_star_chain 记录")
	}
	return &m, nil
}

// SubsidyBaseFindByParams 通过传入的参数查询数据(多条)
func SubsidyBaseFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.SubsidyBase, error) {
	var m []model.SubsidyBase
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}

func SubsidyBaseFindByParamsByPage(Db *xorm.Engine, params map[string]interface{}, page, pageSize int) (*[]model.SubsidyBase, error) {
	var m []model.SubsidyBase
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if page == 0 && pageSize == 0 {
		page = 1
		pageSize = 10
	}

	if params["key"] == nil {
		//查询全部数据
		err := Db.Limit(pageSize, (page-1)*pageSize).Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}
