package db

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// SubsidyWithUserInsert 插入单条数据
func SubsidyWithUserInsert(Db *xorm.Engine, SubsidyWithUser *model.SubsidyWithUser) (int, error) {
	_, err := Db.InsertOne(SubsidyWithUser)
	if err != nil {
		return 0, err
	}
	return SubsidyWithUser.Id, nil
}

// BatchAddSubsidyWithUsers 批量新增数据
func BatchAddSubsidyWithUsers(Db *xorm.Engine, SubsidyWithUserData []*model.SubsidyWithUser) (int64, error) {
	affected, err := Db.Insert(SubsidyWithUserData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetSubsidyWithUserCount(Db *xorm.Engine) int {
	var SubsidyWithUser model.SubsidyWithUser
	session := Db.Where("")
	count, err := session.Count(&SubsidyWithUser)
	if err != nil {
		return 0
	}
	return int(count)
}

// SubsidyWithUserDelete 删除记录
func SubsidyWithUserDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.SubsidyWithUser{})
	} else {
		return Db.Where("id = ?", id).Delete(model.SubsidyWithUser{})
	}
}

// SubsidyWithUserUpdate 更新记录
func SubsidyWithUserUpdate(session *xorm.Session, id interface{}, SubsidyWithUser *model.SubsidyWithUser, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = session.Where("id=?", id).Cols(forceColums...).Update(SubsidyWithUser)
	} else {
		affected, err = session.Where("id=?", id).Update(SubsidyWithUser)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// SubsidyWithUserGetOneByParams 通过传入的参数查询数据(单条)
func SubsidyWithUserGetOneByParams(session *xorm.Session, params map[string]interface{}) (*model.SubsidyWithUser, error) {
	var m model.SubsidyWithUser
	var query = fmt.Sprintf("%s =?", params["key"])
	has, err := session.Where(query, params["value"]).Get(&m)
	if err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	if has == false {
		return nil, errors.New("未查询到相应的 block_star_chain 记录")
	}
	return &m, nil
}

// SubsidyWithUserFindByParams 通过传入的参数查询数据(多条)
func SubsidyWithUserFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.SubsidyWithUser, error) {
	var m []model.SubsidyWithUser
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}

func SubsidyWithUserFindByParamsByPage(Db *xorm.Engine, params map[string]interface{}, page, pageSize int) (*[]model.SubsidyWithUser, error) {
	var m []model.SubsidyWithUser
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if page == 0 && pageSize == 0 {
		page = 1
		pageSize = 10
	}

	if params["key"] == nil {
		//查询全部数据
		err := Db.Limit(pageSize, (page-1)*pageSize).Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}
