package db

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"reflect"
	"xorm.io/xorm"
)

// BatchSelectUserVirtualCoinFlows 批量查询数据 TODO::和下面的方法重复了，建议采用下面的 `UserVirtualCoinFlowFindByParams` 方法
func BatchSelectUserVirtualCoinFlows(Db *xorm.Engine, params map[string]interface{}) (*[]model.UserVirtualCoinFlow, error) {
	var UserVirtualCoinFlowData []model.UserVirtualCoinFlow
	if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).
		Find(&UserVirtualCoinFlowData); err != nil {
		return nil, zhios_order_relate_logx.Warn(err)
	}
	return &UserVirtualCoinFlowData, nil
}

// UserVirtualCoinFlowInsert 插入单条数据
func UserVirtualCoinFlowInsert(session *xorm.Session, UserVirtualCoinFlow *model.UserVirtualCoinFlow) (int64, error) {
	_, err := session.InsertOne(UserVirtualCoinFlow)
	if err != nil {
		return 0, err
	}
	return UserVirtualCoinFlow.Id, nil
}

// BatchAddUserVirtualCoinFlows 批量新增数据
func BatchAddUserVirtualCoinFlows(Db *xorm.Engine, UserVirtualCoinFlowData []*model.UserVirtualCoinFlow) (int64, error) {
	affected, err := Db.Insert(UserVirtualCoinFlowData)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func GetUserVirtualCoinFlowCount(Db *xorm.Engine) int {
	var UserVirtualCoinFlow model.UserVirtualCoinFlow
	session := Db.Where("")
	count, err := session.Count(&UserVirtualCoinFlow)
	if err != nil {
		return 0
	}
	return int(count)
}

// UserVirtualCoinFlowDelete 删除记录
func UserVirtualCoinFlowDelete(Db *xorm.Engine, id interface{}) (int64, error) {
	if reflect.TypeOf(id).Kind() == reflect.Slice {
		return Db.In("id", id).Delete(model.UserVirtualCoinFlow{})
	} else {
		return Db.Where("id = ?", id).Delete(model.UserVirtualCoinFlow{})
	}
}

// UserVirtualCoinFlowUpdate 更新记录
func UserVirtualCoinFlowUpdate(session *xorm.Session, id interface{}, UserVirtualCoinFlow *model.UserVirtualCoinFlow, forceColums ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColums != nil {
		affected, err = session.Where("id=?", id).Cols(forceColums...).Update(UserVirtualCoinFlow)
	} else {
		affected, err = session.Where("id=?", id).Update(UserVirtualCoinFlow)
	}
	if err != nil {
		return 0, err
	}
	return affected, nil
}

// UserVirtualCoinFlowGetOneByParams 通过传入的参数查询数据(单条)
func UserVirtualCoinFlowGetOneByParams(Db *xorm.Engine, params map[string]interface{}) (*model.UserVirtualCoinFlow, error) {
	var m model.UserVirtualCoinFlow
	var query = fmt.Sprintf("%s =?", params["key"])
	if has, err := Db.Where(query, params["value"]).Get(&m); err != nil || has == false {
		return nil, zhios_order_relate_logx.Error(err)
	}
	return &m, nil
}

// UserVirtualCoinFlowFindByParams 通过传入的参数查询数据(多条)
func UserVirtualCoinFlowFindByParams(Db *xorm.Engine, params map[string]interface{}) (*[]model.UserVirtualCoinFlow, error) {
	var m []model.UserVirtualCoinFlow
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if params["key"] == nil {
		//查询全部数据
		err := Db.Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}

func UserVirtualCoinFlowFindByParamsByPage(Db *xorm.Engine, params map[string]interface{}, page, pageSize int) (*[]model.UserVirtualCoinFlow, error) {
	var m []model.UserVirtualCoinFlow
	if params["value"] == nil {
		return nil, errors.New("参数有误")
	}
	if page == 0 && pageSize == 0 {
		page = 1
		pageSize = 10
	}

	if params["key"] == nil {
		//查询全部数据
		err := Db.Limit(pageSize, (page-1)*pageSize).Find(&m)
		if err != nil {
			return nil, zhios_order_relate_logx.Error(err)
		}
		return &m, nil
	} else {
		if reflect.TypeOf(params["value"]).Kind() == reflect.Slice {
			//指定In查询
			if err := Db.In(zhios_order_relate_utils.AnyToString(params["key"]), params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m); err != nil {
				return nil, zhios_order_relate_logx.Warn(err)
			}
			return &m, nil
		} else {
			var query = fmt.Sprintf("%s =?", params["key"])
			err := Db.Where(query, params["value"]).Limit(pageSize, (page-1)*pageSize).Find(&m)
			if err != nil {
				return nil, zhios_order_relate_logx.Error(err)
			}
			return &m, nil
		}

	}
}
