package comm_plan

import "xorm.io/xorm"

var Fn = map[string]func(opt *PlanOpt, totalAmt, integralTotalAmt float64, userList *LvUser, pvd string, sysFee float64, integralSysFee float64, level, levelWeight int, eg *xorm.Engine) error{
	"lv_all":                       CalcAll,
	"niubei_commission":            NiuBeiCalcAll,
	"niubei_amount":                NiuBeiCalcAll,
	"lv_self":                      CalcSelf,
	"lv_subsidy":                   CalcAll,
	"lv_price":                     CalcAll,
	"lv_price_public_platoon":      CalcAll,
	"lv_commission_public_platoon": CalcAll,
	"lv_winery":                    CalcWinery,
	"lv_winery_adv":                CalcAdv,
	"lv_price_other":               CalcOther,
	"lv_ds_check":                  CalcDsCheck,
}

type NiuBeiIntegralReleaseO2oRatio struct {
	Ratio string `json:"ratio"`
	Key   string `json:"key"`
}
type PlanOpt struct {
	BuckleRate       float64
	PointType        int
	IntegralOpen     int
	PlanCommissionId int
	IsCanRunSubsidy  int    //用于后台判断要不要走补贴
	Pvd              string // 供应商
	Mode             string // 分佣方案
	CommissionMode   string // 佣金返佣方式
	//IntegralMode     string           // 积分返佣方式
	//BlockIconsMode   string           // 区块币返佣方式
	SysRate           float64 // 系统占佣比例
	PvdRate           float64 // 供应商占佣比例
	RegionRate        float64 // 区域代理占佣比例
	RegionSubRate     float64 // 区域代理占佣比例
	GlobalRate        float64 // 全球分红占佣比例
	SelfBuyGlobalRate float64
	MerchantRate      float64 //商家占佣比例
	NewAgentRate      float64 //代理分红占佣比例
	OrderBeforeRate   float64
	RateList          map[string]float64
	PushHandRate      float64 //推手占佣比例
	//IntegralBili     float64          // 积分兑换比例
	//BlockIconsBili   float64          // 区块币兑换比例
	UserRate                  map[int]*LvGrade // 供应商对应的等级比例
	VirtualCoinMoneyRatioList map[int]string   // 兑换现金比例列表
}

// 级差结构
type LvGrade struct {
	Lv                      int                    `json:"lv"`                          // 级别
	SubsidyMode             int                    `json:"subsidy_mode"`                // 补贴计算方式, 0按佣金计算,1按平台利润计算
	SubsidyEnable           int                    `json:"subsidy_enable"`              // 是否开启补贴计算方式 0关闭, 1开启
	TikTokTeamSubsidyEnable int                    `json:"tik_tok_team_subsidy_enable"` // 是否开启补贴计算方式 0关闭, 1开启
	SubsidySelfRate         float64                `json:"subsidy_self_rate"`           // 自购补贴比例
	SubsidyShareRate        float64                `json:"subsidy_share_rate"`          // 分享补贴比例
	PayMode                 int                    `json:"pay_mode"`                    // 0团队内部支出, 1平台系统支出
	SelfRate                float64                `json:"self_rate"`                   // 自购比例
	TeamRate                float64                `json:"team_rate"`                   // 团队分成比例
	PeerRate                []float64              `json:"peer_rate"`                   // 同级分成比例
	UserLvUpSubsidyEnable   int                    `json:"user_lv_up_subsidy_enable"`   //分销补贴开关
	UserLvUpSubsidyMode     int                    `json:"user_lv_up_subsidy_mode"`     //补贴方式 0比例 1固定金额
	UserSubsidyType         string                 `json:"user_subsidy_type"`           //  补贴模式  up_lv 推荐会员补贴  buy_goods 购买商品
	UserSubsidyBaseCoinId   string                 `json:"user_subsidy_base_coin_id"`   //  补贴计算基数
	UserLvUpSubsidyList     []interface{}          `json:"user_lv_up_subsidy_list"`     //会员费分销补贴相应方式的列表
	PeerRateList            []map[string]string    `json:"peer_rate_list"`              //同级比例
	NewPeerList             []map[string]string    `json:"new_peer_list"`               //同级比例
	BuyDeliverList          map[string]string      `json:"buy_deliver_list"`            //酒庄制度
	ReturnType              []string               `json:"return_type"`                 //返利类型
	SelfRateList            map[string]string      `json:"self_rate_list"`              // 自购比例
	TeamRateList            map[string]string      `json:"team_rate_list"`              // 团队最高比例
	NewTeamList             map[string]string      `json:"new_team_list"`               // 新团队比例
	NewExtendList           map[string]string      `json:"new_extend_list"`             // 直推比例
	SecondExtendList        interface{}            `json:"second_extend_list"`          // 直推比例
	ToTeamList              map[string]string      `json:"to_team_list"`                // 直推比例
	SameUserCount           string                 `json:"same_user_count"`
	SameExtend              []map[string]string    `json:"same_extend"`
	SubsidyModeList         map[string]interface{} `json:"subsidy_mode_list"` // 各币种返佣模式：bili：比例 money：固定金额
	//SubsidyBlockIconsMode string   `json:"subsidy_block_icons_mode"` //分销 区块币返利类型 bili 比例  money 固定金额
	//SubsidyCommissionMode string   `json:"subsidy_commission_mode"`  //分销 佣金返利类型 bili 比例  money 固定金额
	//SubsidyIntegralMode   string   `json:"subsidy_integral_mode"`    //分销 积分返利类型 bili 比例  money 固定金额
	SubsidyReturnType            []string            `json:"subsidy_return_type"` //分销 返利类型
	SubsidyOwnBiliList           []map[string]string `json:"subsidy_own_bili_list"`
	TikTokTeamSubsidyOwnBiliType []string            `json:"tik_tok_team_subsidy_own_bili_type"` //分销 返利类型
	TikTokTeamSubsidyList        []map[string]string `json:"tik_tok_team_subsidy_list"`
	ToExtendList                 map[string]string   `json:"to_extend_list"`       //
	CommonWealthSystem           interface{}         `json:"common_wealth_system"` //共富制度比例

}
type LvGradePricePublic struct {
	Lv                      int                    `json:"lv"`                          // 级别
	SubsidyMode             int                    `json:"subsidy_mode"`                // 补贴计算方式, 0按佣金计算,1按平台利润计算
	SubsidyEnable           int                    `json:"subsidy_enable"`              // 是否开启补贴计算方式 0关闭, 1开启
	TikTokTeamSubsidyEnable int                    `json:"tik_tok_team_subsidy_enable"` // 是否开启补贴计算方式 0关闭, 1开启
	SubsidySelfRate         float64                `json:"subsidy_self_rate"`           // 自购补贴比例
	SubsidyShareRate        float64                `json:"subsidy_share_rate"`          // 分享补贴比例
	PayMode                 int                    `json:"pay_mode"`                    // 0团队内部支出, 1平台系统支出
	SelfRate                float64                `json:"self_rate"`                   // 自购比例
	TeamRate                float64                `json:"team_rate"`                   // 团队分成比例
	PeerRate                []float64              `json:"peer_rate"`                   // 同级分成比例
	UserLvUpSubsidyEnable   int                    `json:"user_lv_up_subsidy_enable"`   //分销补贴开关
	UserLvUpSubsidyMode     int                    `json:"user_lv_up_subsidy_mode"`     //补贴方式 0比例 1固定金额
	UserSubsidyType         string                 `json:"user_subsidy_type"`           //  补贴模式  up_lv 推荐会员补贴  buy_goods 购买商品
	UserSubsidyBaseCoinId   string                 `json:"user_subsidy_base_coin_id"`   //  补贴计算基数
	UserLvUpSubsidyList     []interface{}          `json:"user_lv_up_subsidy_list"`     //会员费分销补贴相应方式的列表
	PeerRateList            []map[string]string    `json:"peer_rate_list"`              //同级比例
	NewPeerList             []map[string]string    `json:"new_peer_list"`               //同级比例
	BuyDeliverList          map[string]string      `json:"buy_deliver_list"`            //酒庄制度
	ReturnType              []string               `json:"return_type"`                 //返利类型
	SelfRateList            map[string]string      `json:"self_rate_list"`              // 自购比例
	TeamRateList            map[string]string      `json:"team_rate_list"`              // 团队最高比例
	NewTeamList             map[string]string      `json:"new_team_list"`               // 新团队比例
	NewExtendList           map[string]string      `json:"new_extend_list"`             // 直推比例
	SecondExtendList        map[string]string      `json:"second_extend_list"`          // 直推比例
	ToTeamList              map[string]string      `json:"to_team_list"`                // 直推比例
	SameUserCount           string                 `json:"same_user_count"`
	SameExtend              []map[string]string    `json:"same_extend"`
	SubsidyModeList         map[string]interface{} `json:"subsidy_mode_list"` // 各币种返佣模式：bili：比例 money：固定金额
	//SubsidyBlockIconsMode string   `json:"subsidy_block_icons_mode"` //分销 区块币返利类型 bili 比例  money 固定金额
	//SubsidyCommissionMode string   `json:"subsidy_commission_mode"`  //分销 佣金返利类型 bili 比例  money 固定金额
	//SubsidyIntegralMode   string   `json:"subsidy_integral_mode"`    //分销 积分返利类型 bili 比例  money 固定金额
	SubsidyReturnType            []string            `json:"subsidy_return_type"` //分销 返利类型
	SubsidyOwnBiliList           []map[string]string `json:"subsidy_own_bili_list"`
	TikTokTeamSubsidyOwnBiliType []string            `json:"tik_tok_team_subsidy_own_bili_type"` //分销 返利类型
	TikTokTeamSubsidyList        []map[string]string `json:"tik_tok_team_subsidy_list"`
	ToExtendList                 map[string]string   `json:"to_extend_list"`       //
	CommonWealthSystem           map[string]string   `json:"common_wealth_system"` //共富制度比例
}
type LvGradeWineryAdv struct {
	Lv           int                 `json:"lv"`             // 级别
	PayMode      int                 `json:"pay_mode"`       // 0团队内部支出, 1平台系统支出
	NewPeerList  []map[string]string `json:"new_peer_list"`  //同级比例
	ReturnType   []string            `json:"return_type"`    //返利类型
	ToExtendList map[string]string   `json:"to_extend_list"` //
	ToTeamList   map[string]string   `json:"to_team_list"`   // 直推比例
}
type LvGradeDs struct {
	Lv                           int                 `json:"lv"`                          // 级别
	SubsidyMode                  int                 `json:"subsidy_mode"`                // 补贴计算方式, 0按佣金计算,1按平台利润计算
	SubsidyEnable                int                 `json:"subsidy_enable"`              // 是否开启补贴计算方式 0关闭, 1开启
	TikTokTeamSubsidyEnable      int                 `json:"tik_tok_team_subsidy_enable"` // 是否开启补贴计算方式 0关闭, 1开启
	SubsidySelfRate              float64             `json:"subsidy_self_rate"`           // 自购补贴比例
	SubsidyShareRate             float64             `json:"subsidy_share_rate"`          // 分享补贴比例
	PayMode                      int                 `json:"pay_mode"`                    // 0团队内部支出, 1平台系统支出
	SelfRate                     float64             `json:"self_rate"`                   // 自购比例
	TeamRate                     float64             `json:"team_rate"`                   // 团队分成比例
	PeerRate                     []float64           `json:"peer_rate"`                   // 同级分成比例
	UserLvUpSubsidyEnable        int                 `json:"user_lv_up_subsidy_enable"`   //分销补贴开关
	UserLvUpSubsidyMode          int                 `json:"user_lv_up_subsidy_mode"`     //补贴方式 0比例 1固定金额
	UserSubsidyType              string              `json:"user_subsidy_type"`           //  补贴模式  up_lv 推荐会员补贴  buy_goods 购买商品
	UserSubsidyBaseCoinId        string              `json:"user_subsidy_base_coin_id"`   //  补贴计算基数
	UserLvUpSubsidyList          []interface{}       `json:"user_lv_up_subsidy_list"`     //会员费分销补贴相应方式的列表
	PeerRateList                 []map[string]string `json:"peer_rate_list"`              //同级比例
	BuyDeliverList               map[string]string   `json:"buy_deliver_list"`            //酒庄制度
	ReturnType                   []string            `json:"return_type"`                 //返利类型
	SelfRateList                 map[string]string   `json:"self_rate_list"`              // 自购比例
	TeamRateList                 map[string]string   `json:"team_rate_list"`              // 团队最高比例
	NewTeamList                  map[string]string   `json:"new_team_list"`               // 新团队比例
	NewExtendList                map[string]string   `json:"new_extend_list"`             // 直推比例
	SecondExtendList             map[string]string   `json:"second_extend_list"`          // 直推比例
	SameUserCount                string              `json:"same_user_count"`
	SameExtend                   []map[string]string `json:"same_extend"`
	TikTokTeamSubsidyOwnBiliType []string            `json:"tik_tok_team_subsidy_own_bili_type"` //分销 返利类型
	TikTokTeamSubsidyList        []map[string]string `json:"tik_tok_team_subsidy_list"`
}

type UserLvUpSubsidyList struct {
	Lv int `json:"lv"` // 等级
	//Bili       float64 `json:"bili"`        // 比例
	IsUse int `json:"is_use"` // 是否共存
	//BlockIcons float64 `json:"block_icons"` //区块币
	Commission float64 `json:"commission"` //佣金  （保留兼容旧版，对应在RateList里面的key为‘0’的值）
	//Integral   float64 `json:"integral"`    //积分
	RateList map[string]interface{} // 各币种分佣设置
}
type SettleCommissionToGuide struct {
	Profit     float64 `json:"profit"`
	PvdFee     float64 `json:"pvd_fee"`
	SysFee     float64 `json:"sys_fee"`
	SubsidyFee float64 `json:"subsidy_fee"`
	LvUser     *LvUser `json:"lv_user"`
}
type SettleCommissionToAdminRes struct {
	DistriCommission  string                       `json:"distriCommission"`
	PaymentCommission string                       `json:"paymentCommission"`
	PvdCommission     float64                      `json:"pvdCommission"`
	SubsidyCommission float64                      `json:"subsidyCommission"`
	Relation          []*SettleCommissionToAdminLv `json:"relation"`
	RemainCommission  string                       `json:"remainCommission"`
	SysCommission     float64                      `json:"sysCommission"`
	TotalCommission   string                       `json:"totalCommission"`
}
type SettleCommissionToAdminLv struct {
	AmountCommission  string `json:"amountCommission"`
	Commission        string `json:"commission"`
	CommissionRate    string `json:"commissionRate"`
	GenKey            string `json:"genKey"`
	IsBuyer           string `json:"isBuyer"`
	LvID              string `json:"lvId"`
	LvName            string `json:"lvName"`
	Name              string `json:"name"`
	SubsidyCommission string `json:"subsidyCommission"`
	SubsidyRate       string `json:"subsidyRate"`
	TierName          string `json:"tierName"`
	Weight            string `json:"weight"`
}
type SettleCommissionToAdmin struct {
	Commission   string                           `json:"commission"`
	ID           int64                            `json:"id"`
	RelationList []SettleCommissionToRelationList `json:"relationList"`
}
type SettleCommissionToRelationList struct {
	GenKey  string `json:"genKey"`
	IsBuyer string `json:"isBuyer"`
	LvID    string `json:"lvId"`
	Name    string `json:"name"`
}
type LvUser struct {
	FirstFee                float64
	IntegralFirstFee        float64
	ProfitBili              float64
	Uid                     int     // 用户ID
	Lv                      int     // 等级
	NewLv                   int     // 升级后等级 针对会员费分佣
	LevelWeight             int     // 权重
	Profit                  float64 // 利润(金额)
	SubsidyFee              float64 // 补贴（金额）
	AdditionalSubsidy       float64
	MinLevel                int
	IsFreeze                int
	MinProfit               float64                  // 利润(金额)
	MinProfitList           []*VirtualCoinCommission // 各币种对用的数量
	ProfitList              []*VirtualCoinCommission // 各币种对用的数量
	SubsidyFeeList          []*VirtualCoinCommission // 各币种对应的补贴数量
	OwnSubsidyFeeList       map[string]float64       // 各币种对应的补贴数量
	TikTokOwnSubsidyFeeList map[string]float64       // 各币种对应的补贴数量
	OwnbuyReturnType        int                      //0有返利 1没有返利
	Diff                    int                      // 与当前用户级别差
	OldDiff                 int                      // 旧的与当前用户级别差
	ParentUser              *LvUser                  // 父用户
	LastProfit              float64
	LastProfitList          []*VirtualCoinCommission // 各币种对用的数量
	ExtendType              int
}

// 虚拟币分佣结构体
type VirtualCoinCommission struct {
	Cid        string  `json:"cid"` // 虚拟币id
	Val        float64 `json:"val"` // 数量
	Bili       float64 `json:"bili"`
	ProfitBili float64 `json:"profit_bili"`
}

type VirtualCoinRateList struct {
	Cid string `json:"cid"` // 虚拟币id
	Val string `json:"val"` // 现金兑换比例
}

// 初始化级差模式
func Init(g map[int]*LvGrade) map[int]*LvGrade {
	var MinLv int = -1 // 最低级别
	for k, v := range g {
		if MinLv == -1 {
			MinLv = v.Lv
			g[k].TeamRate = v.SelfRate
		} else if v.Lv < MinLv {
			MinLv = v.Lv
			g[k].TeamRate = v.SelfRate
		}
	}
	return g
}
