package rule

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/enum"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"strconv"
	"time"
	"xorm.io/xorm"
)

func InitForGreen(redisAddr string) (err error) {
	if redisAddr != "" {
		cache.NewRedis(redisAddr)
	}
	_, err = cache.SelectDb(md.RedisDataBase)
	return
}

const PessimismLockKeyForGreen = "daily_settlement_block_green_chain_pessimism_lock_key"
const PessimismLockValueForGreen = "running"

// DailySettlementBlockGreenChain 每日结算“绿色积分”
func DailySettlementBlockGreenChain(engine *xorm.Engine, mid string, isTask bool) (err error) {
	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()
	now := time.Now()
	today := now.Format("2006-01-02")

	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", now.Hour())
	if isTask && (now.Hour() > 8) {
		//TODO::只在凌晨一点 ~ 凌晨 8 点运行
		return errors.New("非运行时间")
	}

	//1、查找 `block_green_chain` 基础设置
	blockGreenChain, err := db.BlockGreenChainGetOneByParams(session, map[string]interface{}{
		"key":   "is_use",
		"value": 1,
	})
	if err != nil {
		_ = session.Rollback()
		return err
	}
	if blockGreenChain.SettlementDate == today {
		_ = session.Rollback()
		return errors.New("今日“绿色积分”已结算")
	}

	isHasBlockGreenChainSettlementRecords, err := db.BlockGreenChainSettlementRecordsGetOneByParams(session, map[string]interface{}{
		"key":   "date",
		"value": today,
	})
	if isHasBlockGreenChainSettlementRecords != nil {
		_ = session.Rollback()
		return errors.New("今日绿色积分已结算！！！")
	}

	//TODO::增加“悲观锁”防止串行
	getString, _ := cache.GetString(PessimismLockKeyForGreen)
	//if err != nil {
	//	return err
	//}
	if getString == PessimismLockValueForGreen {
		fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", "上一次结算未执行完")
		return errors.New("上一次结算未执行完")
	}
	cache.SetEx(PessimismLockKeyForGreen, PessimismLockValueForGreen, 3600*8) //8小时

	//TODO::关闭系统
	zhios_order_relate_utils.CurlPost("http://zhios-app:5000/api/v1/check_app_over_time", map[string]string{
		"mid":      mid,
		"is_close": "1",
	}, map[string]string{})

	todayPublishCoin := zhios_order_relate_utils.StrToFloat64(blockGreenChain.TodayPublishCoin)             //今日区块币发行数量(若为0,则按当前每日区块币发行数量)
	nowEverydayPublishCoin := zhios_order_relate_utils.StrToFloat64(blockGreenChain.NowEverydayPublishCoin) //当前每日区块币发行数量

	//1、判断今日是否有系统销毁
	var destroyCoinForSystem string
	if todayPublishCoin != 0 && todayPublishCoin != nowEverydayPublishCoin {
		destroyCoinForSystemValue, _ := decimal.NewFromFloat(nowEverydayPublishCoin).Sub(decimal.NewFromFloat(todayPublishCoin)).Float64()
		if destroyCoinForSystemValue < 0 {
			_ = session.Rollback()
			return errors.New("当前\"今日区块币发行数量\"大于\"当前每日区块币发行数量\"")
		}
		err := DealDestroyCoinForGreen(session, int(enum.SystemDestroyForGreen), destroyCoinForSystemValue, enum.SystemDestroyForGreen.String(), blockGreenChain)
		if err != nil {
			_ = session.Rollback()
			return err
		}
		destroyCoinForSystem = zhios_order_relate_utils.Float64ToStrPrec10(destroyCoinForSystemValue)
		nowEverydayPublishCoin = todayPublishCoin
	}
	//2、计算出"消费返利"、"平台空投" 区各需发放多少枚区块币
	var publishCoin = nowEverydayPublishCoin //实际空投虚拟数量

	publishCoinConsumeRate, _ := decimal.NewFromString(blockGreenChain.PublishCoinConsumeRate) //区块币发行消费占比
	publishCoinConsumeRate = publishCoinConsumeRate.Div(decimal.NewFromFloat(100))
	consumeAreaCoinNums := zhios_order_relate_utils.StrToFloat64(decimal.NewFromFloat(publishCoin).Mul(publishCoinConsumeRate).String()) //消费区区块币发行数量

	publishCoinAirdropRate, _ := decimal.NewFromString(blockGreenChain.PublishCoinAirdropRate) //区块币发行空投占比
	publishCoinAirdropRate = publishCoinAirdropRate.Div(decimal.NewFromFloat(100))
	airdropAreaCoinNums := zhios_order_relate_utils.StrToFloat64(decimal.NewFromFloat(publishCoin).Mul(publishCoinAirdropRate).String()) //空投区区块币发行数量

	//3、进行"消费返利"、"平台空投" 区区块币发放
	err = dealIssueCoinForGreen(session, int(enum.ConsumeIssue), consumeAreaCoinNums, enum.ConsumeIssue.String(), blockGreenChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	err = dealIssueCoinForGreen(session, int(enum.AirdropIssue), airdropAreaCoinNums, enum.AirdropIssue.String(), blockGreenChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//userIds, err := db.UserLevelDescByWeightLow(session)
	//if err != nil {
	//	_ = session.Rollback()
	//	return err
	//}
	var userIds []int
	//4、进行消费返利-区块币统计分配
	err, destroyCoinForConsume := statisticsAndDistributeCoinForConsume(session, userIds, mid, consumeAreaCoinNums, blockGreenChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//5、进行空投区-区块币统计分配
	err, destroyCoinForAirdrop := statisticsAndDistributeCoinForAirdrop(session, mid, airdropAreaCoinNums, blockGreenChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//6、插入 block_star_chain_settlement_records 记录
	var blockGreenChainSettlementRecords = model.BlockGreenChainSettlementRecords{
		CoinId:                     blockGreenChain.Coin1,
		TodayPublishCoin:           zhios_order_relate_utils.Float64ToStr(publishCoin),
		TodayDestroyCoinForSystem:  destroyCoinForSystem,
		TodayPublishCoinForConsume: zhios_order_relate_utils.Float64ToStr(consumeAreaCoinNums),
		TodayDestroyCoinForConsume: zhios_order_relate_utils.Float64ToStr(destroyCoinForConsume),
		TodayPublishCoinForAirdrop: zhios_order_relate_utils.Float64ToStr(airdropAreaCoinNums),
		TodayDestroyCoinForAirdrop: zhios_order_relate_utils.Float64ToStr(destroyCoinForAirdrop),
		Date:                       now.Format("2006-01-02"),
		CreateAt:                   now,
		UpdateAt:                   now,
	}
	_, err = db.BlockGreenChainSettlementRecordsInsert(session, &blockGreenChainSettlementRecords)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//7、计算当前每日区块币应发行数量
	err = calcNowEverydayPublishCoinForGreen(session, blockGreenChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	//8、更新 block_green_chain 中的 settlement_date 字段
	blockGreenChain.SettlementDate = today
	//updateAffected, err := db.BlockGreenChainUpdate(session, blockGreenChain.Id, blockGreenChain, "settlement_date")
	updateAffected, err := db.BlockGreenChainUpdate(session, blockGreenChain.Id, blockGreenChain)
	if err != nil {
		_ = session.Rollback()
		return err
	}
	if updateAffected == 0 {
		_ = session.Rollback()
		return errors.New("更新 block_green_chain 的 settlement_date 记录失败")
	}

	//9、将昨日的“平台区块币指导价”插入到 block_coin_platform_guide_price_for_coin_records
	var blockGreenCoinPlatformGuidePriceForCoinRecords = model.BlockGreenCoinPlatformGuidePriceForCoinRecords{
		CoinId:                    blockGreenChain.Coin1,
		PlatformGuidePriceForCoin: blockGreenChain.PlatformGuidePriceForCoin,
		Date:                      now.AddDate(0, 0, -1).Format("2006-01-02"),
		CreateAt:                  now,
		UpdateAt:                  now,
	}
	_, err = db.BlockGreenCoinPlatformGuidePriceForCoinRecordsInsert(session, &blockGreenCoinPlatformGuidePriceForCoinRecords)
	if err != nil {
		_ = session.Rollback()
		return err
	}

	err = session.Commit()
	if err != nil {
		_ = session.Rollback()
		return errors.New("事务提交失败")
	}

	zhios_order_relate_utils.CurlPost("http://zhios-app:5000/api/v1/check_app_over_time", map[string]string{
		"mid":      mid,
		"is_close": "0",
	}, map[string]string{})

	//cache.Del(PessimismLockKeyForGreen)

	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>绿色积分结束<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")
	return nil
}

/*
   统计分配区块币-消费区
   TODO:: 公式【 个人贡献值/全网贡献值x2100枚=每天获取的绿色积分 】
*/
func statisticsAndDistributeCoinForConsume(session *xorm.Session, userIds []int, mid string, publishCoin float64, chain *model.BlockGreenChain) (err error, unassignedTotalCoinValue float64) {
	publishCoinValue := decimal.NewFromFloat(publishCoin) //消费区发行区块币数量
	//platformGuidePriceForCoinValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(chain.PlatformGuidePriceForCoin)) //今日平台区块币指导价
	var unassignedTotalCoin = decimal.NewFromFloat(0) //分配的区块币
	var userVirtualAmount model.UserVirtualAmount
	var userVirtualAmounts []model.UserVirtualAmount
	//1、统计出消费区的总贡献值
	sumStatic, err := session.Table("user_virtual_amount").Where("coin_id =?", chain.Coin2).Sum(&userVirtualAmount, "amount")
	if err != nil {
		return err, unassignedTotalCoinValue
	}
	sumStaticValue := decimal.NewFromFloat(sumStatic)

	//2、查询出所有拥有贡献值的用户
	err = session.Table("user_virtual_amount").NotIn("uid", userIds).Where("coin_id =?", chain.Coin2).And("amount > 0").Find(&userVirtualAmounts)
	if err != nil {
		return err, unassignedTotalCoinValue
	}

	for _, item := range userVirtualAmounts {
		amount := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(item.Amount)) //用户贡献值余额
		getCoin := amount.Div(sumStaticValue).Mul(publishCoinValue)                        //得到的区块币
		//needDestroyContribution := getCoin.Mul(platformGuidePriceForCoinValue)             //需销毁贡献值
		needDestroyContribution := amount //需销毁贡献值

		coinAmount, err := svc.GetUserCoinAmount(session, mid, chain.Coin2, item.Uid) //获取此时用户的"贡献值"
		if err != nil {
			return err, unassignedTotalCoinValue
		}
		coinAmountValue, _ := decimal.NewFromString(coinAmount)

		//3.1判断静态贡献值是否足够
		if needDestroyContribution.GreaterThan(coinAmountValue) {
			////TODO::公式【得到的区块币 = ((需销毁贡献值 - 用户贡献值余额) / 今日平台区块币指导价)】
			//tempCoin := (needDestroyContribution.Sub(coinAmountValue)).Div(platformGuidePriceForCoinValue)
			//getCoin = getCoin.Sub(tempCoin)
			needDestroyContribution = coinAmountValue
		}
		unassignedTotalCoin = unassignedTotalCoin.Add(getCoin)

		//3.2给相应用户加上分配到的虚拟币
		err = DealUserCoinForGreen(session, md.DealUserCoinReq{
			Kind:         "add",
			Mid:          mid,
			Title:        md.BlockGreenChainConsumeAreaDistributionTitleForUserVirtualCoinFlow,
			TransferType: md.BlockGreenChainConsumeAreaDistributionTransferTypeForUserVirtualCoinFlow,
			OrdId:        "",
			CoinId:       chain.Coin1,
			Uid:          item.Uid,
			Amount:       zhios_order_relate_utils.StrToFloat64(getCoin.String()),
		})
		if err != nil {
			return err, unassignedTotalCoinValue
		}

		//3.3给相应用户扣除 "消费区发放(销毁贡献值)"
		err = DealUserCoinForGreen(session, md.DealUserCoinReq{
			Kind:         "sub",
			Mid:          mid,
			Title:        md.BlockGreenChainConsumeAreaIssueAndDestroyTitleForUserVirtualCoinFlow,
			TransferType: md.BlockGreenChainConsumeAreaIssueAndDestroyTransferTypeForUserVirtualCoinFlow,
			OrdId:        "",
			CoinId:       chain.Coin2,
			Uid:          item.Uid,
			Amount:       zhios_order_relate_utils.StrToFloat64(needDestroyContribution.String()),
		})
		if err != nil {
			return err, unassignedTotalCoinValue
		}
	}

	//4、处理未分配完的区块币-消费区
	if publishCoinValue.GreaterThan(unassignedTotalCoin) {
		unassignedTotalCoinValue, _ = publishCoinValue.Sub(unassignedTotalCoin).Float64()
		err := DealDestroyCoinForGreen(session, int(enum.ConsumeUnallocatedAndDestroy), unassignedTotalCoinValue, enum.ConsumeUnallocatedAndDestroy.String(), chain)
		if err != nil {
			return err, unassignedTotalCoinValue
		}
	}
	return nil, unassignedTotalCoinValue
}

/*
   统计分配区块币-空投区
*/
func statisticsAndDistributeCoinForAirdrop(session *xorm.Session, mid string, publishCoin float64, chain *model.BlockGreenChain) (err error, unassignedTotalCoinValue float64) {
	publishCoinValue := decimal.NewFromFloat(publishCoin) //空投区-发行区块币数量
	var totalCoin = decimal.NewFromFloat(0)

	//1、查询出所有需要空投的用户
	var userList []*model.BlockGreenChainCustomUserAirdrop
	totalUser, err := session.Table("block_green_chain_custom_user_airdrop").Where("airdrop_frequency > 0").FindAndCount(&userList)
	if err != nil {
		return err, unassignedTotalCoinValue
	}
	if totalUser > 0 {
		now := time.Now()
		tday := now.Format("2006-01-02")
		for _, item := range userList {
			var singleValue = decimal.NewFromFloat(0)

			if item.AirdropDate != "0000-00-00" {
				startAt, err := time.ParseInLocation("2006-01-02", chain.StartAt, time.Local) //起始时间
				if err != nil {
					return err, unassignedTotalCoinValue
				}
				diffDays := zhios_order_relate_utils.GetDiffDays(now, startAt) //相差天数
				remainder := diffDays % item.AirdropFrequency
				if remainder == 0 {
					//需要空投
					singleValue, _ = decimal.NewFromString(item.AirdropNums)
				}
			} else {
				singleValue, _ = decimal.NewFromString(item.AirdropNums)
			}

			if singleValue.GreaterThan(decimal.NewFromFloat(0)) {
				totalCoin = totalCoin.Add(singleValue)
				err = DealUserCoinForGreen(session, md.DealUserCoinReq{
					Kind:         "add",
					Mid:          mid,
					Title:        md.BlockGreenChainAirdropAreaDistributionTitleForUserVirtualCoinFlow,
					TransferType: md.BlockGreenChainAirdropAreaDistributionTransferTypeForUserVirtualCoinFlow,
					OrdId:        "",
					CoinId:       chain.Coin1,
					Uid:          item.Uid,
					Amount:       zhios_order_relate_utils.StrToFloat64(singleValue.String()),
				})
				if err != nil {
					return err, unassignedTotalCoinValue
				}
				//TODO::更新 airdrop_date(空投日期)
				item.AirdropDate = tday
				_, err := db.BlockGreenChainCustomUserAirdropUpdate(session, item.Id, item)
				if err != nil {
					return err, unassignedTotalCoinValue
				}
			}
		}
	}

	//处理未分配完的区块币-空投区
	unassignedTotalCoinValue, _ = publishCoinValue.Sub(totalCoin).Float64()
	if unassignedTotalCoinValue > 0 {
		err := DealDestroyCoinForGreen(session, int(enum.AirdropUnallocatedAndDestroy), unassignedTotalCoinValue, enum.AirdropUnallocatedAndDestroy.String(), chain)
		if err != nil {
			return err, unassignedTotalCoinValue
		}
	}
	return nil, unassignedTotalCoinValue
}

//计算当前每日区块币应发行数量
func calcNowEverydayPublishCoinForGreen(session *xorm.Session, chain *model.BlockGreenChain) error {
	now := time.Now()
	startAt, err := time.ParseInLocation("2006-01-02", chain.StartAt, time.Local) //起始时间
	if err != nil {
		return err
	}
	diffDays := zhios_order_relate_utils.GetDiffDays(now, startAt) //相差天数
	remainder := diffDays % 30
	if remainder == 0 {
		everyThirtyDaysReduceRate := zhios_order_relate_utils.StrToFloat64(chain.EveryThirtyDaysReduceRate) / 100 //每三十日递减发行百分比
		nowEverydayPublishCoin := zhios_order_relate_utils.StrToFloat64(chain.NowEverydayPublishCoin)             //当前每日区块币发行数量
		chain.NowEverydayPublishCoin = zhios_order_relate_utils.Float64ToStrPrec10(nowEverydayPublishCoin * (1 - everyThirtyDaysReduceRate))
		//updateAffected, err := db.BlockGreenChainUpdate(session, chain.Id, chain, "now_everyday_publish_coin")
		//if err != nil {
		//	return err
		//}
		//if updateAffected == 0 {
		//	err = errors.New("更新 block_green_chain 的 now_everyday_publish_coin 记录失败")
		//	return err
		//}
	}
	return nil
}

//处理虚拟币 - 发放
func dealIssueCoinForGreen(session *xorm.Session, kind int, amount float64, title string, chain *model.BlockGreenChain) error {
	amountValue := decimal.NewFromFloat(amount)
	now := time.Now()
	var blockGreenChainFlow model.BlockGreenChainFlow
	blockGreenChainFlow.CoinId = chain.Coin1
	blockGreenChainFlow.Direction = md.FlowDirectionIncome
	blockGreenChainFlow.Kind = kind
	blockGreenChainFlow.Title = title
	blockGreenChainFlow.CreateTime = now
	blockGreenChainFlow.Amount = amountValue.RoundFloor(4).String()
	switch kind {
	case int(enum.ConsumeIssue):
		blockGreenChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		beforeTotalNowCoin, _ := decimal.NewFromString(chain.TotalNowCoin)
		blockGreenChainFlow.AfterTotalNowCoin = beforeTotalNowCoin.Add(amountValue).RoundFloor(4).String()

		blockGreenChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		beforeTotalRemainderCoin, _ := decimal.NewFromString(chain.TotalRemainderCoin)
		blockGreenChainFlow.AfterTotalRemainderCoin = beforeTotalRemainderCoin.Sub(amountValue).RoundFloor(4).String()

		blockGreenChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		beforeTotalPublishCoin, _ := decimal.NewFromString(chain.TotalPublishCoin)
		blockGreenChainFlow.AfterTotalPublishCoin = beforeTotalPublishCoin.Add(amountValue).RoundFloor(4).String()
		blockGreenChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockGreenChainFlow.AfterTotalDestroyCoin = chain.TotalDestroyCoin
		break
	case int(enum.AirdropIssue):
		blockGreenChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		beforeTotalNowCoin, _ := decimal.NewFromString(chain.TotalNowCoin)
		blockGreenChainFlow.AfterTotalNowCoin = beforeTotalNowCoin.Add(amountValue).RoundFloor(4).String()

		blockGreenChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		beforeTotalRemainderCoin, _ := decimal.NewFromString(chain.TotalRemainderCoin)
		blockGreenChainFlow.AfterTotalRemainderCoin = beforeTotalRemainderCoin.Sub(amountValue).RoundFloor(4).String()

		blockGreenChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		beforeTotalPublishCoin, _ := decimal.NewFromString(chain.TotalPublishCoin)
		blockGreenChainFlow.AfterTotalPublishCoin = beforeTotalPublishCoin.Add(amountValue).RoundFloor(4).String()
		blockGreenChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		blockGreenChainFlow.AfterTotalDestroyCoin = chain.TotalDestroyCoin
		break
	}
	chain.TotalNowCoin = blockGreenChainFlow.AfterTotalNowCoin
	chain.TotalRemainderCoin = blockGreenChainFlow.AfterTotalRemainderCoin
	chain.TotalPublishCoin = blockGreenChainFlow.AfterTotalPublishCoin
	chain.TotalDestroyCoin = blockGreenChainFlow.AfterTotalDestroyCoin

	//更新 `block_green_chain` 表
	updateAffected, err := db.BlockGreenChainUpdate(session, chain.Id, chain)
	if err != nil {
		return err
	}
	if updateAffected == 0 {
		err = errors.New("更新 block_green_chain 记录失败")
		return err
	}

	//插入 `block_star_chain_flow` 记录
	_, err = db.BlockGreenChainFlowInsert(session, &blockGreenChainFlow)
	if err != nil {
		return err
	}
	return nil
}

//DealDestroyCoinForGreen 处理虚拟币 - 销毁
func DealDestroyCoinForGreen(session *xorm.Session, kind int, amount float64, title string, chain *model.BlockGreenChain) error {
	amountValue := decimal.NewFromFloat(amount)
	now := time.Now()
	var blockGreenChainFlow model.BlockGreenChainFlow
	blockGreenChainFlow.CoinId = chain.Coin1
	blockGreenChainFlow.Direction = md.FlowDirectionExpenditure
	blockGreenChainFlow.Kind = kind
	blockGreenChainFlow.Title = title
	blockGreenChainFlow.Amount = amountValue.RoundFloor(4).String()
	blockGreenChainFlow.CreateTime = now
	switch kind {
	case int(enum.ConsumeUnallocatedAndDestroy):
		blockGreenChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		blockGreenChainFlow.AfterTotalNowCoin = chain.TotalNowCoin
		blockGreenChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockGreenChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockGreenChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockGreenChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockGreenChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		beforeTotalDestroyCoin, _ := decimal.NewFromString(chain.TotalDestroyCoin)
		blockGreenChainFlow.AfterTotalDestroyCoin = beforeTotalDestroyCoin.Add(amountValue).RoundFloor(4).String()
		break
	case int(enum.AirdropUnallocatedAndDestroy):
		blockGreenChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		beforeTotalNowCoin, _ := decimal.NewFromString(chain.TotalNowCoin)
		blockGreenChainFlow.AfterTotalNowCoin = beforeTotalNowCoin.Sub(amountValue).RoundFloor(4).String()
		blockGreenChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockGreenChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockGreenChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockGreenChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockGreenChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		beforeTotalDestroyCoin, _ := decimal.NewFromString(chain.TotalDestroyCoin)
		blockGreenChainFlow.AfterTotalDestroyCoin = beforeTotalDestroyCoin.Add(amountValue).RoundFloor(4).String()
		break
	case int(enum.TransactionDestroy):
		blockGreenChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		beforeTotalNowCoin, _ := decimal.NewFromString(chain.TotalNowCoin)
		blockGreenChainFlow.AfterTotalNowCoin = beforeTotalNowCoin.Sub(amountValue).RoundFloor(4).String()
		blockGreenChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockGreenChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockGreenChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockGreenChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockGreenChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		beforeTotalDestroyCoin, _ := decimal.NewFromString(chain.TotalDestroyCoin)
		blockGreenChainFlow.AfterTotalDestroyCoin = beforeTotalDestroyCoin.Add(amountValue).RoundFloor(4).String()
		break
	case int(enum.AdminDestroy):
		blockGreenChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		beforeTotalNowCoin, _ := decimal.NewFromString(chain.TotalNowCoin)
		blockGreenChainFlow.AfterTotalNowCoin = beforeTotalNowCoin.Sub(amountValue).RoundFloor(4).String()
		blockGreenChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockGreenChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockGreenChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockGreenChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockGreenChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		beforeTotalDestroyCoin, _ := decimal.NewFromString(chain.TotalDestroyCoin)
		blockGreenChainFlow.AfterTotalDestroyCoin = beforeTotalDestroyCoin.Add(amountValue).RoundFloor(4).String()
		break
	case int(enum.SystemDestroyForGreen):
		blockGreenChainFlow.BeforeTotalNowCoin = chain.TotalNowCoin
		beforeTotalNowCoin, _ := decimal.NewFromString(chain.TotalNowCoin)
		blockGreenChainFlow.AfterTotalNowCoin = beforeTotalNowCoin.Sub(amountValue).RoundFloor(4).String()
		blockGreenChainFlow.BeforeTotalRemainderCoin = chain.TotalRemainderCoin
		blockGreenChainFlow.AfterTotalRemainderCoin = chain.TotalRemainderCoin
		blockGreenChainFlow.BeforeTotalPublishCoin = chain.TotalPublishCoin
		blockGreenChainFlow.AfterTotalPublishCoin = chain.TotalPublishCoin
		blockGreenChainFlow.BeforeTotalDestroyCoin = chain.TotalDestroyCoin
		beforeTotalDestroyCoin, _ := decimal.NewFromString(chain.TotalDestroyCoin)
		blockGreenChainFlow.AfterTotalDestroyCoin = beforeTotalDestroyCoin.Add(amountValue).RoundFloor(4).String()
		break
	}

	chain.TotalNowCoin = blockGreenChainFlow.AfterTotalNowCoin
	chain.TotalDestroyCoin = blockGreenChainFlow.AfterTotalDestroyCoin
	//更新 `block_green_chain` 表
	_, err := db.BlockGreenChainUpdate(session, chain.Id, chain)
	if err != nil {
		return err
	}

	//插入 `block_star_chain_flow` 记录
	_, err = db.BlockGreenChainFlowInsert(session, &blockGreenChainFlow)
	if err != nil {
		return err
	}
	return nil
}

// DealUserCoinForGreen 处理给用户虚拟币积分
func DealUserCoinForGreen(session *xorm.Session, req md.DealUserCoinReq) (err error) {
	if req.Amount < 0 {
		req.Amount = 0
	}
	//1、分布式锁阻拦
	requestIdPrefix := fmt.Sprintf(md.DealUserCoinForGreenRequestIdPrefix, req.Mid, req.CoinId, req.Uid)
	cb, err := svc.HandleDistributedLock(req.Mid, strconv.Itoa(req.Uid), requestIdPrefix)
	if err != nil {
		return err
	}
	if cb != nil {
		defer cb() // 释放锁
	}

	//2、计算&&组装数据
	now := time.Now()
	coinAmount, err := svc.GetUserCoinAmount(session, req.Mid, req.CoinId, req.Uid)
	if err != nil {
		return err
	}
	coinAmountValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(coinAmount))
	amountValue := decimal.NewFromFloat(req.Amount).RoundFloor(4)

	var userVirtualCoinFlow model.UserVirtualCoinFlow
	userVirtualCoinFlow.CoinId = req.CoinId
	userVirtualCoinFlow.Title = req.Title
	userVirtualCoinFlow.TransferType = req.TransferType
	userVirtualCoinFlow.Uid = req.Uid
	userVirtualCoinFlow.ToUid = req.ToUid
	userVirtualCoinFlow.OrdId = req.OrdId
	userVirtualCoinFlow.BeforeAmout = coinAmount
	userVirtualCoinFlow.Amout = amountValue.String()
	userVirtualCoinFlow.CreateTime = now

	if req.Kind == "add" {
		userVirtualCoinFlow.Direction = 1
		userVirtualCoinFlow.AfterAmout = coinAmountValue.Add(amountValue).RoundFloor(4).String()
	} else if req.Kind == "sub" {
		userVirtualCoinFlow.Direction = 2
		userVirtualCoinFlow.AfterAmout = coinAmountValue.Sub(amountValue).RoundFloor(4).String()
	} else {
		err = errors.New("错误的kind类型")
		return err
	}
	if zhios_order_relate_utils.StrToFloat64(userVirtualCoinFlow.AfterAmout) < 0 {
		var coin model.VirtualCoin
		_, err = session.Where("id = ?", req.CoinId).Get(&coin)
		if err != nil {
			return err
		}
		zhios_order_relate_utils.FilePutContents("virtual_coin_not", zhios_order_relate_utils.SerializeStr(map[string]interface{}{
			"uid":           userVirtualCoinFlow.Uid,
			"amount":        userVirtualCoinFlow.Amout,
			"before_amount": userVirtualCoinFlow.BeforeAmout,
			"after_amount":  userVirtualCoinFlow.AfterAmout,
			"coin_id":       userVirtualCoinFlow.CoinId,
		}))
		return errors.New("用户" + zhios_order_relate_utils.IntToStr(userVirtualCoinFlow.Uid) + "的" + coin.Name + "不足")
		//userVirtualCoinFlow.AfterAmout = "0"
	}
	//3、插入 `user_virtual_coin_flow` 记录
	_, err = db.UserVirtualCoinFlowInsert(session, &userVirtualCoinFlow)
	if err != nil {
		return err
	}

	//4、修改 `user_virtual_amount`的amount值 && 及缓存
	err = svc.SetCacheUserVirtualAmount(session, req.Mid, userVirtualCoinFlow.AfterAmout, req.CoinId, req.Uid, true)
	if err != nil {
		return err
	}

	return nil
}
