package rule

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/enum"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"strconv"
	"time"
	"xorm.io/xorm"
)

func InitForIntegralRelease(redisAddr string) (err error) {
	if redisAddr != "" {
		cache.NewRedis(redisAddr)
	}
	_, err = cache.SelectDb(md.RedisDataBase)
	return
}

const PessimismLockKeyForIntegralRelease = "daily_settlement_integral_release_pessimism_lock_key"
const PessimismLockKeyForIntegralReleaseForInterposition = "daily_settlement_integral_release_for_interposition_pessimism_lock_key"
const PessimismLockValueForIntegralRelease = "running"
const DealUserCoinForIntegralReleaseRequestIdPrefix = "%s:integral_release_deal_user_coin:%d:uid:%d"

// DailySettlementIntegralRelease 每日结算“积分释放”
func DailySettlementIntegralRelease(engine *xorm.Engine, mid string, isTask bool) (err error) {
	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()
	now := time.Now()
	today := now.Format("2006-01-02")

	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", now.Hour())
	if isTask && (now.Hour() > 8) {
		//TODO::只在凌晨一点 ~ 凌晨 8 点运行
		return errors.New("非运行时间")
	}

	//1、查找 `integral_release` 基础设置
	integralRelease, err := db.IntegralReleaseGetOneByParams(session, map[string]interface{}{
		"key":   "is_use",
		"value": 1,
	})
	if err != nil {
		_ = session.Rollback()
		return err
	}
	if integralRelease.ReleaseDate == today {
		_ = session.Rollback()
		return errors.New("今日“积分释放”已结算")
	}
	if isTask {
		//TODO::增加“悲观锁”防止串行
		getString, _ := cache.GetString(PessimismLockKeyForIntegralRelease)
		if getString == PessimismLockValueForIntegralRelease {
			fmt.Println("-----------------", "上一次结算未执行完")
			return errors.New("上一次结算未执行完")
		}
		cache.SetEx(PessimismLockKeyForIntegralRelease, PessimismLockValueForIntegralRelease, 3600*8) //8小时
	}

	//TODO::关闭系统
	zhios_order_relate_utils.CurlPost("http://zhios-app:5000/api/v1/check_app_over_time", map[string]string{
		"mid":      mid,
		"is_close": "1",
	}, map[string]string{})

	//2、查询所有 `integral_release_user_with_ord` 记录
	integralReleaseUserWithOrds, err := db.IntegralReleaseUserWithOrdFindByParams(session, map[string]interface{}{
		"key":   "state",
		"value": 1,
	})
	var uids = make([]int, 0)
	for _, ord := range *integralReleaseUserWithOrds {
		uids = append(uids, ord.Uid)
	}
	uidList, err := db.DbsUserFindByIdsSess(session, uids)
	if err != nil {
		_ = session.Rollback()
		return err
	}
	userMap := make(map[int]model.User)
	for _, v := range *uidList {
		userMap[v.Uid] = v
	}

	for _, ord := range *integralReleaseUserWithOrds {
		_, ok := userMap[ord.Uid]
		if ok == false {
			//判断下这个用户有没有被删了
			continue
		}
		if zhios_order_relate_utils.StrToFloat64(ord.OrdDiscountRatio) == 0 {
			continue
		}
		//计算释放系数
		err, releaseSpeed := CalcReleaseRate(integralRelease, zhios_order_relate_utils.StrToFloat64(ord.OrdDiscountRatio))
		if err != nil {
			_ = session.Rollback()
			return err
		}
		if zhios_order_relate_utils.StrToFloat64(releaseSpeed) <= 0 {
			continue
		}
		releaseRate, _ := decimal.NewFromString(releaseSpeed)

		//释放绿积分
		err = ReleaseGreenCoinForUser(session, mid, integralRelease, &ord, releaseRate)
		if err != nil {
			_ = session.Rollback()
			return err
		}
	}

	//3、更新 `integral_release_basic` 表
	integralRelease.ReleaseDate = today
	updateAffected, err := db.IntegralReleaseUpdate(session, integralRelease.Id, integralRelease)
	if err != nil {
		_ = session.Rollback()
		return err
	}
	if updateAffected == 0 {
		_ = session.Rollback()
		return errors.New("更新 integral_release_basic 记录失败")
	}

	//4、提交事务
	err = session.Commit()
	if err != nil {
		_ = session.Rollback()
		return errors.New("事务提交失败")
	}

	zhios_order_relate_utils.CurlPost("http://zhios-app:5000/api/v1/check_app_over_time", map[string]string{
		"mid":      mid,
		"is_close": "0",
	}, map[string]string{})

	//cache.Del(PessimismLockKeyForIntegralRelease)

	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>积分释放结束<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")
	return nil
}

// DailySettlementIntegralReleaseForInterposition 每日结算“服务收益”
func DailySettlementIntegralReleaseForInterposition(engine *xorm.Engine, mid string, isTask bool) (err error) {
	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()
	now := time.Now()
	today := now.Format("2006-01-02")

	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", now.Hour())
	if isTask && (now.Hour() > 8) {
		//TODO::只在凌晨一点 ~ 凌晨 8 点运行
		return errors.New("非运行时间")
	}

	//1、查找 `integral_release` 基础设置
	integralRelease, err := db.IntegralReleaseGetOneByParams(session, map[string]interface{}{
		"key":   "is_use",
		"value": 1,
	})
	if err != nil {
		_ = session.Rollback()
		return err
	}
	if integralRelease.ReleaseDate != today {
		_ = session.Rollback()
		return errors.New("今日“积分释放”未结算")
	}

	if isTask {
		//TODO::增加“悲观锁”防止串行
		getString, _ := cache.GetString(PessimismLockKeyForIntegralReleaseForInterposition)
		if getString == PessimismLockValueForIntegralRelease {
			fmt.Println("-----------------", "上一次结算未执行完")
			return errors.New("上一次结算未执行完")
		}
		cache.SetEx(PessimismLockKeyForIntegralReleaseForInterposition, PessimismLockValueForIntegralRelease, 3600*10) //10小时
	}

	//2、查询所有 `integral_release_interposition_user_amount` 记录
	var users []model.IntegralReleaseInterpositionUserAmount
	err = session.Where("waite_amount > 0").Find(&users)
	if err != nil {
		_ = session.Rollback()
		return err
	}
	for _, user := range users {
		//查询当前用户今日的积分释放值
		var integralReleaseUserWithOrdFlows []model.IntegralReleaseUserWithOrdFlow
		err := session.Where("uid =? and release_date =?", user.OriginalUid, today).And("release_green_coin_num > 0").Find(&integralReleaseUserWithOrdFlows)
		if err != nil {
			zhios_order_relate_utils.FilePutContents("find_integral_release_user_with_ord_flow_err", zhios_order_relate_utils.SerializeStr(map[string]interface{}{
				"uid":  user.Uid,
				"date": today,
				"err":  err,
			}))
		}
		if len(integralReleaseUserWithOrdFlows) == 0 {
			continue
		}
		for _, integralReleaseUserWithOrdFlow := range integralReleaseUserWithOrdFlows {
			//发放服务收益
			err = DealIntegralReleaseInterpositionUserAmount(session, md.DealIntegralReleaseInterpositionUserAmountReq{
				Kind:        "sub",
				Mid:         mid,
				OrdId:       integralReleaseUserWithOrdFlow.OrdId,
				CoinId:      0,        //TODO::直接写死为0，返现金
				Uid:         user.Uid, //TODO::替换成间推用户uid
				OriginalUid: user.OriginalUid,
				Amount:      zhios_order_relate_utils.StrToFloat64(integralReleaseUserWithOrdFlow.ReleaseGreenCoinNum),
			})
			if err != nil {
				_ = session.Rollback()
				return err
			}
		}
	}

	//3、提交事务
	err = session.Commit()
	if err != nil {
		_ = session.Rollback()
		return errors.New("事务提交失败")
	}

	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>每日结算-服务收益-结束<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")
	return nil
}

// CalcReleaseRate 计算释放系数
func CalcReleaseRate(integralReleaseBasic *model.IntegralReleaseBasic, ordDiscountRatio float64) (err error, releaseSpeed string) {
	var businessDiscountSet []*md.BusinessDiscountSet
	if err := json.Unmarshal([]byte(integralReleaseBasic.BusinessDiscountSet), &businessDiscountSet); err != nil {
		return zhios_order_relate_logx.Warn("解析【商家让利释放速度设置】出错"), releaseSpeed
	}

	for _, value := range businessDiscountSet {
		if value.Range[0] <= ordDiscountRatio && ordDiscountRatio < value.Range[1] {
			releaseSpeed = value.ReleaseSpeed
		}
	}
	return nil, releaseSpeed
}

// ReleaseGreenCoinForUser 释放绿积分
func ReleaseGreenCoinForUser(session *xorm.Session, mid string, integralReleaseBasic *model.IntegralReleaseBasic, integralReleaseUserWithOrd *model.IntegralReleaseUserWithOrd, releaseSpeed decimal.Decimal) error {
	//1、计算需要释放多少绿积分
	totalRedCoinNum, _ := decimal.NewFromString(integralReleaseUserWithOrd.TotalRedCoinNum)
	releasedGreenCoinNum, _ := decimal.NewFromString(integralReleaseUserWithOrd.ReleasedGreenCoinNum)
	releaseCoinNum := totalRedCoinNum.Sub(releasedGreenCoinNum).Mul(releaseSpeed).RoundFloor(8)

	//2、释放绿积分
	var title = md.IntegralReleaseGreenCoinGrantByOrdTitleForUserVirtualCoinFlow
	var transferType = md.IntegralReleaseGreenCoinGrantByOrdTransferTypeForUserVirtualCoinFlow
	var titleRed = md.IntegralReleaseRedCoinGrantByOrdTitleDeductForUserVirtualCoinFlow
	var transferTypeRed = md.IntegralReleaseRedCoinGrantByOrdTransferTypeDeductForUserVirtualCoinFlow
	if integralReleaseUserWithOrd.Pvd == 0 {
		title = md.IntegralReleaseGreenCoinGrantBySignInTitleForUserVirtualCoinFlow
		transferType = md.IntegralReleaseGreenCoinGrantBySignInTransferTypeForUserVirtualCoinFlow
		titleRed = md.IntegralReleaseRedCoinGrantBySignInTitleDeductForUserVirtualCoinFlow
		transferTypeRed = md.IntegralReleaseRedCoinGrantBySignInTransferTypeDeductForUserVirtualCoinFlow
	}
	err := DealUserCoinForIntegralRelease(session, md.DealUserCoinReq{
		Kind:         "add",
		Mid:          mid,
		Title:        title,
		TransferType: transferType,
		OrdId:        integralReleaseUserWithOrd.OrdId,
		CoinId:       integralReleaseBasic.Coin2,
		Uid:          integralReleaseUserWithOrd.Uid,
		Amount:       zhios_order_relate_utils.StrToFloat64(releaseCoinNum.String()),
	})
	if err != nil {
		return err
	}
	//扣除红积分
	err = DealUserCoinForIntegralRelease(session, md.DealUserCoinReq{
		Kind:         "sub",
		Mid:          mid,
		Title:        titleRed,
		TransferType: transferTypeRed,
		OrdId:        integralReleaseUserWithOrd.OrdId,
		CoinId:       integralReleaseBasic.Coin1,
		Uid:          integralReleaseUserWithOrd.Uid,
		Amount:       zhios_order_relate_utils.StrToFloat64(releaseCoinNum.String()),
	})
	if err != nil {
		return err
	}
	releasedGreenCoinNumBefore, _ := decimal.NewFromString(integralReleaseUserWithOrd.ReleasedGreenCoinNum)
	releasedGreenCoinNumAfter := releasedGreenCoinNumBefore.Add(releaseCoinNum)

	//3、更改`integral_release_user_with_ord`记录
	integralReleaseUserWithOrd.ReleasedGreenCoinNum = releasedGreenCoinNumAfter.String()
	updateAffected, err := db.IntegralReleaseUserWithOrdUpdate(session, integralReleaseUserWithOrd.Id, integralReleaseUserWithOrd)
	if err != nil {
		return err
	}
	if updateAffected == 0 {
		err = errors.New("更新 integral_release_user_with_ord 记录失败")
		return err
	}

	//4、插入`integral_release_user_with_ord_flow`记录
	now := time.Now()
	var integralReleaseUserWithOrdFlow = model.IntegralReleaseUserWithOrdFlow{
		Uid:                        integralReleaseUserWithOrd.Uid,
		OrdId:                      integralReleaseUserWithOrd.OrdId,
		Pvd:                        integralReleaseUserWithOrd.Pvd,
		TotalRedCoinNum:            integralReleaseUserWithOrd.TotalRedCoinNum,
		ReleasedGreenCoinNumBefore: releasedGreenCoinNumBefore.String(),
		ReleasedGreenCoinNumAfter:  releasedGreenCoinNumAfter.String(),
		ReleaseGreenCoinNum:        releaseCoinNum.String(),
		ReleaseRateDay:             releaseSpeed.String(),
		ReleaseDate:                now.Format("2006-01-02"),
		CreateAt:                   now,
		UpdateAt:                   now,
	}
	_, err = db.IntegralReleaseUserWithOrdFlowInsert(session, &integralReleaseUserWithOrdFlow)
	if err != nil {
		return err
	}
	return nil
}

// DealUserCoinForIntegralRelease 处理给用户虚拟币积分
func DealUserCoinForIntegralRelease(session *xorm.Session, req md.DealUserCoinReq) (err error) {
	if req.Amount < 0 {
		req.Amount = 0
	}
	//1、分布式锁阻拦
	requestIdPrefix := fmt.Sprintf(DealUserCoinForIntegralReleaseRequestIdPrefix, req.Mid, req.CoinId, req.Uid)
	cb, err := svc.HandleDistributedLock(req.Mid, strconv.Itoa(req.Uid), requestIdPrefix)
	if err != nil {
		return err
	}
	if cb != nil {
		defer cb() // 释放锁
	}

	//2、计算&&组装数据
	now := time.Now()
	coinAmount, err := svc.GetUserCoinAmount(session, req.Mid, req.CoinId, req.Uid)
	if err != nil {
		return err
	}
	coinAmountValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(coinAmount))
	amountValue := decimal.NewFromFloat(req.Amount).RoundFloor(8)

	var userVirtualCoinFlow model.UserVirtualCoinFlow
	userVirtualCoinFlow.CoinId = req.CoinId
	userVirtualCoinFlow.Title = req.Title
	userVirtualCoinFlow.TransferType = req.TransferType
	userVirtualCoinFlow.Uid = req.Uid
	userVirtualCoinFlow.ToUid = req.ToUid
	userVirtualCoinFlow.OrdId = req.OrdId
	userVirtualCoinFlow.BeforeAmout = coinAmount
	userVirtualCoinFlow.Amout = amountValue.String()
	userVirtualCoinFlow.CreateTime = now
	if req.Kind == "add" {
		userVirtualCoinFlow.Direction = 1
		userVirtualCoinFlow.AfterAmout = coinAmountValue.Add(amountValue).RoundFloor(8).String()
	} else if req.Kind == "sub" {
		userVirtualCoinFlow.Direction = 2
		userVirtualCoinFlow.AfterAmout = coinAmountValue.Sub(amountValue).RoundFloor(8).String()
	} else {
		err = errors.New("错误的kind类型")
		return err
	}
	if zhios_order_relate_utils.StrToFloat64(userVirtualCoinFlow.AfterAmout) < 0 {
		var coin model.VirtualCoin
		_, err = session.Where("id = ?", req.CoinId).Get(&coin)
		if err != nil {
			return err
		}
		zhios_order_relate_utils.FilePutContents("virtual_coin_not", zhios_order_relate_utils.SerializeStr(map[string]interface{}{
			"uid":           userVirtualCoinFlow.Uid,
			"amount":        userVirtualCoinFlow.Amout,
			"before_amount": userVirtualCoinFlow.BeforeAmout,
			"after_amount":  userVirtualCoinFlow.AfterAmout,
			"coin_id":       userVirtualCoinFlow.CoinId,
		}))
		return errors.New("用户" + zhios_order_relate_utils.IntToStr(userVirtualCoinFlow.Uid) + "的" + coin.Name + "不足")
		//userVirtualCoinFlow.AfterAmout = "0"
	}
	//3、插入 `user_virtual_coin_flow` 记录
	_, err = db.UserVirtualCoinFlowInsert(session, &userVirtualCoinFlow)
	if err != nil {
		return err
	}

	//4、修改 `user_virtual_amount`的amount值 && 及缓存
	err = svc.SetCacheUserVirtualAmount(session, req.Mid, userVirtualCoinFlow.AfterAmout, req.CoinId, req.Uid, true)
	if err != nil {
		return err
	}

	return nil
}

// DealIntegralReleaseInterpositionUserAmount 处理间推用户服务收益
func DealIntegralReleaseInterpositionUserAmount(session *xorm.Session, req md.DealIntegralReleaseInterpositionUserAmountReq) (err error) {
	if req.Amount < 0 {
		req.Amount = 0
	}

	//1、计算&&组装数据
	now := time.Now()
	userAmount, err := db.IntegralReleaseInterpositionUserAmountGetOneByParams(session, req.CoinId, map[string]interface{}{
		"key":   "uid",
		"value": req.Uid,
	})
	if err != nil && err.Error() == "not found" {
		userAmount = &model.IntegralReleaseInterpositionUserAmount{
			Uid:         req.Uid,
			OriginalUid: req.OriginalUid,
			CoinId:      req.CoinId,
			CreateAt:    now,
			UpdateAt:    now,
		}
		_, err = db.IntegralReleaseInterpositionUserAmountInsert(session, userAmount)
	}
	if err != nil {
		return err
	}

	totalAmountValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(userAmount.TotalAmount))
	waiteAmountValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(userAmount.WaiteAmount))
	alreadyAmountValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(userAmount.AlreadyAmount))
	amountValue := decimal.NewFromFloat(req.Amount).RoundFloor(8)

	var integralReleaseInterpositionUserFlow = model.IntegralReleaseInterpositionUserFlow{
		Uid:      req.Uid,
		CoinId:   req.CoinId,
		OrdId:    req.OrdId,
		Amount:   amountValue.String(),
		Date:     "",
		CreateAt: now,
		UpdateAt: now,
	}
	if req.Kind == "add" {
		integralReleaseInterpositionUserFlow.Kind = 1
		integralReleaseInterpositionUserFlow.BeforeTotalAmount = totalAmountValue.String()
		integralReleaseInterpositionUserFlow.AfterTotalAmount = totalAmountValue.Add(amountValue).String()
		integralReleaseInterpositionUserFlow.BeforeWaiteAmount = waiteAmountValue.String()
		integralReleaseInterpositionUserFlow.AfterWaiteAmount = waiteAmountValue.Add(amountValue).String()
		integralReleaseInterpositionUserFlow.BeforeAlreadyAmount = alreadyAmountValue.String()
		integralReleaseInterpositionUserFlow.AfterAlreadyAmount = alreadyAmountValue.String()
	} else if req.Kind == "sub" {
		if zhios_order_relate_utils.StrToFloat64(waiteAmountValue.Sub(amountValue).String()) < 0 { //不够扣就把剩余的扣了
			amountValue = waiteAmountValue
			req.Amount = zhios_order_relate_utils.StrToFloat64(waiteAmountValue.String())
		}
		integralReleaseInterpositionUserFlow.Kind = 2
		integralReleaseInterpositionUserFlow.BeforeTotalAmount = totalAmountValue.String()
		integralReleaseInterpositionUserFlow.AfterTotalAmount = totalAmountValue.String()
		integralReleaseInterpositionUserFlow.BeforeWaiteAmount = waiteAmountValue.String()
		integralReleaseInterpositionUserFlow.AfterWaiteAmount = waiteAmountValue.Sub(amountValue).String()
		integralReleaseInterpositionUserFlow.BeforeAlreadyAmount = alreadyAmountValue.String()
		integralReleaseInterpositionUserFlow.AfterAlreadyAmount = alreadyAmountValue.Add(amountValue).String()
		integralReleaseInterpositionUserFlow.Date = now.Format("2006-01-02")
		//TODO::判断增加用户 余额 or 虚拟币积分
		if req.CoinId == 0 {
			orderType := enum.FinUserFlowOrderActionString(md.IntegralReleaseServiceRevenueOrderTypeForFinUserFlow)
			var dealUserAmount = md.DealUserAmount{
				Kind:      "add",
				Mid:       req.Mid,
				Title:     md.IntegralReleaseServiceRevenueTitleForFinUserFlow,
				OrderType: orderType,
				OrdAction: md.IntegralReleaseServiceRevenueOrderTypeForFinUserFlow,
				OrdId:     req.OrdId,
				Uid:       req.Uid,
				Amount:    req.Amount,
			}
			err = svc.DealUserAmount(session, dealUserAmount)
		} else {
			//释放服务收益
			err = DealUserCoinForIntegralRelease(session, md.DealUserCoinReq{
				Kind:         "add",
				Mid:          req.Mid,
				Title:        md.IntegralReleaseServiceRevenueTitleForUserVirtualCoinFlow,
				TransferType: md.IntegralReleaseServiceRevenueTransferTypeForUserVirtualCoinFlow,
				OrdId:        req.OrdId,
				CoinId:       req.CoinId,
				Uid:          req.Uid,
				Amount:       req.Amount,
			})
		}
		if err != nil {
			return err
		}
	} else {
		err = errors.New("错误的kind类型")
		return err
	}

	//2、插入 `integral_release_interposition_user_flow` 记录
	_, err = db.IntegralReleaseInterpositionUserFlowInsert(session, &integralReleaseInterpositionUserFlow)
	if err != nil {
		return err
	}

	//3、修改 `integral_release_interposition_user_amount` 的相关 amount 值
	userAmount.TotalAmount = integralReleaseInterpositionUserFlow.AfterTotalAmount
	userAmount.WaiteAmount = integralReleaseInterpositionUserFlow.AfterWaiteAmount
	userAmount.AlreadyAmount = integralReleaseInterpositionUserFlow.AfterAlreadyAmount
	updateAffected, _ := db.IntegralReleaseInterpositionUserAmountUpdate(session, userAmount.Id, userAmount)
	if err != nil {
		return err
	}
	if updateAffected == 0 {
		return errors.New("更新 integral_release_interposition_user_flow 记录失败")
	}
	return nil
}
