package one_circles

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"strconv"
	"time"
	"xorm.io/xorm"
)

const PessimismLockKey = "daily_settlement_block_star_chain_pessimism_lock_key"
const PessimismLockValue = "running"

// DealUserCoin 处理给用户虚拟币积分
func DealUserCoin(session *xorm.Session, req md.DealUserCoinReq) (err error) {
	if req.Amount < 0 {
		req.Amount = 0
	}
	//1、分布式锁阻拦
	requestIdPrefix := fmt.Sprintf(md2.DealUserCoinRequestIdPrefix, req.Mid, req.CoinId, req.Uid)
	cb, err := svc.HandleDistributedLock(req.Mid, strconv.Itoa(req.Uid), requestIdPrefix)
	if err != nil {
		return err
	}
	if cb != nil {
		defer cb() // 释放锁
	}

	//2、计算&&组装数据
	now := time.Now()
	coinAmount, err := svc.GetUserCoinAmount(session, req.Mid, req.CoinId, req.Uid)
	if err != nil {
		return err
	}
	coinAmountValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(coinAmount))
	amountValue := decimal.NewFromFloat(req.Amount).RoundFloor(8)

	var userVirtualCoinFlow model.UserVirtualCoinFlow
	userVirtualCoinFlow.CoinId = req.CoinId
	userVirtualCoinFlow.Title = req.Title
	userVirtualCoinFlow.TransferType = req.TransferType
	userVirtualCoinFlow.Uid = req.Uid
	userVirtualCoinFlow.ToUid = req.ToUid
	userVirtualCoinFlow.OrdId = req.OrdId
	userVirtualCoinFlow.BeforeAmout = coinAmount
	userVirtualCoinFlow.Amout = amountValue.String()
	userVirtualCoinFlow.CreateTime = now

	if req.Kind == "add" {
		userVirtualCoinFlow.Direction = 1
		userVirtualCoinFlow.AfterAmout = coinAmountValue.Add(amountValue).RoundFloor(8).String()
	} else if req.Kind == "sub" {
		userVirtualCoinFlow.Direction = 2
		userVirtualCoinFlow.AfterAmout = coinAmountValue.Sub(amountValue).RoundFloor(8).String()
	} else {
		err = errors.New("错误的kind类型")
		return err
	}
	if zhios_order_relate_utils.StrToFloat64(userVirtualCoinFlow.AfterAmout) < 0 {
		var coin model.VirtualCoin
		_, err = session.Where("id = ?", req.CoinId).Get(&coin)
		if err != nil {
			return err
		}
		zhios_order_relate_utils.FilePutContents("virtual_coin_not", zhios_order_relate_utils.SerializeStr(map[string]interface{}{
			"uid":           userVirtualCoinFlow.Uid,
			"amount":        userVirtualCoinFlow.Amout,
			"before_amount": userVirtualCoinFlow.BeforeAmout,
			"after_amount":  userVirtualCoinFlow.AfterAmout,
			"coin_id":       userVirtualCoinFlow.CoinId,
		}))
		return errors.New("用户" + zhios_order_relate_utils.IntToStr(userVirtualCoinFlow.Uid) + "的" + coin.Name + "不足")
	}
	//3、插入 `user_virtual_coin_flow` 记录
	_, err = db.UserVirtualCoinFlowInsert(session, &userVirtualCoinFlow)
	if err != nil {
		return err
	}

	//4、修改 `user_virtual_amount`的amount值 && 及缓存
	err = svc.SetCacheUserVirtualAmount(session, req.Mid, userVirtualCoinFlow.AfterAmout, req.CoinId, req.Uid, true)
	if err != nil {
		return err
	}

	return nil
}
