package one_circles

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"time"
	"xorm.io/xorm"
)

const LockKey = "auto_release_exchange_green_energy_lock_key"

// AutoReleaseExchangeGreenEnergy 结算绿色能量自动释放成可用绿色能量
func AutoReleaseExchangeGreenEnergy(engine *xorm.Engine, masterId string) (err error) {
	now := time.Now()
	fmt.Println(now.Hour())
	if !(now.Hour() > 1 && now.Hour() < 8) {
		//TODO::只在凌晨一点 ~ 凌晨 8 点运行
		return errors.New("非运行时间")
	}
	//TODO::增加“悲观锁”防止串行
	getString, _ := cache.GetString(LockKey)
	//if err != nil {
	//	return err
	//}
	if getString != "" {
		fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", "上一次结算未执行完")
		return errors.New("上一次结算未执行完")
	}
	cache.SetEx(LockKey, "running", 3600*8) //8小时

	//1、查找 `one_circles_green_energy_basic_setting` 基础设置 && `one_circles_public_platoon_basic_setting` 基础设置
	oneCirclesGreenEnergyBasicSetting, err := db.OneCirclesGreenEnergyBasicSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}

	settlementQuantity := oneCirclesGreenEnergyBasicSetting.SettlementQuantity
	fmt.Println("settlementQuantity>>>>>>>>>>>>", settlementQuantity)

	//2、当前 "可用"绿色能量可以兑换的用户数据
	var list1 []model.UserVirtualAmount
	err = engine.Where("coin_id = ?", oneCirclesGreenEnergyBasicSetting.TeamGreenEnergyCoinId).And("amount > ?", 0).Find(&list1)
	if err != nil {
		fmt.Println("err:::::1111", err)
		return
	}

	//4、处理释放
	settlementQuantityRadio := decimal.NewFromFloat(100)
	settlementQuantityValue := decimal.NewFromInt(int64(settlementQuantity)).Div(settlementQuantityRadio)
	for _, v := range list1 {
		userAmount, _ := decimal.NewFromString(v.Amount)
		settlementQuantityAmount, _ := userAmount.Mul(settlementQuantityValue).Float64()
		if settlementQuantityAmount > 0 {
			session := engine.NewSession()
			defer func() {
				session.Close()
				if err := recover(); err != nil {
					_ = zhios_order_relate_logx.Error(err)
				}
			}()
			session.Begin()

			//4.2给相应的用户加上个人的绿色积分(可用数量)
			err = DealUserCoin(session, md.DealUserCoinReq{
				Kind:         "add",
				Mid:          masterId,
				Title:        md.OneCirclesSettlementGreenEnergyExchangeGreenEnergy,
				TransferType: md.OneCirclesSettlementGreenEnergyExchangeGreenEnergyForUserVirtualCoinFlow,
				OrdId:        "",
				CoinId:       oneCirclesGreenEnergyBasicSetting.PersonGreenEnergyCoinId,
				Uid:          v.Uid,
				Amount:       settlementQuantityAmount,
			})

			//4.2给相应的用户减少个人的绿色积分(结算数量)
			err = DealUserCoin(session, md.DealUserCoinReq{
				Kind:         "sub",
				Mid:          masterId,
				Title:        md.OneCirclesSettlementGreenEnergyExchangeTobeGreenEnergy,
				TransferType: md.OneCirclesSettlementGreenEnergyExchangeTobeGreenEnergyForUserVirtualCoinFlow,
				OrdId:        "",
				CoinId:       oneCirclesGreenEnergyBasicSetting.TeamGreenEnergyCoinId,
				Uid:          v.Uid,
				Amount:       settlementQuantityAmount,
			})
			if err != nil {
				_ = session.Rollback()
				fmt.Println("err:::::33333", err)
				continue
			}

			err = session.Commit()
			if err != nil {
				_ = session.Rollback()
				continue
			}
		}
	}

	return
}
