package one_circles

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/md"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"encoding/json"
	"errors"
	"fmt"
	"strings"
	"time"
	"xorm.io/xorm"
)

// SettlementPublicGiveActivityCoin 计算观看激励视屏得到活跃积分
func SettlementPublicGiveActivityCoin(engine *xorm.Engine, masterId string, uid int) (err error) {
	//1、查找 `one_circles_public_platoon_basic_setting` 基础设置
	oneCirclesPublicPlatoonBasicSetting, err := db.OneCirclesPublicPlatoonBasicSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}
	if oneCirclesPublicPlatoonBasicSetting.VideoRewardIsOpen == 1 {
		var videoRewardSystem *md2.VideoRewardSystemStruct
		err = json.Unmarshal([]byte(oneCirclesPublicPlatoonBasicSetting.VideoRewardSystem), &videoRewardSystem)
		if err != nil {
			return
		}
		if videoRewardSystem.RewardValue == "" || videoRewardSystem.RewardTotalNum == "" || videoRewardSystem.IntervalMinutes == "" || videoRewardSystem.EachRoundHour == "" {
			err = errors.New("视屏奖励机制设置未完全！")
			return
		}
		var rewardSystem []*md2.RewardSystemStruct
		err = json.Unmarshal([]byte(oneCirclesPublicPlatoonBasicSetting.RewardSystem), &rewardSystem)
		if err != nil {
			return
		}
		if len(rewardSystem) == 0 {
			err = errors.New("未设置奖励机制！")
			return
		}
		rewardValue := zhios_order_relate_utils.StrToFloat64(videoRewardSystem.RewardValue) //奖励多少个活跃积分
		var rewardSystemMap = map[int]*md2.RewardSystemStruct{}
		for _, v := range rewardSystem {
			rewardSystemMap[v.Level] = v
		}
		oneCirclesPublicPlatoonUserRelation, err1 := db.OneCirclesPublicPlatoonUserRelationGetOneByParams(engine, map[string]interface{}{
			"key":   "uid",
			"value": uid,
		})
		if err1 != nil {
			return err1
		}

		var rewardFather []struct {
			Uid         int     `json:"uid"`          //用户id
			RewardValue float64 `json:"reward_value"` //奖励值
		}

		var fatherUids []string
		if oneCirclesPublicPlatoonUserRelation != nil {
			fatherUids = strings.Split(oneCirclesPublicPlatoonUserRelation.FatherUid, "-")
		}
		for k, id := range fatherUids {
			tmpOneCirclesPublicPlatoonUserRelation, err11 := db.OneCirclesPublicPlatoonUserRelationGetOneByParams(engine, map[string]interface{}{
				"key":   "id",
				"value": id,
			})
			if err11 != nil {
				return err11
			}
			if tmpOneCirclesPublicPlatoonUserRelation == nil {
				continue
			}
			if tmpOneCirclesPublicPlatoonUserRelation.Uid <= 0 {
				//待填充位
				continue
			}

			fatherUid := zhios_order_relate_utils.IntToStr(tmpOneCirclesPublicPlatoonUserRelation.Uid)
			fatherReward := rewardSystemMap[k+1]
			//TODO::判断是否活跃
			var m model.OneCirclesGreenEnergySignIn
			has, err33 := engine.Where("uid =?", fatherUid).And("end_time >=?", time.Now().Format("2006-01-02 15:04:05")).Get(&m)
			if err33 != nil {
				return err33
			}
			if !has {
				//不活跃不需要奖励
				continue
			}
			//判断是否满足奖励条件
			userCount, _, err2 := db.SumUserRelateByParentUid(engine, fatherUid)
			if err2 != nil {
				return err2
			}

			if fatherReward != nil && userCount >= zhios_order_relate_utils.StrToInt64(fatherReward.RewardCondition) {
				fatherRewardValue := rewardValue * (zhios_order_relate_utils.StrToFloat64(fatherReward.RewardValue) / 100)
				rewardFather = append(rewardFather, struct {
					Uid         int     `json:"uid"`          //用户id
					RewardValue float64 `json:"reward_value"` //奖励值
				}{
					Uid:         zhios_order_relate_utils.StrToInt(fatherUid),
					RewardValue: fatherRewardValue,
				})
			}
		}

		//增加 one_circles_user_watch_records 记录
		oneCirclesUserWatchRecords, err1 := db.OneCirclesUserWatchRecordsGetOneByParams(engine, map[string]interface{}{
			"key":   "uid",
			"value": uid,
		})
		if err1 != nil {
			return err1
		}
		now := time.Now()
		if oneCirclesUserWatchRecords == nil {
			residueWatchAdNum := zhios_order_relate_utils.StrToInt(videoRewardSystem.RewardTotalNum) - 1
			if residueWatchAdNum < 0 {
				residueWatchAdNum = zhios_order_relate_utils.StrToInt(videoRewardSystem.RewardTotalNum)
			}
			_, err2 := db.OneCirclesUserWatchRecordsInsert(engine, &model.OneCirclesUserWatchRecords{
				Uid:               uid,
				NextWatchAdDate:   now.Add(time.Hour * time.Duration(zhios_order_relate_utils.StrToInt(videoRewardSystem.EachRoundHour))),
				ResidueWatchAdNum: zhios_order_relate_utils.StrToInt(videoRewardSystem.RewardTotalNum) - 1,
				CreateAt:          now,
				UpdateAt:          now,
			})
			if err2 != nil {
				return err2
			}
		} else {
			residueWatchAdNum := oneCirclesUserWatchRecords.ResidueWatchAdNum - 1
			nextWatchAdDate := oneCirclesUserWatchRecords.NextWatchAdDate
			if residueWatchAdNum == 0 { //最后一条广告
				residueWatchAdNum = zhios_order_relate_utils.StrToInt(videoRewardSystem.RewardTotalNum)
			}
			if residueWatchAdNum == zhios_order_relate_utils.StrToInt(videoRewardSystem.RewardTotalNum)-1 { //第一条广告
				nextWatchAdDate = now.Add(time.Hour * time.Duration(zhios_order_relate_utils.StrToInt(videoRewardSystem.EachRoundHour)))
			}
			oneCirclesUserWatchRecords.ResidueWatchAdNum = residueWatchAdNum
			oneCirclesUserWatchRecords.NextWatchAdDate = nextWatchAdDate
			_, err2 := db.OneCirclesUserWatchRecordsUpdate(engine, oneCirclesUserWatchRecords.Id, oneCirclesUserWatchRecords, "residue_watch_ad_num", "next_watch_ad_date")
			if err2 != nil {
				return err2
			}
		}

		session := engine.NewSession()
		defer func() {
			session.Close()
			if err := recover(); err != nil {
				_ = zhios_order_relate_logx.Error(err)
			}
		}()
		session.Begin()

		//给相应的用户加上"个人"活跃积分
		err = DealUserCoin(session, md.DealUserCoinReq{
			Kind:         "add",
			Mid:          masterId,
			Title:        md.OneCirclesWatchAdRewardPersonalActiveCoin,
			TransferType: md.OneCirclesWatchAdRewardPersonalActiveCoinForUserVirtualCoinFlow,
			OrdId:        "",
			CoinId:       oneCirclesPublicPlatoonBasicSetting.PersonActivePointsCoinId,
			Uid:          uid,
			Amount:       rewardValue,
		})
		if err != nil {
			_ = session.Rollback()
			fmt.Println("err:::::2222", err)
			return err
		}

		//给相应的用户加上"团队"活跃积分
		for _, vv := range rewardFather {
			err = DealUserCoin(session, md.DealUserCoinReq{
				Kind:         "add",
				Mid:          masterId,
				Title:        md.OneCirclesWatchAdRewardTeamActiveCoin,
				TransferType: md.OneCirclesWatchAdRewardTeamActiveCoinForUserVirtualCoinFlow,
				OrdId:        "",
				CoinId:       oneCirclesPublicPlatoonBasicSetting.TeamActivePointsCoinId,
				Uid:          vv.Uid,
				Amount:       vv.RewardValue,
			})
			if err != nil {
				_ = session.Rollback()
				fmt.Println("err:::::2222", err)
				return err
			}
		}
		err = session.Commit()
		if err != nil {
			_ = session.Rollback()
			return errors.New("事务提交失败")
		}

		redisKey := fmt.Sprintf(md2.UserNextWatchAdDate, masterId, uid)
		var watchAdDate string
		if oneCirclesUserWatchRecords.ResidueWatchAdNum == zhios_order_relate_utils.StrToInt(videoRewardSystem.RewardTotalNum) {
			if oneCirclesUserWatchRecords.NextWatchAdDate.Before(time.Now()) {
				watchAdDate = ""
			} else {
				watchAdDate = oneCirclesUserWatchRecords.NextWatchAdDate.Format("2006-01-02 15:04:05")
			}
		} else {
			watchAdDate = time.Now().Add(time.Duration(zhios_order_relate_utils.StrToInt64(videoRewardSystem.IntervalMinutes)) * time.Minute).Format("2006-01-02 15:04:05")
			//watchAdDate = oneCirclesUserWatchRecords.NextWatchAdDate.Format("2006-01-02 15:04:05")
		}
		cache.SetEx(redisKey, watchAdDate, 60*60*24) //TODO::默认缓存1小时
	}
	return
}

// CalcUserContinuousDailyActivityDays 计算用户连续活跃天数
func CalcUserContinuousDailyActivityDays(engine *xorm.Engine, masterId string, uid int, startDate string, endDate string) (err error, days int, isContinuousDailyActivity bool) {
	startAt, err := time.ParseInLocation("2006-01-02", startDate, time.Local) //起始时间
	endAt, err := time.ParseInLocation("2006-01-02", endDate, time.Local)     //起始时间
	var list []model.OneCirclesUserActivity
	err = engine.Where("date >= ?", startAt.Format("2006-01-02")).
		And("date < ?", endAt.Format("2006-01-02")).
		And("uid =?", uid).Find(&list)
	if err != nil {
		fmt.Println("err:::::1111", err)
		return
	}
	days = len(list)
	diffDays := zhios_order_relate_utils.GetDiffDays(endAt, startAt) //相差天数
	if days == diffDays {
		isContinuousDailyActivity = true
	}
	return
}
