package one_circles

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"math"
	"strconv"
	"strings"
	"time"
	"xorm.io/xorm"
)

// AddOneCirclesPublicPlatoonUserRelationCommission 新增公排用户关系记录
func AddOneCirclesPublicPlatoonUserRelationCommission(engine *xorm.Engine, AddOneCirclesPublicPlatoonUserRelationCommissionReqList []*md.AddOneCirclesPublicPlatoonUserRelationCommissionReq) (map[string]*model.OneCirclesPublicPlatoonUserRelation, error) {
	var resp = map[string]*model.OneCirclesPublicPlatoonUserRelation{}
	//查找 `one_circles_public_platoon_basic_setting` 基础设置
	oneCirclesPublicPlatoonBasicSetting, err := db.OneCirclesPublicPlatoonBasicSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return nil, err
	}
	if oneCirclesPublicPlatoonBasicSetting == nil {
		return nil, errors.New("公排制度未开启")
	}

	for _, param := range AddOneCirclesPublicPlatoonUserRelationCommissionReqList {
		isHasOneCirclesPublicPlatoonUserRelation, err11 := db.OneCirclesPublicPlatoonUserRelationGetOneByParams(engine, map[string]interface{}{
			"key":   "uid",
			"value": param.Uid,
		})
		if err11 != nil {
			return nil, err11
		}
		if isHasOneCirclesPublicPlatoonUserRelation != nil {
			return nil, errors.New("当前用户已加入公排")
		}

		//1、判断是否有推荐人
		if param.RecommendUid != "" {
			//2、有推荐人
			//判断是否有uid为-1 (代表等待新用户填充) 的记录
			oneCirclesPublicPlatoonUserRelation, err1 := db.OneCirclesPublicPlatoonUserRelationGetOneByPid(engine, param.RecommendUid, map[string]interface{}{
				"key":   "uid",
				"value": -1,
			})
			if err1 != nil {
				return nil, err1
			}
			if oneCirclesPublicPlatoonUserRelation != nil {
				now := time.Now()
				oldUid := oneCirclesPublicPlatoonUserRelation.Uid
				oneCirclesPublicPlatoonUserRelation.Uid = zhios_order_relate_utils.StrToInt(param.Uid)
				oneCirclesPublicPlatoonUserRelation.RecommendUid = zhios_order_relate_utils.StrToInt(param.RecommendUid)
				oneCirclesPublicPlatoonUserRelation.CreateAt = now.Format("2006-01-02 15:04:05")
				oneCirclesPublicPlatoonUserRelation.UpdateAt = now.Format("2006-01-02 15:04:05")
				oneCirclesPublicPlatoonUserRelation.JoinAt = now.Format("2006-01-02 15:04:05")
				updateAffected, err2 := db.OneCirclesPublicPlatoonUserRelationUpdate(engine.NewSession(), oneCirclesPublicPlatoonUserRelation.Id, oneCirclesPublicPlatoonUserRelation)
				if err2 != nil {
					return nil, err2
				}
				if updateAffected == 0 {
					err = errors.New("更新 one_circles_public_platoon_user_relation 记录失败")
					return nil, err
				} else {
					for n := 1; n <= 9; n++ {
						str := "father_uid" + strconv.Itoa(n)
						sql := fmt.Sprintf("UPDATE `one_circles_public_platoon_user_relation` SET %s=%s where %s=%s", str, param.Uid, str, strconv.Itoa(oldUid))
						fmt.Println(">>>>>>>sql>>>>>>>", sql)
						_, err = engine.Exec(sql)
						if err != nil {
							return nil, err
						}
					}
					resp[param.Uid] = oneCirclesPublicPlatoonUserRelation
				}
				continue
			}
			res, err := OneCirclesPublicPlatoon(engine, zhios_order_relate_utils.StrToInt(param.Uid), zhios_order_relate_utils.StrToInt(param.RecommendUid), *oneCirclesPublicPlatoonBasicSetting)
			if err != nil {
				return nil, err
			}
			resp[param.Uid] = &res
		}
	}
	return resp, nil
}

/*
   公排方法
  TODO 相关公式:
  1: 每个等级的起始值(1+5^0+5^1+5^2+...+5^x-2), 每个等级的结束值(`5^0+5^1+5^2+...+5^x-1)
  2: 根据position查找父级position { (position-1)/5 }
  3: 根据position查找等级level {position-5^0-5^1-5^2-...-5^x 是否 <0 ?  => x+1 }
  4: 根据最新自增`id` 逆向推导 position {levelFirstPosition + (position-1)%5}
*/
func OneCirclesPublicPlatoon(engine *xorm.Engine, uid, recommendUid int, oneCirclesPublicPlatoonBasicSetting model.OneCirclesPublicPlatoonBasicSetting) (model.OneCirclesPublicPlatoonUserRelation, error) {
	var standardSmallPublicPlatoonRelation md.StandardSmallPublicPlatoonRelation
	var oneCirclesPublicPlatoonUserRelation model.OneCirclesPublicPlatoonUserRelation
	var fatherUid int
	//1、 查找当前 one_circles_public_platoon_user_relation 中 recommendUid 的记录
	m, err := db.OneCirclesPublicPlatoonUserRelationGetOneByParams(engine, map[string]interface{}{
		"key":   "uid",
		"value": recommendUid,
	})
	if err != nil {
		return model.OneCirclesPublicPlatoonUserRelation{}, err
	}

	//2、 查找当前 one_circles_public_platoon_user_relation 中 recommendUid 所有的子记录
	oneCirclesPublicPlatoonUserRelations, err := db.OneCirclesPublicPlatoonUserRelationFindByPid(engine, m.Uid, "father_uid1", "position1")
	if err != nil {
		return model.OneCirclesPublicPlatoonUserRelation{}, err
	}

	if len(oneCirclesPublicPlatoonUserRelations) == 0 {
		fatherUid = m.Uid

		//证明是第一个
		standardSmallPublicPlatoonRelation.FatherUid1 = m.Uid
		standardSmallPublicPlatoonRelation.FatherUid2 = m.FatherUid1
		standardSmallPublicPlatoonRelation.FatherUid3 = m.FatherUid2
		standardSmallPublicPlatoonRelation.FatherUid4 = m.FatherUid3
		standardSmallPublicPlatoonRelation.FatherUid5 = m.FatherUid4
		standardSmallPublicPlatoonRelation.FatherUid6 = m.FatherUid5
		standardSmallPublicPlatoonRelation.FatherUid7 = m.FatherUid6
		standardSmallPublicPlatoonRelation.FatherUid8 = m.FatherUid7
		standardSmallPublicPlatoonRelation.FatherUid9 = m.FatherUid8

		standardSmallPublicPlatoonRelation.Pid1 = m.Position
		standardSmallPublicPlatoonRelation.Pid2 = m.Position1
		standardSmallPublicPlatoonRelation.Pid3 = m.Position2
		standardSmallPublicPlatoonRelation.Pid4 = m.Position3
		standardSmallPublicPlatoonRelation.Pid5 = m.Position4
		standardSmallPublicPlatoonRelation.Pid6 = m.Position5
		standardSmallPublicPlatoonRelation.Pid7 = m.Position6
		standardSmallPublicPlatoonRelation.Pid8 = m.Position7
		standardSmallPublicPlatoonRelation.Pid9 = m.Position8

		standardSmallPublicPlatoonRelation.Position1 = oneCirclesSmallSearchPositionFirstSonPosition(m.Position, oneCirclesPublicPlatoonBasicSetting.SeveralTimes)
		standardSmallPublicPlatoonRelation.Position2 = oneCirclesSmallReverseDeductionPositionByFather(m.Position1, m.Level1, oneCirclesPublicPlatoonBasicSetting.SeveralTimes, 0)
		standardSmallPublicPlatoonRelation.Position3 = oneCirclesSmallReverseDeductionPositionByFather(m.Position2, m.Level2, oneCirclesPublicPlatoonBasicSetting.SeveralTimes, 0)
		standardSmallPublicPlatoonRelation.Position4 = oneCirclesSmallReverseDeductionPositionByFather(m.Position3, m.Level3, oneCirclesPublicPlatoonBasicSetting.SeveralTimes, 0)
		standardSmallPublicPlatoonRelation.Position5 = oneCirclesSmallReverseDeductionPositionByFather(m.Position4, m.Level4, oneCirclesPublicPlatoonBasicSetting.SeveralTimes, 0)
		standardSmallPublicPlatoonRelation.Position6 = oneCirclesSmallReverseDeductionPositionByFather(m.Position5, m.Level5, oneCirclesPublicPlatoonBasicSetting.SeveralTimes, 0)
		standardSmallPublicPlatoonRelation.Position7 = oneCirclesSmallReverseDeductionPositionByFather(m.Position6, m.Level6, oneCirclesPublicPlatoonBasicSetting.SeveralTimes, 0)
		standardSmallPublicPlatoonRelation.Position8 = oneCirclesSmallReverseDeductionPositionByFather(m.Position7, m.Level7, oneCirclesPublicPlatoonBasicSetting.SeveralTimes, 0)
		standardSmallPublicPlatoonRelation.Position9 = oneCirclesSmallReverseDeductionPositionByFather(m.Position8, m.Level8, oneCirclesPublicPlatoonBasicSetting.SeveralTimes, 0)

		if m.Level1 == 0 {
			m.Level1--
		}
		if m.Level2 == 0 {
			m.Level2--
		}
		if m.Level3 == 0 {
			m.Level3--
		}
		if m.Level4 == 0 {
			m.Level4--
		}
		if m.Level5 == 0 {
			m.Level5--
		}
		if m.Level6 == 0 {
			m.Level6--
		}
		if m.Level7 == 0 {
			m.Level7--
		}
		if m.Level8 == 0 {
			m.Level8--
		}
		if m.Level9 == 0 {
			m.Level9--
		}

		standardSmallPublicPlatoonRelation.Level1 = m.Level + 1
		standardSmallPublicPlatoonRelation.Level2 = m.Level1 + 1
		standardSmallPublicPlatoonRelation.Level3 = m.Level2 + 1
		standardSmallPublicPlatoonRelation.Level4 = m.Level3 + 1
		standardSmallPublicPlatoonRelation.Level5 = m.Level4 + 1
		standardSmallPublicPlatoonRelation.Level6 = m.Level5 + 1
		standardSmallPublicPlatoonRelation.Level7 = m.Level6 + 1
		standardSmallPublicPlatoonRelation.Level8 = m.Level7 + 1
		standardSmallPublicPlatoonRelation.Level9 = m.Level8 + 1
		standardSmallPublicPlatoonRelation.LevelTotal = m.LevelTotal + 1
	} else {
		//TODO::判断直属下级是否排满
		if len(oneCirclesPublicPlatoonUserRelations) >= oneCirclesPublicPlatoonBasicSetting.SeveralTimes {
			var fatherLevel float64
			smallMakeSearchLevel(&m.Position, float64(oneCirclesPublicPlatoonBasicSetting.SeveralTimes), &fatherLevel)
			var times = 0
			data, father, err := oneCirclesSmallFindSuitablePosition(engine, &m.Position, float64(oneCirclesPublicPlatoonBasicSetting.SeveralTimes), m.Uid, int(fatherLevel), &times)
			if err != nil {
				return model.OneCirclesPublicPlatoonUserRelation{}, err
			}
			fatherUid = father
			standardSmallPublicPlatoonRelation = data
		} else {
			fatherUid = m.Uid

			standardSmallPublicPlatoonRelation.FatherUid1 = oneCirclesPublicPlatoonUserRelations[0].FatherUid1
			standardSmallPublicPlatoonRelation.FatherUid2 = oneCirclesPublicPlatoonUserRelations[0].FatherUid2
			standardSmallPublicPlatoonRelation.FatherUid3 = oneCirclesPublicPlatoonUserRelations[0].FatherUid3
			standardSmallPublicPlatoonRelation.FatherUid4 = oneCirclesPublicPlatoonUserRelations[0].FatherUid4
			standardSmallPublicPlatoonRelation.FatherUid5 = oneCirclesPublicPlatoonUserRelations[0].FatherUid5
			standardSmallPublicPlatoonRelation.FatherUid6 = oneCirclesPublicPlatoonUserRelations[0].FatherUid6
			standardSmallPublicPlatoonRelation.FatherUid7 = oneCirclesPublicPlatoonUserRelations[0].FatherUid7
			standardSmallPublicPlatoonRelation.FatherUid8 = oneCirclesPublicPlatoonUserRelations[0].FatherUid8
			standardSmallPublicPlatoonRelation.FatherUid9 = oneCirclesPublicPlatoonUserRelations[0].FatherUid9

			standardSmallPublicPlatoonRelation.Pid1 = oneCirclesPublicPlatoonUserRelations[0].Pid1
			standardSmallPublicPlatoonRelation.Pid2 = oneCirclesPublicPlatoonUserRelations[0].Pid2
			standardSmallPublicPlatoonRelation.Pid3 = oneCirclesPublicPlatoonUserRelations[0].Pid3
			standardSmallPublicPlatoonRelation.Pid4 = oneCirclesPublicPlatoonUserRelations[0].Pid4
			standardSmallPublicPlatoonRelation.Pid5 = oneCirclesPublicPlatoonUserRelations[0].Pid5
			standardSmallPublicPlatoonRelation.Pid6 = oneCirclesPublicPlatoonUserRelations[0].Pid6
			standardSmallPublicPlatoonRelation.Pid7 = oneCirclesPublicPlatoonUserRelations[0].Pid7
			standardSmallPublicPlatoonRelation.Pid8 = oneCirclesPublicPlatoonUserRelations[0].Pid8
			standardSmallPublicPlatoonRelation.Pid9 = oneCirclesPublicPlatoonUserRelations[0].Pid9

			standardSmallPublicPlatoonRelation.Position1 = oneCirclesPublicPlatoonUserRelations[0].Position1 + 1
			if oneCirclesPublicPlatoonUserRelations[0].Position2 != 0 {
				standardSmallPublicPlatoonRelation.Position2 = oneCirclesPublicPlatoonUserRelations[0].Position2 + 1
			}
			if oneCirclesPublicPlatoonUserRelations[0].Position3 != 0 {
				standardSmallPublicPlatoonRelation.Position3 = oneCirclesPublicPlatoonUserRelations[0].Position3 + 1
			}
			if oneCirclesPublicPlatoonUserRelations[0].Position4 != 0 {
				standardSmallPublicPlatoonRelation.Position4 = oneCirclesPublicPlatoonUserRelations[0].Position4 + 1
			}
			if oneCirclesPublicPlatoonUserRelations[0].Position5 != 0 {
				standardSmallPublicPlatoonRelation.Position5 = oneCirclesPublicPlatoonUserRelations[0].Position5 + 1
			}
			if oneCirclesPublicPlatoonUserRelations[0].Position6 != 0 {
				standardSmallPublicPlatoonRelation.Position6 = oneCirclesPublicPlatoonUserRelations[0].Position6 + 1
			}
			if oneCirclesPublicPlatoonUserRelations[0].Position7 != 0 {
				standardSmallPublicPlatoonRelation.Position7 = oneCirclesPublicPlatoonUserRelations[0].Position7 + 1
			}
			if oneCirclesPublicPlatoonUserRelations[0].Position8 != 0 {
				standardSmallPublicPlatoonRelation.Position8 = oneCirclesPublicPlatoonUserRelations[0].Position8 + 1
			}
			if oneCirclesPublicPlatoonUserRelations[0].Position9 != 0 {
				standardSmallPublicPlatoonRelation.Position9 = oneCirclesPublicPlatoonUserRelations[0].Position9 + 1
			}

			standardSmallPublicPlatoonRelation.Level1 = oneCirclesPublicPlatoonUserRelations[0].Level1
			standardSmallPublicPlatoonRelation.Level2 = oneCirclesPublicPlatoonUserRelations[0].Level2
			standardSmallPublicPlatoonRelation.Level3 = oneCirclesPublicPlatoonUserRelations[0].Level3
			standardSmallPublicPlatoonRelation.Level4 = oneCirclesPublicPlatoonUserRelations[0].Level4
			standardSmallPublicPlatoonRelation.Level5 = oneCirclesPublicPlatoonUserRelations[0].Level5
			standardSmallPublicPlatoonRelation.Level6 = oneCirclesPublicPlatoonUserRelations[0].Level6
			standardSmallPublicPlatoonRelation.Level7 = oneCirclesPublicPlatoonUserRelations[0].Level7
			standardSmallPublicPlatoonRelation.Level8 = oneCirclesPublicPlatoonUserRelations[0].Level8
			standardSmallPublicPlatoonRelation.Level9 = oneCirclesPublicPlatoonUserRelations[0].Level9
			standardSmallPublicPlatoonRelation.LevelTotal = oneCirclesPublicPlatoonUserRelations[0].LevelTotal

		}

	}

	//2、查找对应pid的 one_circles_public_platoon_user_relation 记录
	var m1 model.OneCirclesPublicPlatoonUserRelation
	if has, err := engine.Where("uid=?", fatherUid).Get(&m1); err != nil || has == false {
		return oneCirclesPublicPlatoonUserRelation, err
	}
	var fatherUidStr string
	if m1.FatherUid == "" {
		//TODO::顶级
		fatherUidStr = zhios_order_relate_utils.IntToStr(m1.Id)
	} else {
		fatherUids := strings.Split(m1.FatherUid, "-")
		if len(fatherUids) >= oneCirclesPublicPlatoonBasicSetting.SeveralRows {
			fatherUidStr = zhios_order_relate_utils.IntToStr(m1.Id) + "-" + strings.Join(fatherUids[0:oneCirclesPublicPlatoonBasicSetting.SeveralRows:len(fatherUids)], "-")
		} else {
			fatherUidStr = zhios_order_relate_utils.IntToStr(m1.Id) + "-" + m1.FatherUid
		}
	}

	// 3、插入 one_circles_public_platoon_user_relation 记录
	now := time.Now()
	oneCirclesPublicPlatoonUserRelation.Uid = uid
	oneCirclesPublicPlatoonUserRelation.FatherUid = fatherUidStr

	oneCirclesPublicPlatoonUserRelation.FatherUid1 = standardSmallPublicPlatoonRelation.FatherUid1
	oneCirclesPublicPlatoonUserRelation.FatherUid2 = standardSmallPublicPlatoonRelation.FatherUid2
	oneCirclesPublicPlatoonUserRelation.FatherUid3 = standardSmallPublicPlatoonRelation.FatherUid3
	oneCirclesPublicPlatoonUserRelation.FatherUid4 = standardSmallPublicPlatoonRelation.FatherUid4
	oneCirclesPublicPlatoonUserRelation.FatherUid5 = standardSmallPublicPlatoonRelation.FatherUid5
	oneCirclesPublicPlatoonUserRelation.FatherUid6 = standardSmallPublicPlatoonRelation.FatherUid6
	oneCirclesPublicPlatoonUserRelation.FatherUid7 = standardSmallPublicPlatoonRelation.FatherUid7
	oneCirclesPublicPlatoonUserRelation.FatherUid8 = standardSmallPublicPlatoonRelation.FatherUid8
	oneCirclesPublicPlatoonUserRelation.FatherUid9 = standardSmallPublicPlatoonRelation.FatherUid9
	oneCirclesPublicPlatoonUserRelation.Pid1 = standardSmallPublicPlatoonRelation.Pid1
	oneCirclesPublicPlatoonUserRelation.Pid2 = standardSmallPublicPlatoonRelation.Pid2
	oneCirclesPublicPlatoonUserRelation.Pid3 = standardSmallPublicPlatoonRelation.Pid3
	oneCirclesPublicPlatoonUserRelation.Pid4 = standardSmallPublicPlatoonRelation.Pid4
	oneCirclesPublicPlatoonUserRelation.Pid5 = standardSmallPublicPlatoonRelation.Pid5
	oneCirclesPublicPlatoonUserRelation.Pid6 = standardSmallPublicPlatoonRelation.Pid6
	oneCirclesPublicPlatoonUserRelation.Pid7 = standardSmallPublicPlatoonRelation.Pid7
	oneCirclesPublicPlatoonUserRelation.Pid8 = standardSmallPublicPlatoonRelation.Pid8
	oneCirclesPublicPlatoonUserRelation.Pid9 = standardSmallPublicPlatoonRelation.Pid9
	oneCirclesPublicPlatoonUserRelation.Position = 1
	oneCirclesPublicPlatoonUserRelation.Position1 = standardSmallPublicPlatoonRelation.Position1
	oneCirclesPublicPlatoonUserRelation.Position2 = standardSmallPublicPlatoonRelation.Position2
	oneCirclesPublicPlatoonUserRelation.Position3 = standardSmallPublicPlatoonRelation.Position3
	oneCirclesPublicPlatoonUserRelation.Position4 = standardSmallPublicPlatoonRelation.Position4
	oneCirclesPublicPlatoonUserRelation.Position5 = standardSmallPublicPlatoonRelation.Position5
	oneCirclesPublicPlatoonUserRelation.Position6 = standardSmallPublicPlatoonRelation.Position6
	oneCirclesPublicPlatoonUserRelation.Position7 = standardSmallPublicPlatoonRelation.Position7
	oneCirclesPublicPlatoonUserRelation.Position8 = standardSmallPublicPlatoonRelation.Position8
	oneCirclesPublicPlatoonUserRelation.Position9 = standardSmallPublicPlatoonRelation.Position9
	oneCirclesPublicPlatoonUserRelation.Level = 1
	oneCirclesPublicPlatoonUserRelation.Level1 = standardSmallPublicPlatoonRelation.Level1
	oneCirclesPublicPlatoonUserRelation.Level2 = standardSmallPublicPlatoonRelation.Level2
	oneCirclesPublicPlatoonUserRelation.Level3 = standardSmallPublicPlatoonRelation.Level3
	oneCirclesPublicPlatoonUserRelation.Level4 = standardSmallPublicPlatoonRelation.Level4
	oneCirclesPublicPlatoonUserRelation.Level5 = standardSmallPublicPlatoonRelation.Level5
	oneCirclesPublicPlatoonUserRelation.Level6 = standardSmallPublicPlatoonRelation.Level6
	oneCirclesPublicPlatoonUserRelation.Level7 = standardSmallPublicPlatoonRelation.Level7
	oneCirclesPublicPlatoonUserRelation.Level8 = standardSmallPublicPlatoonRelation.Level8
	oneCirclesPublicPlatoonUserRelation.Level9 = standardSmallPublicPlatoonRelation.Level9
	oneCirclesPublicPlatoonUserRelation.LevelTotal = standardSmallPublicPlatoonRelation.LevelTotal
	oneCirclesPublicPlatoonUserRelation.RecommendUid = recommendUid
	oneCirclesPublicPlatoonUserRelation.JoinAt = now.Format("2006-01-02 15:04:05")
	oneCirclesPublicPlatoonUserRelation.CreateAt = now.Format("2006-01-02 15:04:05")
	oneCirclesPublicPlatoonUserRelation.UpdateAt = now.Format("2006-01-02 15:04:05")
	oneCirclesPublicPlatoonUserRelation.HasSonNum = 0
	_, err = db.OneCirclesPublicPlatoonUserRelationInsert(engine, &oneCirclesPublicPlatoonUserRelation)
	if err != nil {
		return model.OneCirclesPublicPlatoonUserRelation{}, err
	}

	// 4、改变直属父级的 has_son_num 数量
	update, err := engine.Where("uid = ?", fatherUid).SetExpr("has_son_num", "has_son_num + 1").Update(new(model.OneCirclesPublicPlatoonUserRelation))
	if err != nil {
		return oneCirclesPublicPlatoonUserRelation, err
	}
	if update == 0 {
		return oneCirclesPublicPlatoonUserRelation, errors.New("更新has_son_num数据出错")
	}
	return oneCirclesPublicPlatoonUserRelation, nil
}

// oneCirclesSmallReverseDeductionPosition 逆向推导 position
func oneCirclesSmallReverseDeductionPosition(calcPosition, levelFirstPosition, severalTimes int) (position int) {
	remainder := (calcPosition - 1) % severalTimes
	position = levelFirstPosition + remainder
	return
}

func oneCirclesSmallReverseDeductionPositionByFather(fatherPosition, level, severalTimes, remainder int) (position int) {
	if fatherPosition == 0 {
		position = 0
		return
	}
	firstPosition := oneCirclesSmallGetLevelForFirstPosition(level, severalTimes)
	endPosition := firstPosition + int(math.Pow(float64(severalTimes), float64(level-1))) - 1
	position = fatherPosition + (fatherPosition-firstPosition)*severalTimes + (endPosition - fatherPosition) + 1
	return
}

func oneCirclesSmallFindSuitablePosition(engine *xorm.Engine, position *int, severalTimes float64, fuid, fatherLevel int, times *int) (standardSmallPublicPlatoonRelation md.StandardSmallPublicPlatoonRelation, fatherUid int, err error) {
	*times++
	positionName := "position" + zhios_order_relate_utils.IntToStr(*times)
	fatherName := "father_uid" + zhios_order_relate_utils.IntToStr(*times)
	firstSonPosition := oneCirclesSmallSearchPositionFirstSonPosition(*position, int(severalTimes))
	var tempLevel float64
	smallMakeSearchLevel(&firstSonPosition, severalTimes, &tempLevel)
	firstSonLevel := int(tempLevel)
	endSonPosition := firstSonPosition + int(math.Pow(severalTimes, float64(firstSonLevel-fatherLevel))) - 1
	list, _, err := db.OneCirclesPublicPlatoonUserRelationFindCountByPosition(engine, fuid, fatherName, positionName, int64(firstSonPosition), int64(endSonPosition))
	if err != nil {
		return standardSmallPublicPlatoonRelation, fatherUid, err
	}

	if list[0].HasSonNum == int(severalTimes) {
		//TODO::则证明该层直属下级无空位
		position = &firstSonPosition
		return oneCirclesSmallFindSuitablePosition(engine, position, severalTimes, fuid, fatherLevel, times)
	} else if list[0].HasSonNum == 0 {
		fatherUid = list[0].Uid

		//TODO::新的一层
		standardSmallPublicPlatoonRelation.FatherUid1 = list[0].Uid
		standardSmallPublicPlatoonRelation.FatherUid2 = list[0].FatherUid1
		standardSmallPublicPlatoonRelation.FatherUid3 = list[0].FatherUid2
		standardSmallPublicPlatoonRelation.FatherUid4 = list[0].FatherUid3
		standardSmallPublicPlatoonRelation.FatherUid5 = list[0].FatherUid4
		standardSmallPublicPlatoonRelation.FatherUid6 = list[0].FatherUid5
		standardSmallPublicPlatoonRelation.FatherUid7 = list[0].FatherUid6
		standardSmallPublicPlatoonRelation.FatherUid8 = list[0].FatherUid7
		standardSmallPublicPlatoonRelation.FatherUid9 = list[0].FatherUid8

		standardSmallPublicPlatoonRelation.Pid1 = list[0].Position
		standardSmallPublicPlatoonRelation.Pid2 = list[0].Position1
		standardSmallPublicPlatoonRelation.Pid3 = list[0].Position2
		standardSmallPublicPlatoonRelation.Pid4 = list[0].Position3
		standardSmallPublicPlatoonRelation.Pid5 = list[0].Position4
		standardSmallPublicPlatoonRelation.Pid6 = list[0].Position5
		standardSmallPublicPlatoonRelation.Pid7 = list[0].Position6
		standardSmallPublicPlatoonRelation.Pid8 = list[0].Position7
		standardSmallPublicPlatoonRelation.Pid9 = list[0].Position8

		standardSmallPublicPlatoonRelation.Position1 = oneCirclesSmallSearchPositionFirstSonPosition(list[0].Position, int(severalTimes))
		standardSmallPublicPlatoonRelation.Position2 = oneCirclesSmallReverseDeductionPositionByFather(list[0].Position1, list[0].Level1, int(severalTimes), 0)
		standardSmallPublicPlatoonRelation.Position3 = oneCirclesSmallReverseDeductionPositionByFather(list[0].Position2, list[0].Level2, int(severalTimes), 0)
		standardSmallPublicPlatoonRelation.Position4 = oneCirclesSmallReverseDeductionPositionByFather(list[0].Position3, list[0].Level3, int(severalTimes), 0)
		standardSmallPublicPlatoonRelation.Position5 = oneCirclesSmallReverseDeductionPositionByFather(list[0].Position4, list[0].Level4, int(severalTimes), 0)
		standardSmallPublicPlatoonRelation.Position6 = oneCirclesSmallReverseDeductionPositionByFather(list[0].Position5, list[0].Level5, int(severalTimes), 0)
		standardSmallPublicPlatoonRelation.Position7 = oneCirclesSmallReverseDeductionPositionByFather(list[0].Position6, list[0].Level6, int(severalTimes), 0)
		standardSmallPublicPlatoonRelation.Position8 = oneCirclesSmallReverseDeductionPositionByFather(list[0].Position7, list[0].Level7, int(severalTimes), 0)
		standardSmallPublicPlatoonRelation.Position9 = oneCirclesSmallReverseDeductionPositionByFather(list[0].Position8, list[0].Level8, int(severalTimes), 0)

		if list[0].Level1 == 0 {
			list[0].Level1--
		}
		if list[0].Level2 == 0 {
			list[0].Level2--
		}
		if list[0].Level3 == 0 {
			list[0].Level3--
		}
		if list[0].Level4 == 0 {
			list[0].Level4--
		}
		if list[0].Level5 == 0 {
			list[0].Level5--
		}
		if list[0].Level6 == 0 {
			list[0].Level6--
		}
		if list[0].Level7 == 0 {
			list[0].Level7--
		}
		if list[0].Level8 == 0 {
			list[0].Level8--
		}
		if list[0].Level9 == 0 {
			list[0].Level9--
		}
		standardSmallPublicPlatoonRelation.Level1 = list[0].Level + 1
		standardSmallPublicPlatoonRelation.Level2 = list[0].Level1 + 1
		standardSmallPublicPlatoonRelation.Level3 = list[0].Level2 + 1
		standardSmallPublicPlatoonRelation.Level4 = list[0].Level3 + 1
		standardSmallPublicPlatoonRelation.Level5 = list[0].Level4 + 1
		standardSmallPublicPlatoonRelation.Level6 = list[0].Level5 + 1
		standardSmallPublicPlatoonRelation.Level7 = list[0].Level6 + 1
		standardSmallPublicPlatoonRelation.Level8 = list[0].Level7 + 1
		standardSmallPublicPlatoonRelation.Level9 = list[0].Level8 + 1
		standardSmallPublicPlatoonRelation.LevelTotal = list[0].LevelTotal + 1
	} else {
		fatherUid = list[0].Uid

		_, _, _, _, getPositionName, _, _, _ := oneCirclesGetPosition(list[0], *times+1)
		//oneCirclesPublicPlatoonUserRelations, err := db.OneCirclesPublicPlatoonUserRelationFindByPid(engine, m.Position, m.Uid, "pid1", "father_uid1", "position1")

		oneCirclesPublicPlatoonUserRelations, err := db.OneCirclesPublicPlatoonUserRelationFindByPid(engine, list[0].Uid, "father_uid1", getPositionName)

		if err != nil {
			return standardSmallPublicPlatoonRelation, fatherUid, err
		}

		if oneCirclesPublicPlatoonUserRelations[0].Position2 == 0 {
			oneCirclesPublicPlatoonUserRelations[0].Position2 = -1
		}
		if oneCirclesPublicPlatoonUserRelations[0].Position3 == 0 {
			standardSmallPublicPlatoonRelation.Position3 = -1
		}
		if oneCirclesPublicPlatoonUserRelations[0].Position4 == 0 {
			standardSmallPublicPlatoonRelation.Position4 = -1
		}
		if oneCirclesPublicPlatoonUserRelations[0].Position5 == 0 {
			standardSmallPublicPlatoonRelation.Position5 = -1
		}
		if oneCirclesPublicPlatoonUserRelations[0].Position6 == 0 {
			standardSmallPublicPlatoonRelation.Position6 = -1
		}
		if oneCirclesPublicPlatoonUserRelations[0].Position7 == 0 {
			standardSmallPublicPlatoonRelation.Position7 = -1
		}
		if oneCirclesPublicPlatoonUserRelations[0].Position8 == 0 {
			standardSmallPublicPlatoonRelation.Position8 = -1
		}
		if oneCirclesPublicPlatoonUserRelations[0].Position9 == 0 {
			standardSmallPublicPlatoonRelation.Position9 = -1
		}

		standardSmallPublicPlatoonRelation = md.StandardSmallPublicPlatoonRelation{
			FatherUid1: oneCirclesPublicPlatoonUserRelations[0].FatherUid1,
			FatherUid2: oneCirclesPublicPlatoonUserRelations[0].FatherUid2,
			FatherUid3: oneCirclesPublicPlatoonUserRelations[0].FatherUid3,
			FatherUid4: oneCirclesPublicPlatoonUserRelations[0].FatherUid4,
			FatherUid5: oneCirclesPublicPlatoonUserRelations[0].FatherUid5,
			FatherUid6: oneCirclesPublicPlatoonUserRelations[0].FatherUid6,
			FatherUid7: oneCirclesPublicPlatoonUserRelations[0].FatherUid7,
			FatherUid8: oneCirclesPublicPlatoonUserRelations[0].FatherUid8,
			FatherUid9: oneCirclesPublicPlatoonUserRelations[0].FatherUid9,
			Pid1:       oneCirclesPublicPlatoonUserRelations[0].Pid1,
			Pid2:       oneCirclesPublicPlatoonUserRelations[0].Pid2,
			Pid3:       oneCirclesPublicPlatoonUserRelations[0].Pid3,
			Pid4:       oneCirclesPublicPlatoonUserRelations[0].Pid4,
			Pid5:       oneCirclesPublicPlatoonUserRelations[0].Pid5,
			Pid6:       oneCirclesPublicPlatoonUserRelations[0].Pid6,
			Pid7:       oneCirclesPublicPlatoonUserRelations[0].Pid7,
			Pid8:       oneCirclesPublicPlatoonUserRelations[0].Pid8,
			Pid9:       oneCirclesPublicPlatoonUserRelations[0].Pid9,
			Position1:  oneCirclesPublicPlatoonUserRelations[0].Position1 + 1,
			Position2:  oneCirclesPublicPlatoonUserRelations[0].Position2 + 1,
			Position3:  oneCirclesPublicPlatoonUserRelations[0].Position3 + 1,
			Position4:  oneCirclesPublicPlatoonUserRelations[0].Position4 + 1,
			Position5:  oneCirclesPublicPlatoonUserRelations[0].Position5 + 1,
			Position6:  oneCirclesPublicPlatoonUserRelations[0].Position6 + 1,
			Position7:  oneCirclesPublicPlatoonUserRelations[0].Position7 + 1,
			Position8:  oneCirclesPublicPlatoonUserRelations[0].Position8 + 1,
			Position9:  oneCirclesPublicPlatoonUserRelations[0].Position9 + 1,
			Level1:     oneCirclesPublicPlatoonUserRelations[0].Level1,
			Level2:     oneCirclesPublicPlatoonUserRelations[0].Level2,
			Level3:     oneCirclesPublicPlatoonUserRelations[0].Level3,
			Level4:     oneCirclesPublicPlatoonUserRelations[0].Level4,
			Level5:     oneCirclesPublicPlatoonUserRelations[0].Level5,
			Level6:     oneCirclesPublicPlatoonUserRelations[0].Level6,
			Level7:     oneCirclesPublicPlatoonUserRelations[0].Level7,
			Level8:     oneCirclesPublicPlatoonUserRelations[0].Level8,
			Level9:     oneCirclesPublicPlatoonUserRelations[0].Level9,
			LevelTotal: oneCirclesPublicPlatoonUserRelations[0].LevelTotal,
		}
		return standardSmallPublicPlatoonRelation, fatherUid, nil
	}
	return
}

func oneCirclesGetPosition(m model.OneCirclesPublicPlatoonUserRelation, times int) (position, pid, level, fatherUid int, positionName, pidName, levelName, fatherUidName string) {
	if times == 1 {
		position = m.Position1
		positionName = "position1"
		pid = m.Pid1
		pidName = "pid1"
		level = m.Level1
		levelName = "level1"
		fatherUid = m.FatherUid1
		fatherUidName = "father_uid1"
	}
	if times == 2 {
		position = m.Position2
		positionName = "position2"
		pid = m.Pid2
		pidName = "pid2"
		level = m.Level2
		levelName = "level2"
		fatherUid = m.FatherUid2
		fatherUidName = "father_uid2"
	}
	if times == 3 {
		position = m.Position3
		positionName = "position3"
		pid = m.Pid3
		pidName = "pid3"
		level = m.Level3
		levelName = "level3"
		fatherUid = m.FatherUid3
		fatherUidName = "father_uid3"
	}
	if times == 4 {
		position = m.Position4
		positionName = "position4"
		pid = m.Pid4
		pidName = "pid4"
		level = m.Level4
		levelName = "level4"
		fatherUid = m.FatherUid4
		fatherUidName = "father_uid4"
	}
	if times == 5 {
		position = m.Position5
		positionName = "position5"
		pid = m.Pid5
		pidName = "pid5"
		level = m.Level5
		levelName = "level5"
		fatherUid = m.FatherUid5
		fatherUidName = "father_uid5"
	}
	if times == 6 {
		position = m.Position6
		positionName = "position6"
		pid = m.Pid6
		pidName = "pid6"
		level = m.Level6
		levelName = "level6"
		fatherUid = m.FatherUid6
		fatherUidName = "father_uid6"
	}
	if times == 7 {
		position = m.Position7
		positionName = "position7"
		pid = m.Pid7
		pidName = "pid7"
		level = m.Level7
		levelName = "level7"
		fatherUid = m.FatherUid7
		fatherUidName = "father_uid7"
	}
	if times == 8 {
		position = m.Position8
		positionName = "position8"
		pid = m.Pid8
		pidName = "pid8"
		level = m.Level8
		levelName = "level8"
		fatherUid = m.FatherUid8
		fatherUidName = "father_uid8"
	}
	if times == 9 {
		position = m.Position9
		positionName = "position9"
		pid = m.Pid9
		pidName = "pid9"
		level = m.Level9
		levelName = "level9"
		fatherUid = m.FatherUid9
		fatherUidName = "father_uid9"
	}

	return
}

// oneCirclesSmallGetLevelForFirstPosition 返回当前等级的起始值
func oneCirclesSmallGetLevelForFirstPosition(level, severalTimes int) (position int) {
	position = position + 1
	for n := 0; n <= (level - 2); n++ {
		position += int(math.Pow(float64(severalTimes), float64(n)))
	}
	return
}

//递归查找等级
func oneCirclesSmallMakeSearchLevel(position *int, rows float64, times *float64) (level int) {
	difference := *position - int(math.Pow(rows, *times))
	*times++
	if difference <= 0 {
		return int(*times)
	} else {
		position = &difference
		return oneCirclesSmallMakeSearchLevel(position, rows, times)
	}
}

//查找归属父级id
func oneCirclesSmallMakeSearchPid(position int, row int) (pid int) {
	divisionValue := (position - 1) / row
	if divisionValue == 0 {
		pid = 1
		return
	} else {
		if (divisionValue*row + 1) == position {
			pid = divisionValue
			return
		} else {
			pid = divisionValue + 1
			return
		}
	}
}

//查找当前位置下的第一个子集位置
func oneCirclesSmallSearchPositionFirstSonPosition(position int, times int) (sonPosition int) {
	sonPosition = (position * times) - (times - 2)
	return
}

//查找当前位位置相当于父级的等级
func oneCirclesSmallSearchPositionSonForLevel(fatherUid string, fatherPosition int) (level int) {
	fatherUids := strings.Split(fatherUid, "-")
	for key, item := range fatherUids {
		if item == strconv.Itoa(fatherPosition) {
			level = key + 1
			break
		}
	}
	return
}

// OneCirclesSmallFindWaitForDealUsers 查询待处理的用户
func OneCirclesSmallFindWaitForDealUsers(engine *xorm.Engine, page, pageSize int) (err error, resp []int) {
	now := time.Now().Format("2006-01-02")
	lists, err := db.OneCirclesPublicPlatoonUserRelationFindByParamsByPage(engine, map[string]interface{}{
		"key":   "wait_for_settlement_date",
		"value": now,
	}, page, pageSize)
	if err != nil {
		return
	}
	for _, list := range *lists {
		resp = append(resp, list.Uid)
	}
	return
}

//递归查找等级
func smallMakeSearchLevel(position *int, rows float64, times *float64) (level int) {
	difference := *position - int(math.Pow(rows, *times))
	*times++
	if difference <= 0 {
		return int(*times)
	} else {
		position = &difference
		return smallMakeSearchLevel(position, rows, times)
	}
}

// OneCirclesDealCommonWealthPunish 处理公排处罚
func OneCirclesDealCommonWealthPunish(engine *xorm.Engine, uid int, reason string) (err error, resp []map[string]string) {
	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()

	//1、查找 `user_public_platoon_setting` 基础设置
	oneCirclesPublicPlatoonBasicSetting, err := db.OneCirclesPublicPlatoonBasicSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		_ = session.Rollback()
		return
	}

	//2、查询出 `one_circles_public_platoon_user_relation` 中相关记录 && 将该记录的uid置为 -1
	params, err := db.OneCirclesPublicPlatoonUserRelationGetOneByParams(engine, map[string]interface{}{
		"key":   "uid",
		"value": uid,
	})
	if err != nil {
		_ = session.Rollback()
		return
	}
	if params == nil {
		fmt.Println("未查询到公排关系记录!!!!!!!", uid)
		return
	}

	//TODO::判断是否为推荐用户
	if params.RecommendUid == 0 {
		params.Uid = params.Uid - int(time.Now().Unix())
	} else {
		params.Uid = params.Uid - int(time.Now().Unix())
	}

	for n := 1; n <= 9; n++ {
		str := "father_uid" + strconv.Itoa(n)
		sql := fmt.Sprintf("UPDATE `one_circles_public_platoon_user_relation` SET %s=%s where %s=%s", str, strconv.Itoa(params.Uid), str, strconv.Itoa(uid))
		fmt.Println(">>>>>>>sql>>>>>>>", sql)
		_, err = session.Exec(sql)
		if err != nil {
			_ = session.Rollback()
			return
		}
	}

	updateAffected, err := db.OneCirclesPublicPlatoonUserRelationUpdate(session, params.Id, params)
	if err != nil {
		_ = session.Rollback()
		return
	}
	if updateAffected == 0 {
		err = errors.New("更新 one_circles_public_platoon_user_relation 记录失败")
		_ = session.Rollback()
		return
	}

	//3、新增一条 `one_circles_public_platoon_user_relation` 记录
	params.Uid = uid
	res, err := OneCirclesPublicPlatoon(engine, params.Uid, params.RecommendUid, *oneCirclesPublicPlatoonBasicSetting)
	if err != nil {
		_ = session.Rollback()
		return
	}

	//4、新增一条 `one_circles_user_public_platoon_system_punish_records` 记录
	now := time.Now()
	insertAffected, err := db.OneCirclesUserPublicPlatoonSystemPunishRecordsInsert(session, &model.OneCirclesUserPublicPlatoonSystemPunishRecords{
		Uid:        params.Uid,
		OldPostion: params.Position,
		NewPostion: res.Position,
		Date:       now.AddDate(0, 0, 30).Format("2006-01-02"),
		Title:      "共富收益-系统处罚记录",
		Reason:     reason,
		Type:       2,
		CreateAt:   now,
		UpdateAt:   now,
	})
	if err != nil {
		_ = session.Rollback()
		return
	}
	if insertAffected == 0 {
		err = errors.New("新增 one_circles_user_public_platoon_system_punish_records 记录失败")
		_ = session.Rollback()
		return
	}

	err = session.Commit()
	return
}

const OneCirclesDealUserPublicPlatoonPunishLockKey = "one_circles_deal_user_public_platoon_lock_key"

// OneCirclesDealUserPublicPlatoonPunish 处理公排处罚
func OneCirclesDealUserPublicPlatoonPunish(engine *xorm.Engine, masterId string) (err error) {
	now := time.Now()
	fmt.Println(now.Hour())
	if !(now.Hour() > 22 && now.Hour() < 24) {
		//TODO::只在凌晨一点 ~ 凌晨 8 点运行
		return errors.New("非运行时间")
	}
	//TODO::增加“悲观锁”防止串行
	getString, _ := cache.GetString(OneCirclesDealUserPublicPlatoonPunishLockKey)
	if getString != "" {
		fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", "上一次结算未执行完")
		return errors.New("上一次结算未执行完")
	}
	cache.SetEx(OneCirclesDealUserPublicPlatoonPunishLockKey, "running", 3600*8) //8小时

	//查找 `one_circles_public_platoon_basic_setting` 基础设置
	oneCirclesPublicPlatoonBasicSetting, err := db.OneCirclesPublicPlatoonBasicSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return err
	}
	if oneCirclesPublicPlatoonBasicSetting == nil {
		return errors.New("公排制度未开启")
	}
	mapOneCirclesPublicPlatoonFreePunishWithUser, err := db.FindAllOneCirclesPublicPlatoonFreePunishWithUser(engine)
	if err != nil {
		return err
	}
	if oneCirclesPublicPlatoonBasicSetting.SystemPunishReplace == 1 {
		systemPunishReplaceValue := oneCirclesPublicPlatoonBasicSetting.SystemPunishReplaceValue
		startDate := now.AddDate(0, 0, -systemPunishReplaceValue).Format("2006-01-02") + " 00:00:00"
		var page = 1
		var pageSize = 100
		for {
			var users []model.User
			err = engine.Limit(pageSize, (page-1)*pageSize).Desc("uid").Find(&users)
			if err != nil {
				return err
			}
			if len(users) <= 0 {
				break
			}
			for _, user := range users {
				var list []model.OneCirclesGreenEnergySignIn
				err = engine.Where("start_time >= ?", startDate).And("uid = ?", user.Uid).Find(&list)
				if len(list) <= 0 && mapOneCirclesPublicPlatoonFreePunishWithUser[user.Uid] == nil {
					var oneCirclesPublicPlatoonRecordsPunishWithUser model.OneCirclesPublicPlatoonRecordsPunishWithUser
					has, err1 := engine.Where("uid = ?", user.Uid).Get(&oneCirclesPublicPlatoonRecordsPunishWithUser)
					if err1 != nil {
						return err1
					}
					oneCirclesPublicPlatoonRecordsPunishWithUserDate, _ := time.ParseInLocation("2006-01-02", oneCirclesPublicPlatoonRecordsPunishWithUser.Date, time.Local)
					if has && now.AddDate(0, 0, -7).After(oneCirclesPublicPlatoonRecordsPunishWithUserDate) {
						//TODO::不进行重复处罚
						continue
					}
					//进行处罚
					err, _ = OneCirclesDealCommonWealthPunish(engine, user.Uid, "公排处罚")
					if err != nil {
						return err
					}

					//添加 one_circles_public_platoon_records_punish_with_user 记录
					if has {
						oneCirclesPublicPlatoonRecordsPunishWithUser.Date = now.Format("2006-01-02")
						_, err2 := db.OneCirclesPublicPlatoonRecordsPunishWithUserUpdate(engine, oneCirclesPublicPlatoonRecordsPunishWithUser.Id, &oneCirclesPublicPlatoonRecordsPunishWithUser, "date")
						if err2 != nil {
							return err2
						}
					} else {
						_, err2 := db.OneCirclesPublicPlatoonRecordsPunishWithUserInsert(engine, &model.OneCirclesPublicPlatoonRecordsPunishWithUser{
							Uid:  user.Uid,
							Date: now.Format("2006-01-02"),
						})
						if err2 != nil {
							return err2
						}
					}
				}
			}
			page++
		}
	}
	return
}
