package one_circles

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/enum"
	md2 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule/one_circles/md"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"math/rand"
	"time"
	"xorm.io/xorm"
)

// SettlementSignInGreenEnergy 计算签到得到绿色能量
func SettlementSignInGreenEnergy(engine *xorm.Engine, masterId string, ch *rabbit.Channel) (err error) {
	//1、查找 `one_circles_green_energy_basic_setting` 基础设置
	oneCirclesGreenEnergyBasicSetting, err := db.OneCirclesGreenEnergyBasicSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}
	if oneCirclesGreenEnergyBasicSetting == nil {
		return errors.New("开关未开！！")
	}
	//3、统计签到结束的用户数据
	now := time.Now()
	var page = 1
	var pageSize = 100
	for {
		var list []model.OneCirclesGreenEnergySignIn
		err = engine.Where("end_time <= ?", now.Format("2006-01-02 15:04:05")).And("is_completed =?", 0).Limit(pageSize, (page-1)*pageSize).Asc("id").Find(&list)
		if err != nil {
			fmt.Println("err:::::1111", err)
			return
		}
		if len(list) <= 0 {
			break
		}

		var ids []int64
		for _, v := range list {
			//TODO::推入rabbitmq 异步处理
			ch.Publish(md.OneCirclesExchange, md.OneCirclesStructForSignIn{
				MasterId: masterId,
				Uid:      v.Uid,
				Id:       v.Id,
			}, md.OneCirclesRoutKeyForSignIn)
			ids = append(ids, v.Id)
		}
		//6、更新 `one_circles_green_energy_sign_in` 中的 is_completed 状态
		if len(ids) > 0 {
			_, err = engine.In("id", ids).Update(&model.OneCirclesGreenEnergySignIn{
				IsCompleted: 1,
			})
		}
		page++
		time.Sleep(time.Millisecond * time.Duration(rand.Intn(1000)))
	}

	return
}

func HandleSettlementSignInGreenEnergy(engine *xorm.Engine, masterId string, id int64, uid int) (err error) {
	//1、查找 `one_circles_green_energy_basic_setting` 基础设置
	oneCirclesGreenEnergyBasicSetting, err := db.OneCirclesGreenEnergyBasicSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}
	var teamRewardSetting *md2.TeamRewardSettingStruct
	err = json.Unmarshal([]byte(oneCirclesGreenEnergyBasicSetting.TeamReward), &teamRewardSetting)
	if err != nil {
		return
	}
	if teamRewardSetting.RewardDecrementValue == "" || teamRewardSetting.RewardEndValue == "" || teamRewardSetting.MemberSelfIsOpenGetTeamReward == "" || teamRewardSetting.OneRoundDuration == "" {
		err = errors.New("团队奖励设置未完全！")
		return
	}
	var oneRoundDuration = zhios_order_relate_utils.StrToInt(teamRewardSetting.OneRoundDuration)
	var rewardDecrement = zhios_order_relate_utils.StrToFloat64(teamRewardSetting.RewardDecrementValue) / 100 //递减百分比
	var rewardEndValue = zhios_order_relate_utils.StrToFloat64(teamRewardSetting.RewardEndValue)              //奖励结束值
	fmt.Println("rewardDecrement>>>>>>>>>>>>", rewardDecrement)
	fmt.Println("rewardEndValue>>>>>>>>>>>>", rewardEndValue)

	//2、获取当前签到收益
	err, reward := CalcNowSignInGreenEnergy(engine, oneCirclesGreenEnergyBasicSetting)
	if err != nil {
		return err
	}
	rewardValue := zhios_order_relate_utils.Float64ToStrPrec8(zhios_order_relate_utils.StrToFloat64(reward) * float64(oneRoundDuration) * 60 * 60)

	//3、统计签到结束的用户数据
	session := engine.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()

	var reduceTotalGreenEnergy float64
	//4.2给相应的用户加上个人的绿色积分(可用数量)
	err = DealUserCoin(session, md.DealUserCoinReq{
		Kind:         "add",
		Mid:          masterId,
		Title:        md.OneCirclesGreenEnergySignInSettlementPersonalReward,
		TransferType: md.OneCirclesGreenEnergySignInSettlementPersonalRewardForUserVirtualCoinFlow,
		OrdId:        "",
		CoinId:       oneCirclesGreenEnergyBasicSetting.PersonGreenEnergyCoinId,
		Uid:          uid,
		Amount:       zhios_order_relate_utils.StrToFloat64(rewardValue),
	})
	if err != nil {
		_ = session.Rollback()
		fmt.Println("err:::::2222", err)
		return err
	}
	reduceTotalGreenEnergy += zhios_order_relate_utils.StrToFloat64(rewardValue)

	//4.2给相应的上级用户加上团队奖励的绿色积分(结算数量)
	relates, err1 := db.DbsUserRelate(engine, uid, 0)
	if err1 != nil {
		_ = session.Rollback()
		fmt.Println("err:::::3333", err1)
		return err1
	}
	var parentRewardValue = zhios_order_relate_utils.StrToFloat64(rewardValue)
	if relates != nil {
		for _, relate := range *relates {
			if parentRewardValue <= rewardEndValue {
				break
			}
			parentRewardValue = parentRewardValue * rewardDecrement
			//TODO::判断是否活跃
			var m model.OneCirclesGreenEnergySignIn
			has, err3333333 := engine.Where("uid =?", relate.ParentUid).And("end_time >=?", time.Now().Format("2006-01-02 15:04:05")).Get(&m)
			if err3333333 != nil {
				_ = session.Rollback()
				fmt.Println("err:::::3333333", err3333333)
				return err3333333
			}
			if !has {
				//不活跃不需要奖励
				continue
			}
			err = DealUserCoin(session, md.DealUserCoinReq{
				Kind:         "add",
				Mid:          masterId,
				Title:        md.OneCirclesGreenEnergySignInSettlementTeamReward,
				TransferType: md.OneCirclesGreenEnergySignInSettlementTeamRewardForUserVirtualCoinFlow,
				OrdId:        "",
				CoinId:       oneCirclesGreenEnergyBasicSetting.TeamGreenEnergyCoinId,
				Uid:          relate.ParentUid,
				Amount:       parentRewardValue,
			})
			if err != nil {
				_ = session.Rollback()
				fmt.Println("err:::::44444", err)
				return err
			}
			reduceTotalGreenEnergy += parentRewardValue
		}
	}

	//5、减少“活跃赠送” 中的绿色能量
	if reduceTotalGreenEnergy > 0 {
		err = DealAvailableGreenEnergyCoin(session, int(enum.SignInReward), reduceTotalGreenEnergy, 0, enum.SignInReward.String(), oneCirclesGreenEnergyBasicSetting, oneCirclesGreenEnergyBasicSetting.NowPrice)
		if err != nil {
			_ = session.Rollback()
			fmt.Println("err:::::55555", err)
			return err
		}
	}

	if err != nil {
		_ = session.Rollback()
		fmt.Println("err:::::1111", err)
		return
	}

	err = session.Commit()
	if err != nil {
		_ = session.Rollback()
		return errors.New("事务提交失败")
	}

	return
}

// CalcNowSignInGreenEnergy 计算当前签到拿多少绿色能量/秒
func CalcNowSignInGreenEnergy(engine *xorm.Engine, oneCirclesGreenEnergyBasicSetting *model.OneCirclesGreenEnergyBasicSetting) (err error, rewardValue string) {
	if oneCirclesGreenEnergyBasicSetting == nil {
		//1、查找 `one_circles_green_energy_basic_setting` 基础设置
		oneCirclesGreenEnergyBasicSetting, err = db.OneCirclesGreenEnergyBasicSettingGetOneByParams(engine, map[string]interface{}{
			"key":   "is_open",
			"value": 1,
		})
		if err != nil {
			return
		}
	}

	var signInRewards []*md2.SignInRewardStruct
	err = json.Unmarshal([]byte(oneCirclesGreenEnergyBasicSetting.SignInReward), &signInRewards)
	if err != nil {
		return
	}
	if len(signInRewards) == 0 {
		err = errors.New("未设置签到奖励！")
		return
	}

	//2、统计全网用户数
	sqlStr := "SELECT COUNT(*) AS total FROM user "
	nativeString, _ := db.QueryNativeString(engine, sqlStr)
	userCount := zhios_order_relate_utils.StrToInt64(nativeString[0]["total"])

	for _, v := range signInRewards {
		if zhios_order_relate_utils.StrToInt64(v.VipMemberEndNums) >= userCount && userCount >= zhios_order_relate_utils.StrToInt64(v.VipMemberStartNums) {
			//rewardValue = zhios_order_relate_utils.Float64ToStrPrec8(zhios_order_relate_utils.StrToFloat64(v.RewardValue) * float64(oneRoundDuration) * 60 * 60)
			rewardValue = v.RewardValue
		}
	}
	return
}

// CalcTodayGreenEnergyPriceRises 计算绿色能量今日价格涨幅
func CalcTodayGreenEnergyPriceRises(engine *xorm.Engine, oneCirclesGreenEnergyBasicSetting *model.OneCirclesGreenEnergyBasicSetting) (err error, rises float64, isRises bool) {
	now := time.Now()
	if oneCirclesGreenEnergyBasicSetting == nil {
		//1、查找 `one_circles_green_energy_basic_setting` 基础设置
		oneCirclesGreenEnergyBasicSetting, err = db.OneCirclesGreenEnergyBasicSettingGetOneByParams(engine, map[string]interface{}{
			"key":   "is_open",
			"value": 1,
		})
		if err != nil {
			return
		}
	}

	//2、查找昨日价格
	var m model.OneCirclesGreenEnergyPrice
	if has, err1 := engine.Where("date =?", now.AddDate(0, 0, -1).Format("2006-01-02")).And("hour = 23").Get(&m); err1 != nil || has == false {
		err = err1
		return
	}
	yesterdayPrice, _ := decimal.NewFromString(m.Price)
	todayPrice, _ := decimal.NewFromString(oneCirclesGreenEnergyBasicSetting.NowPrice)
	if todayPrice.GreaterThanOrEqual(yesterdayPrice) {
		isRises = true
		rises, _ = todayPrice.Sub(yesterdayPrice).Div(yesterdayPrice).Float64()
	} else {
		rises, _ = yesterdayPrice.Sub(todayPrice).Div(todayPrice).Float64()
	}
	return
}
