package svc

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/lib/comm_plan"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/jinzhu/copier"
	"github.com/shopspring/decimal"
	"xorm.io/xorm"
)

// getRewardCommission is 获取制度后的佣金
// 返回：单个佣金、层级佣金、错误
func GetRewardCommission(engine *xorm.Engine, rmd *md.CommissionParam, isShare bool, userId, provider, masterId string, returnAllLevel bool, extraData map[string]string, commArr map[string]string, rewardOpts map[string]*model.PlanReward, commissionOpts map[int]*model.PlanCommission, virtualCoinMoneyRate map[int]string) (float64, float64, float64, float64, *comm_plan.LvUser, error) {
	virType := commArr["virType"]
	if virType == "commission" {
		rmd.PaidPrice = rmd.Commission
		if zhios_order_relate_utils.InArr(provider, []string{md.PVD_TB, md.PVD_TM, md.PVD_PDD, md.PVD_SN, md.PVD_KL, md.PVD_JD, md.PVD_JDOwn, md.PVD_VIP}) == false {
			rmd.GoodsPrice = rmd.Commission
		}
	}
	if rmd.Oid == "" {
		rmd.Oid = "1"
	}
	var (
		err       error
		uid       = 0
		level     = 0
		userLevel = 0
		newLevel  = 0
		minLevel  = 0
	)
	user, _ := db.UserFindByID(engine, userId)
	if user != nil {
		uid = user.Uid
		level = user.Level
		newLevel = user.Level
		userLevel = user.Level
	}

	minLevelList, _ := db.UserLevelInIDescByWeightLowWithOne(engine)
	if minLevelList != nil {
		minLevel = minLevelList.Id
		if uid == 0 {
			level = minLevel
			newLevel = minLevel
			userLevel = minLevel
		}
	}
	// 获取抽成方案
	newProvider := provider
	if newProvider == md.PVD_TM { //抽成方案只有淘宝的要替换回来
		newProvider = md.PVD_TB
	}
	if newProvider == "userlvup" {
		newProvider = "user_level_up"
	}
	if newProvider == md.PVD_MALL_GOODS_USER_LV || newProvider == "user_level_up" { //升级礼包按升级前等级计算佣金
		level = zhios_order_relate_utils.StrToInt(extraData["old_lv"])
		newLevel = zhios_order_relate_utils.StrToInt(extraData["new_lv"])
	}
	if rmd.ShowLevel != "" {
		level = zhios_order_relate_utils.StrToInt(rmd.ShowLevel)
	}
	if extraData["show_level"] != "" {
		level = zhios_order_relate_utils.StrToInt(extraData["show_level"])
	}
	ownbuyReturnType := zhios_order_relate_utils.StrToInt(extraData["ownbuy_return_type"])
	cfg, err := GetPlanCfg(engine, newProvider, masterId, rewardOpts, commissionOpts, virtualCoinMoneyRate, rmd)
	if err != nil {
		return 0, 0, 0, 0, nil, err
	}
	if cfg == nil {
		return 0, 0, 0, 0, nil, errors.New("分佣方案未设置")
	}
	// 获取全部佣金
	com, price := getCommission(rmd, provider)
	fmt.Println(com)
	fmt.Println(price)
	comf := zhios_order_relate_utils.StrToFloat64(com)
	if zhios_order_relate_utils.InArr(cfg.Mode, []string{"lv_price_public_platoon", "lv_price", "lv_price_other", "lv_winery", "extend_price", "niubei_amount"}) && zhios_order_relate_utils.StrToFloat64(rmd.OldPrice) > 0 { //价格为基数
		comf = zhios_order_relate_utils.StrToFloat64(rmd.OldPrice)
	}
	if cfg.Mode == "public_platoon" {
		//公排
		//佣金扣除抽成后
		fee, _, pvdFee, sysFee := CommFee(comf, cfg, "commission", rmd)
		////积分扣除抽成后
		integralFee, _, _, _ := CommFee(zhios_order_relate_utils.StrToFloat64(price), cfg, "integral", rmd)
		//调用公排制度

		userPublicPlatoonSetting, err := db.UserPublicPlatoonSettingGetOneByParams(engine, map[string]interface{}{
			"key":   "is_open",
			"value": 1,
		})
		if err != nil || userPublicPlatoonSetting == nil {
			return 0, 0, 0, 0, nil, nil
		}
		var reward = &md.PublicPlatoonRelateCommissionResp{}
		if userPublicPlatoonSetting.IsUseSmallPublicPlatoon == 1 { //小公排
			var PublicPlatoonRelateCommissionReqList = []*md.SmallPublicPlatoonRelateCommissionReq{
				{
					Pvd:             newProvider,
					Uid:             zhios_order_relate_utils.IntToStr(uid),
					UserLevel:       zhios_order_relate_utils.IntToStr(level),
					PendingAmount:   zhios_order_relate_utils.Float64ToStr(fee),
					PendingIntegral: zhios_order_relate_utils.Float64ToStr(integralFee),
					Oid:             rmd.Oid,
				},
			}
			var order = make(map[string]*md.SmallPublicPlatoonRelateCommissionResp)
			if rmd.IsGoods == "1" {
				order, err = BatchSmallGetPublicPlatoonRelateByGoods(engine, masterId, PublicPlatoonRelateCommissionReqList)
			} else {
				order, err = BatchSmallGetPublicPlatoonRelateCommissionLevel(engine, masterId, PublicPlatoonRelateCommissionReqList)
			}
			if err != nil || order[rmd.Oid] == nil {
				return 0, 0, 0, 0, nil, nil
			}
			rewardSmall := order[rmd.Oid]
			err = copier.Copy(reward, &rewardSmall)
			if err != nil || reward == nil {
				return 0, 0, 0, 0, nil, nil
			}
		} else {
			var PublicPlatoonRelateCommissionReqList = []*md.PublicPlatoonRelateCommissionReq{
				{
					Pvd:             newProvider,
					Uid:             zhios_order_relate_utils.IntToStr(uid),
					UserLevel:       zhios_order_relate_utils.IntToStr(level),
					PendingAmount:   zhios_order_relate_utils.Float64ToStr(fee),
					PendingIntegral: zhios_order_relate_utils.Float64ToStr(integralFee),
					Oid:             rmd.Oid,
				},
			}
			var order = make(map[string]*md.PublicPlatoonRelateCommissionResp)
			if rmd.IsGoods == "1" {
				order, err = BatchGetPublicPlatoonRelateCommissionByGoods(engine, masterId, PublicPlatoonRelateCommissionReqList)
			} else {
				order, err = BatchGetPublicPlatoonRelateCommissionByOrderLevel(engine, masterId, PublicPlatoonRelateCommissionReqList)
			}
			if err != nil || order[rmd.Oid] == nil {
				return 0, 0, 0, 0, nil, nil
			}
			reward = order[rmd.Oid]
		}
		profit := zhios_order_relate_utils.StrToFloat64(reward.SelfRateList["0"])
		var ulink = &comm_plan.LvUser{}
		if zhios_order_relate_utils.StrToInt(reward.CommonWealthBenefitUid) != zhios_order_relate_utils.StrToInt(reward.DirectPushBenefitUid) {
			//共富收益处理
			ulink = getRewardLink(reward.CommonWealthBenefitUid, 2, reward.CommonWealthSystem, ulink, "public_platoon", cfg)
			//直推收益处理
			ulink = getRewardLink(reward.DirectPushBenefitUid, 1, reward.DirectPush, ulink, "extend", cfg)
		} else { //共富 直推都是同一个用户的处理
			ulink = getRewardLinkCommWithExtend(reward.CommonWealthBenefitUid, 3, reward.CommonWealthSystem, reward.DirectPush, ulink, cfg)
		}
		//自购
		ulink = getRewardLink(reward.Uid, 0, reward.SelfRateList, ulink, "own", cfg)
		if zhios_order_relate_utils.StrToFloat64(commArr["integralRewardMultiple"]) > 0 {
			for k, v := range ulink.OwnSubsidyFeeList {
				ulink.OwnSubsidyFeeList[k] = v * zhios_order_relate_utils.StrToFloat64(commArr["integralRewardMultiple"])
			}
			for k, v := range ulink.ProfitList {
				ulink.ProfitList[k].Val = v.Val * zhios_order_relate_utils.StrToFloat64(commArr["integralRewardMultiple"])
			}
		}
		comm(commArr, ulink)
		return profit, pvdFee, sysFee, 0, ulink, nil
	}
	if cfg.Mode == "lv_integral_release" { //
		price = rmd.OldPrice
		pvdFee, sysFee, _, ulink, err := IntegralRelease(engine, uid, newLevel, returnAllLevel, rmd.IsShowExtend, newProvider, user, comf, price, cfg, rmd, commArr, virtualCoinMoneyRate)
		if err != nil {
			return 0, 0, 0, 0, nil, nil
		}
		if ulink == nil {
			return 0, 0, 0, 0, nil, nil
		}
		comm(commArr, ulink)
		return ulink.Profit, pvdFee, sysFee, 0, ulink, nil
	}

	if cfg.Mode == "extend_price" { //直推奖励
		var ulink = &comm_plan.LvUser{}
		if rmd.IsGoods == "1" { //自购没奖励,就不计算了
			return 0, 0, 0, 0, ulink, nil
		}
		if newProvider == md.PVD_REGIONAL_AGENT_PAY {
			userLevel = zhios_order_relate_utils.StrToInt(extraData["agent_lv"])
		}
		//佣金扣除抽成后
		fee, _, pvdFee, sysFee := CommFee(comf, cfg, "commission", rmd)
		////积分扣除抽成后
		integralFee, _, _, _ := CommFee(zhios_order_relate_utils.StrToFloat64(price), cfg, "integral", rmd)
		var param = map[string]string{
			"commission": zhios_order_relate_utils.Float64ToStr(fee),
			"integral":   zhios_order_relate_utils.Float64ToStr(integralFee),
			"level":      zhios_order_relate_utils.IntToStr(userLevel),
			"uid":        zhios_order_relate_utils.IntToStr(uid),
			"source":     newProvider,
		}
		reward, parentUid := ExtendPriceReward(engine, param, cfg)
		for k, v := range reward {
			if zhios_order_relate_utils.StrToFloat64(v) < 0 {
				reward[k] = "0"
			}
		}
		//直推收益处理
		if parentUid > 0 {
			ulink = getRewardLink(zhios_order_relate_utils.IntToStr(parentUid), 1, reward, ulink, "extend", cfg)
		}
		//自购
		ulink = getRewardLink(zhios_order_relate_utils.IntToStr(uid), 0, map[string]string{}, ulink, "own", cfg)
		return 0, pvdFee, sysFee, 0, ulink, nil
	}
	//常规级差
	var userRelationship *[]md.UserRelation
	if returnAllLevel {
		userRelationship, err = UserRelativeNetwork(engine, uid, 0)
	} else if rmd.IsShowExtend == "1" {
		//userRelationship, err = UserRelativeNetwork(engine, uid, 1)
		userRelationship = &[]md.UserRelation{
			{
				CurUid: user.Uid,
				Uid:    user.Uid,
				Diff:   1,
				Level:  user.Level,
			},
		}
		uid = 0
		level = 0
	} else {
		userRelationship = nil
	}
	if cfg.Mode == "niubei_commission" || cfg.Mode == "niubei_amount" { //
		price = zhios_order_relate_utils.Float64ToStrByPrec(comf, 8)
		if cfg.Mode == "niubei_amount" {
			price = rmd.OldPrice
		}
		pvdFee, sysFee, subsidyFee, ulink, err := NiuBeiCalcCommission(uid, level, 0, ownbuyReturnType, comf, zhios_order_relate_utils.StrToFloat64(price), isShare, cfg, userRelationship, newProvider, newLevel, engine, rmd.IsGoods, rmd)
		if err != nil {
			return 0, 0, 0, 0, nil, nil
		}
		if ulink == nil {
			return 0, 0, 0, 0, nil, nil
		}
		comm(commArr, ulink)
		return ulink.Profit, pvdFee, sysFee, subsidyFee, ulink, nil
	}
	// userRelationship == nil 是只返回第一层 即用户自己的
	rmd.MinLevel = minLevel
	pvdFee, sysFee, subsidyFee, ulink, err := CalcCommission(uid, level, 0, ownbuyReturnType, comf, zhios_order_relate_utils.StrToFloat64(price), isShare, cfg, userRelationship, newProvider, newLevel, engine, rmd.IsGoods, rmd)
	if err != nil {
		return 0, 0, 0, 0, nil, nil
	}
	if ulink == nil {
		return 0, 0, 0, 0, nil, nil
	}
	if cfg.Mode == "lv_price_public_platoon" || cfg.Mode == "lv_commission_public_platoon" {
		ulink = public(engine, ulink, cfg, newProvider, uid, comf, price, rmd)
	}
	comm(commArr, ulink)
	return ulink.Profit, pvdFee, sysFee, subsidyFee, ulink, nil
}
func public(engine *xorm.Engine, ulinkParent *comm_plan.LvUser, cfg *comm_plan.PlanOpt, newProvider string, uid int, comf float64, price string, rmd *md.CommissionParam) *comm_plan.LvUser {
	//佣金扣除抽成后
	fee, _, _, _ := CommFee(comf, cfg, "commission", rmd)
	////积分扣除抽成后
	integralFee, _, _, _ := CommFee(zhios_order_relate_utils.StrToFloat64(price), cfg, "integral", rmd)
	//调用公排制度

	userPublicPlatoonSetting, err := db.UserPublicPlatoonSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil || userPublicPlatoonSetting == nil {
		return ulinkParent
	}
	var nowBenefitUid string
	if userPublicPlatoonSetting.IsUseSmallPublicPlatoon == 1 { //小公排
		var PublicPlatoonRelateCommissionReqList = &md.SmallPublicPlatoonRelateCommissionReq{
			Pvd:             newProvider,
			Uid:             zhios_order_relate_utils.IntToStr(uid),
			PendingAmount:   zhios_order_relate_utils.Float64ToStr(fee),
			PendingIntegral: zhios_order_relate_utils.Float64ToStr(integralFee),
			Oid:             rmd.Oid,
		}
		if rmd.IsGoods != "1" {
			nowBenefitUid, _, err = benefitUid(engine, PublicPlatoonRelateCommissionReqList)

		}
	} else {
		var PublicPlatoonRelateCommissionReqList = &md.PublicPlatoonRelateCommissionReq{
			Pvd:             newProvider,
			Uid:             zhios_order_relate_utils.IntToStr(uid),
			PendingAmount:   zhios_order_relate_utils.Float64ToStr(fee),
			PendingIntegral: zhios_order_relate_utils.Float64ToStr(integralFee),
			Oid:             rmd.Oid,
		}
		if rmd.IsGoods != "1" {
			nowBenefitUid, _, err = benefitUidByBig(engine, PublicPlatoonRelateCommissionReqList)
		}
	}
	if err == nil && nowBenefitUid != "" {
		user, _ := db.UserFindByID(engine, nowBenefitUid)
		if user == nil {
			return ulinkParent
		}
		var commonWealthSystem = make(map[string]string)
		grade := cfg.UserRate[user.Level]
		if grade != nil {
			for _, coinId := range grade.ReturnType {
				var pendingAmount = zhios_order_relate_utils.Float64ToStr(fee)
				if zhios_order_relate_utils.StrToInt(coinId) > 0 { //积分更换基数
					pendingAmount = zhios_order_relate_utils.Float64ToStr(integralFee)
				}
				CommonWealthSystems, ok := grade.CommonWealthSystem.(map[string]string)
				if ok {
					commonWealthSystem[coinId] = zhios_order_relate_utils.Float64ToStr(zhios_order_relate_utils.StrToFloat64(CommonWealthSystems[coinId]) * zhios_order_relate_utils.StrToFloat64(pendingAmount) / 100)
				}
			}
		}
		//共富收益处理
		ulink := getRewardLinkNew(nowBenefitUid, 2, commonWealthSystem, &comm_plan.LvUser{}, "public_platoon", cfg)
		return commMax(ulinkParent, ulink)
	}
	return ulinkParent

}
func comm(commArr map[string]string, ulink *comm_plan.LvUser) *comm_plan.LvUser {
	if zhios_order_relate_utils.StrToFloat64(commArr["integralRewardMultiple"]) > 0 {
		for k, v := range ulink.OwnSubsidyFeeList {
			if k == commArr["integralCoinId"] && zhios_order_relate_utils.StrToInt(k) > 0 {
				ulink.OwnSubsidyFeeList[k] = v * zhios_order_relate_utils.StrToFloat64(commArr["integralRewardMultiple"])
			}
		}
		for k, v := range ulink.ProfitList {
			if v.Cid == commArr["integralCoinId"] && zhios_order_relate_utils.StrToInt(v.Cid) > 0 {
				ulink.ProfitList[k].Val = v.Val * zhios_order_relate_utils.StrToFloat64(commArr["integralRewardMultiple"])
			}
		}
		if ulink.ParentUser != nil {
			ulink.ParentUser = comm(commArr, ulink.ParentUser)
		}
	}
	return ulink
}
func commMax(ulinkParent, ulink *comm_plan.LvUser) *comm_plan.LvUser {
	if ulinkParent.ParentUser != nil {
		ulinkParent.ParentUser = commMax(ulinkParent.ParentUser, ulink)
	} else {
		ulinkParent.ParentUser = ulink
	}
	return ulinkParent
}

func getRewardLink(uid string, diff int, reward map[string]string, ulinkParent *comm_plan.LvUser, types string, cfg *comm_plan.PlanOpt) *comm_plan.LvUser {
	var SubsidyFee = zhios_order_relate_utils.StrToFloat64(reward["0"])
	var SubsidyFeeList = make([]*comm_plan.VirtualCoinCommission, 0)
	for k, v := range reward {
		coinId := zhios_order_relate_utils.StrToInt(k)
		ratio := cfg.VirtualCoinMoneyRatioList[coinId]
		rate := zhios_order_relate_utils.StrToFloat64(ratio)
		var tmp = &comm_plan.VirtualCoinCommission{
			Cid: k,
			Val: zhios_order_relate_utils.StrToFloat64(v) * rate,
		}
		SubsidyFeeList = append(SubsidyFeeList, tmp)
	}

	//组装数据
	var ulink = &comm_plan.LvUser{
		Uid:        zhios_order_relate_utils.StrToInt(uid),
		Diff:       diff,
		ParentUser: ulinkParent,
	}
	if types == "public_platoon" {
		ulink.SubsidyFee = SubsidyFee
		ulink.SubsidyFeeList = SubsidyFeeList
	} else {
		ulink.Profit = SubsidyFee
		ulink.ProfitList = SubsidyFeeList
	}
	return ulink
}
func getRewardLinkNew(uid string, diff int, reward map[string]string, ulinkParent *comm_plan.LvUser, types string, cfg *comm_plan.PlanOpt) *comm_plan.LvUser {
	var SubsidyFee = zhios_order_relate_utils.StrToFloat64(reward["0"])
	var SubsidyFeeList = make([]*comm_plan.VirtualCoinCommission, 0)
	for k, v := range reward {
		coinId := zhios_order_relate_utils.StrToInt(k)
		ratio := cfg.VirtualCoinMoneyRatioList[coinId]
		rate := zhios_order_relate_utils.StrToFloat64(ratio)
		var tmp = &comm_plan.VirtualCoinCommission{
			Cid: k,
			Val: zhios_order_relate_utils.StrToFloat64(v) * rate,
		}
		SubsidyFeeList = append(SubsidyFeeList, tmp)
	}

	//组装数据
	var ulink = &comm_plan.LvUser{
		Uid:        zhios_order_relate_utils.StrToInt(uid),
		Diff:       diff,
		ParentUser: ulinkParent,
		ExtendType: 5,
	}
	ulink.Profit = SubsidyFee
	ulink.ProfitList = SubsidyFeeList
	return ulink
}

func getRewardLinkCommWithExtend(uid string, diff int, reward map[string]string, rewardExtend map[string]string, ulinkParent *comm_plan.LvUser, cfg *comm_plan.PlanOpt) *comm_plan.LvUser {
	var SubsidyFee = zhios_order_relate_utils.StrToFloat64(reward["0"])
	var SubsidyFeeList = make([]*comm_plan.VirtualCoinCommission, 0)
	for k, v := range reward {
		coinId := zhios_order_relate_utils.StrToInt(k)
		ratio := cfg.VirtualCoinMoneyRatioList[coinId]
		rate := zhios_order_relate_utils.StrToFloat64(ratio)
		var tmp = &comm_plan.VirtualCoinCommission{
			Cid: k,
			Val: zhios_order_relate_utils.StrToFloat64(v) * rate,
		}
		SubsidyFeeList = append(SubsidyFeeList, tmp)
	}
	var SubsidyFeeExtend = zhios_order_relate_utils.StrToFloat64(rewardExtend["0"])
	var SubsidyFeeListExtend = make([]*comm_plan.VirtualCoinCommission, 0)
	for k, v := range rewardExtend {
		var rate float64 = 1
		coinId := zhios_order_relate_utils.StrToInt(k)
		if coinId > 0 {
			ratio := cfg.VirtualCoinMoneyRatioList[coinId]
			rate = zhios_order_relate_utils.StrToFloat64(ratio)
		}
		var tmp = &comm_plan.VirtualCoinCommission{
			Cid: k,
			Val: zhios_order_relate_utils.StrToFloat64(v) * rate,
		}
		SubsidyFeeListExtend = append(SubsidyFeeListExtend, tmp)
	}
	//组装数据
	var ulink = &comm_plan.LvUser{
		Uid:        zhios_order_relate_utils.StrToInt(uid),
		Diff:       diff,
		ParentUser: ulinkParent,
	}
	ulink.SubsidyFee = SubsidyFee
	ulink.SubsidyFeeList = SubsidyFeeList
	ulink.Profit = SubsidyFeeExtend
	ulink.ProfitList = SubsidyFeeListExtend
	return ulink
}
func getCommission(d *md.CommissionParam, provider string) (string, string) {

	switch provider {
	case md.PVD_TB, md.PVD_TM:
		if zhios_order_relate_utils.StrToFloat64(d.GoodsPrice) == 0 {
			//如果价格没有直接返回佣金
			return d.Commission, d.PaidPrice
		}
		currentPrice := currentPrice(d, provider)
		currentPriceF := zhios_order_relate_utils.StrToFloat64(currentPrice)
		commissionRateF := zhios_order_relate_utils.StrToFloat64(d.CommissionRate) / 100
		com := currentPriceF * commissionRateF
		return zhios_order_relate_utils.Float64ToStr(com), currentPrice
	case md.PVD_JD, md.PVD_JDOwn:
		if zhios_order_relate_utils.StrToFloat64(d.GoodsPrice) == 0 {
			//如果价格没有直接返回佣金
			return d.Commission, d.PaidPrice
		}
		currentPrice := currentPrice(d, provider)
		currentPriceF := zhios_order_relate_utils.StrToFloat64(currentPrice)
		commissionRateF := zhios_order_relate_utils.StrToFloat64(d.CommissionRate) / 100
		fmt.Println("当前劵后价格:", currentPriceF, "佣金比例：", commissionRateF)
		com := currentPriceF * commissionRateF
		return zhios_order_relate_utils.Float64ToStr(com), currentPrice
	case md.PVD_PDD:
		if zhios_order_relate_utils.StrToFloat64(d.GoodsPrice) == 0 {
			//如果价格没有直接返回佣金
			return d.Commission, d.PaidPrice
		}
		currentPrice := currentPrice(d, provider)
		currentPriceF := zhios_order_relate_utils.StrToFloat64(currentPrice)
		commissionRateF := zhios_order_relate_utils.StrToFloat64(d.Commission) / 100
		com := currentPriceF * commissionRateF
		return zhios_order_relate_utils.Float64ToStr(com), currentPrice
	case md.PVD_VIP:
		if zhios_order_relate_utils.StrToFloat64(d.GoodsPrice) == 0 {
			//如果价格没有直接返回佣金
			return d.Commission, d.PaidPrice
		}
		currentPrice := currentPrice(d, provider)
		currentPriceF := zhios_order_relate_utils.StrToFloat64(currentPrice)
		commissionRateF := zhios_order_relate_utils.StrToFloat64(d.CommissionRate) / 100
		com := currentPriceF * commissionRateF
		return zhios_order_relate_utils.Float64ToStr(com), currentPrice
	case md.PVD_SN:
		if zhios_order_relate_utils.StrToFloat64(d.GoodsPrice) == 0 {
			//如果价格没有直接返回佣金
			return d.Commission, d.PaidPrice
		}
		currentPrice := currentPrice(d, provider)
		currentPriceF := zhios_order_relate_utils.StrToFloat64(currentPrice)
		commissionRateF := zhios_order_relate_utils.StrToFloat64(d.CommissionRate) / 100
		fmt.Println("当前劵后价格:", currentPriceF, "佣金比例：", commissionRateF)
		com := currentPriceF * commissionRateF
		return zhios_order_relate_utils.Float64ToStr(com), currentPrice
	case md.PVD_KL:
		if zhios_order_relate_utils.StrToFloat64(d.GoodsPrice) == 0 {
			//如果价格没有直接返回佣金
			return d.Commission, d.PaidPrice
		}
		currentPrice := currentPrice(d, provider)
		currentPriceF := zhios_order_relate_utils.StrToFloat64(currentPrice)
		commissionRateF := zhios_order_relate_utils.StrToFloat64(d.CommissionRate) // TODO 先不除于100
		com := currentPriceF * commissionRateF
		return zhios_order_relate_utils.Float64ToStr(com), currentPrice
	default:
		if zhios_order_relate_utils.StrToFloat64(d.GoodsPrice) == 0 {
			//如果价格没有直接返回佣金
			return d.Commission, d.Commission
		}
		return d.Commission, d.GoodsPrice // 直接传过来就是总佣金了
	}
}

func currentPrice(m *md.CommissionParam, pvd string) string {
	switch pvd {

	case md.PVD_TB, md.PVD_TM:
		if m.CouponPrice != "" {
			price := zhios_order_relate_utils.StrToFloat64(m.WlGoodsPrice) - zhios_order_relate_utils.StrToFloat64(m.CouponPrice)
			return zhios_order_relate_utils.Float64ToStr(price)
		}
		return m.WlGoodsPrice
	case md.PVD_JD, md.PVD_JDOwn:
		//fmt.Println(m.LowestCouponPrice, m.LowerPrice, m.WlGoodsPrice)
		if m.LowestCouponPrice == "" {
			if m.CouponPrice != "" {
				price := zhios_order_relate_utils.StrToFloat64(m.WlGoodsPrice) - zhios_order_relate_utils.StrToFloat64(m.CouponPrice)
				return zhios_order_relate_utils.Float64ToStr(price)
			}
			return m.LowerPrice
		}
		return m.LowestCouponPrice
	case md.PVD_PDD:
		if m.CouponPrice != "" {
			price := zhios_order_relate_utils.StrToFloat64(m.MinGroupPrice) - zhios_order_relate_utils.StrToFloat64(m.CouponPrice)
			return zhios_order_relate_utils.Float64ToStr(price)
		}
		return m.MinGroupPrice
	case md.PVD_VIP:
		if m.CouponPrice != "" {
			price := zhios_order_relate_utils.StrToFloat64(m.GoodsPrice) - zhios_order_relate_utils.StrToFloat64(m.CouponPrice)
			return zhios_order_relate_utils.Float64ToStr(price)
		}
		return m.GoodsPrice
	case md.PVD_SN:
		if m.CouponPrice != "" {
			price := zhios_order_relate_utils.StrToFloat64(m.WlGoodsPrice) - zhios_order_relate_utils.StrToFloat64(m.CouponPrice)
			if price < 0 {
				return m.WlGoodsPrice
			}
			return zhios_order_relate_utils.Float64ToStr(price)
		}
		return m.WlGoodsPrice
	case md.PVD_KL:
		if m.CouponPrice != "" {
			price := zhios_order_relate_utils.StrToFloat64(m.WlGoodsPrice) - zhios_order_relate_utils.StrToFloat64(m.CouponPrice)
			return zhios_order_relate_utils.Float64ToStr(price)
		}
		return m.WlGoodsPrice
	default:
		return ""
	}
}
func CommFee(fee float64, opt *comm_plan.PlanOpt, types string, rmd *md.CommissionParam) (float64, float64, float64, float64) {
	var feeFirst float64 = 1
	if types == "integral" && opt.IntegralOpen == 0 || opt.Mode == "lv_winery" { //积分抽成后台没开启不用扣
		return fee, feeFirst, 0, 0
	}
	isGoods := rmd.IsGoods
	if isGoods == "1" {
		orderBeforeRate := fee * opt.OrderBeforeRate
		orderBeforeRateFirst := feeFirst * opt.OrderBeforeRate
		fee1 := int64(fee*1e4) - int64(orderBeforeRate*1e4)
		feeFirst1 := int64(feeFirst*1e4) - int64(orderBeforeRateFirst*1e4)
		if opt.PointType == 1 { //四舍五入
			fee, _ = decimal.NewFromFloat(float64(fee1) / 10000).Round(2).Float64()
			feeFirst, _ = decimal.NewFromFloat(float64(feeFirst1) / 10000).Round(2).Float64()
		} else {
			fee = float64(int64(float64(fee1)/100)) / 100
			feeFirst = float64(int64(float64(feeFirst1)/100)) / 100
		}
	}
	pvdFee := fee * opt.PvdRate                     // 供应商联盟比例
	sysFee := fee * opt.SysRate                     // 平台比例
	regionFee := fee * opt.RegionRate               // 区域代理比例
	globalFee := fee * opt.GlobalRate               // 全球分红比例
	selfBuyGlobalFee := fee * opt.SelfBuyGlobalRate // 全球分红比例
	pushHandFee := fee * opt.PushHandRate
	merchantFee := fee * opt.MerchantRate
	newAgentFee := fee * opt.NewAgentRate
	regionSubFee := fee * opt.RegionSubRate

	// 剩余可分配的佣金
	total := int64(fee*1e4) - int64(pvdFee*1e4) - int64(sysFee*1e4) - int64(regionFee*1e4) - int64(regionSubFee*1e4) - int64(globalFee*1e4) - int64(selfBuyGlobalFee*1e4) - int64(pushHandFee*1e4) - int64(merchantFee*1e4) - int64(newAgentFee*1e4)
	for _, v := range opt.RateList {
		total -= int64(fee * (v / 100) * 1e4)
	}
	if zhios_order_relate_utils.StrToFloat64(rmd.DouyinBili) > 0 {
		total -= int64(zhios_order_relate_utils.StrToFloat64(rmd.DouyinBili) * 1e4)
	}
	fee = float64(total) / 1e4
	if fee < 0 {
		fee = 0
	}
	pvdFeeFirst := feeFirst * opt.PvdRate                     // 供应商联盟比例
	sysFeeFirst := feeFirst * opt.SysRate                     // 平台比例
	regionFeeFirst := feeFirst * opt.RegionRate               // 区域代理比例
	globalFeeFirst := feeFirst * opt.GlobalRate               // 全球分红比例
	selfBuyGlobalFeeFirst := feeFirst * opt.SelfBuyGlobalRate // 全球分红比例
	pushHandFeeFirst := feeFirst * opt.PushHandRate
	merchantFeeFirst := feeFirst * opt.MerchantRate
	newAgentFeeFirst := feeFirst * opt.NewAgentRate
	regionSubFeeFirst := feeFirst * opt.RegionSubRate

	// 剩余可分配的佣金
	totalFirst := int64(feeFirst*1e4) - int64(pvdFeeFirst*1e4) - int64(sysFeeFirst*1e4) - int64(regionFeeFirst*1e4) - int64(regionSubFeeFirst*1e4) - int64(globalFeeFirst*1e4) - int64(selfBuyGlobalFeeFirst*1e4) - int64(pushHandFeeFirst*1e4) - int64(merchantFeeFirst*1e4) - int64(newAgentFeeFirst*1e4)
	for _, v := range opt.RateList {
		totalFirst -= int64(feeFirst * (v / 100) * 1e4)
	}
	feeFirst = float64(totalFirst) / 1e4
	if feeFirst < 0 {
		feeFirst = 0
		//
	}

	return fee, feeFirst, pvdFee, sysFee
}

// 根据用户计算对应的手续费
//ownbuyReturnType 0返利 1不返利 (免单商品是淘礼金商品的时候用的)
//pvd 只是为了判断是会员升级的订单 可不传
func CalcCommission(uid, level, oldDiff, ownbuyReturnType int, fee, integralFee float64, isShare bool, opt *comm_plan.PlanOpt, userRelationShip *[]md.UserRelation, pvd string, newLevel int, eg *xorm.Engine, isGoods string, rmd *md.CommissionParam) (pvdFee, sysFee, subsidyFee float64, lvUser *comm_plan.LvUser, err error) {
	//佣金扣除抽成后
	var feeFirst float64 = 1
	var integralFeeFirst float64 = 1
	fee, feeFirst, pvdFee, sysFee = CommFee(fee, opt, "commission", rmd)
	//积分扣除抽成后
	var integralSysFee float64 = 0
	integralFee, integralFeeFirst, _, integralSysFee = CommFee(integralFee, opt, "integral", rmd)
	// 计算自购补贴比例
	subsidyFee = 0
	//如果没登录，要找出权重最低的那个
	if uid == 0 {
		lvList, err := db.UserLevelInIDescByWeightLow(eg)
		if err != nil {
			fmt.Println(err)
		}
		if lvList != nil {
			for _, v := range lvList {
				level = v.Id
			}
		}
	}
	if uid == 8 {
		fmt.Println("==================================这里1", level)
	}
	// 判断当前用户等级是否有分佣设置
	if _, ok := opt.UserRate[level]; !ok {
		if uid == 8 {
			fmt.Println("==================================这里2", zhios_order_relate_utils.SerializeStr(opt))
		}
		return 0, 0, 0, nil, errors.New("opt.UserRate[level] is nil uid=" + zhios_order_relate_utils.IntToStr(uid))
	}

	// 金额的补贴
	subsidyFeeList := getOwnSubsidy(opt, level, isShare, fee, sysFee, integralFee)
	subsidyFee1, ok := subsidyFeeList["0"]
	if ok {
		subsidyFee = subsidyFee1
	}
	// 抖团服务费金额的补贴
	tikTokSubsidyFeeList := getTikTokOwnSubsidy(opt, level, isShare, rmd)

	// 获取用户的关系网
	//userRelationShip, err := UserRelativeNetwork(eg, uid)
	// 如果获取用户关系失败或者佣金方案不存在, 那么只计算当前用户的佣金, 平台佣金, 以及供应商方的佣金
	if _, ok := comm_plan.Fn[opt.Mode]; !ok || userRelationShip == nil || len(*userRelationShip) == 0 {

		// 获得自购的
		commission, profitBili, amountList, ratioList := comm_plan.CalReturnAmountAndRatio(level, ownbuyReturnType, 0, "own", fee, integralFee, opt)
		if opt.Mode == "lv_winery" {
			commission, profitBili, amountList, ratioList = comm_plan.CalReturnAmountAndRatioToWinery(level, fee, integralFee, opt)
		}
		if opt.Mode == "lv_ds_check" {
			commission, profitBili, amountList, ratioList = comm_plan.CalReturnAmountAndRatioDsOwn(level, ownbuyReturnType, 0, "own", fee, integralFee, opt)
		}
		if opt.Mode == "lv_winery_adv" {
			commission = 0
			amountList = make([]*comm_plan.VirtualCoinCommission, 0)
			ratioList = make([]*comm_plan.VirtualCoinCommission, 0)
			profitBili = 0
		}

		ratioListMap := convertList2Map(ratioList)
		for k, v := range amountList {
			amountList[k].Val = ratioListMap[v.Cid] * v.Val
		}
		//重新计算佣金
		return pvdFee, sysFee, subsidyFee, &comm_plan.LvUser{ProfitBili: profitBili, FirstFee: feeFirst, IntegralFirstFee: integralFeeFirst, TikTokOwnSubsidyFeeList: tikTokSubsidyFeeList, OwnSubsidyFeeList: subsidyFeeList, Uid: uid, Lv: level, OldDiff: oldDiff, Profit: commission, ProfitList: amountList, Diff: 0}, nil
	}

	//查出所有等级替换权重
	levelList, _ := db.UserLevlEgAll(eg)
	levelWeight := 0
	for _, v1 := range levelList {
		if v1.Id == level {
			levelWeight = v1.LevelWeight
		}
	}
	lvUser = &comm_plan.LvUser{MinLevel: rmd.MinLevel, TikTokOwnSubsidyFeeList: tikTokSubsidyFeeList, OwnSubsidyFeeList: subsidyFeeList, Uid: uid, Lv: level, Diff: 0, OldDiff: oldDiff, NewLv: newLevel, LevelWeight: levelWeight, OwnbuyReturnType: ownbuyReturnType}
	var node = lvUser
	//关系链处理
	for _, v := range *userRelationShip {
		if v.Uid == 0 {
			continue
		}
		for _, v1 := range levelList {
			levelId := v.Level
			if v1.Id == levelId {
				levelWeight = v1.LevelWeight
			}
		}
		node.ParentUser = &comm_plan.LvUser{Uid: v.Uid, Lv: v.Level, Diff: v.Diff, OldDiff: v.OldDiff, LevelWeight: levelWeight}
		node = node.ParentUser
	}
	levelWeight1 := 0
	level1 := 0
	lvWeight, _ := db.UserLevelInIDescByWeightLowWithOne(eg)
	if lvWeight != nil {
		levelWeight1 = lvWeight.LevelWeight
		level1 = lvWeight.Id
	}
	if err := comm_plan.Fn[opt.Mode](opt, fee, integralFee, lvUser, pvd, sysFee, integralSysFee, level1, levelWeight1, eg); err != nil {
		fmt.Println("方案有问题1")
		return 0, 0, 0, nil, err
	}
	if opt.Mode == "lv_winery" {
		commission, _, amountList, ratioList := comm_plan.CalReturnAmountAndRatioToWinery(level, fee, integralFee, opt)
		lvUser.Profit = commission     // 另外出来的佣金 兼容旧的
		lvUser.ProfitList = amountList // 各币种分佣
		lvUser.SubsidyFee = 0
		ratioListMap := convertList2Map(ratioList)
		for k, v := range lvUser.ProfitList {
			lvUser.ProfitList[k].Val = ratioListMap[v.Cid] * v.Val
		}
	}
	return pvdFee, sysFee, subsidyFee, lvUser, nil
}

// 根据用户计算对应的手续费
//ownbuyReturnType 0返利 1不返利 (免单商品是淘礼金商品的时候用的)
//pvd 只是为了判断是会员升级的订单 可不传
func NiuBeiCalcCommission(uid, level, oldDiff, ownbuyReturnType int, fee, integralFee float64, isShare bool, opt *comm_plan.PlanOpt, userRelationShip *[]md.UserRelation, pvd string, newLevel int, eg *xorm.Engine, isGoods string, rmd *md.CommissionParam) (pvdFee, sysFee, subsidyFee float64, lvUser *comm_plan.LvUser, err error) {
	//佣金扣除抽成后
	fee, _, pvdFee, sysFee = CommFee(fee, opt, "commission", rmd)
	//积分扣除抽成后
	var integralSysFee float64 = 0
	integralFee, _, _, integralSysFee = CommFee(integralFee, opt, "integral", rmd)
	integralFeeForPrice, _, _, _ := CommFee(zhios_order_relate_utils.StrToFloat64(rmd.OldPrice), opt, "integral", rmd)
	// 计算自购补贴比例
	subsidyFee = 0
	//如果没登录，要找出权重最低的那个
	if uid == 0 {
		lvList, err := db.UserLevelInIDescByWeightLow(eg)
		if err != nil {
			fmt.Println(err)
		}
		if lvList != nil {
			for _, v := range lvList {
				level = v.Id
			}
		}
	}
	// 判断当前用户等级是否有分佣设置
	if _, ok := opt.UserRate[level]; !ok {
		return 0, 0, 0, nil, errors.New("opt.UserRate[level] is nil uid=" + zhios_order_relate_utils.IntToStr(uid))
	}
	// 获得自购的
	commission, _, amountList, ratioList := comm_plan.NiuBeiCalReturnAmountAndRatio(eg, pvd, rmd.GoodsCommission, integralFeeForPrice, opt)
	ratioListMap := convertList2Map(ratioList)
	for k, v := range amountList {
		amountList[k].Val = ratioListMap[v.Cid] * v.Val
	}
	// 如果获取用户关系失败或者佣金方案不存在, 那么只计算当前用户的佣金, 平台佣金, 以及供应商方的佣金
	if _, ok := comm_plan.Fn[opt.Mode]; !ok || userRelationShip == nil || len(*userRelationShip) == 0 {
		//重新计算佣金
		return pvdFee, sysFee, subsidyFee, &comm_plan.LvUser{Uid: uid, Lv: level, OldDiff: oldDiff, Profit: commission, ProfitList: amountList, Diff: 0}, nil
	}

	//查出所有等级替换权重
	levelList, _ := db.UserLevlEgAll(eg)
	levelWeight := 0
	for _, v1 := range levelList {
		if v1.Id == level {
			levelWeight = v1.LevelWeight
		}
	}
	lvUser = &comm_plan.LvUser{Uid: uid, Lv: level, Diff: 0, OldDiff: oldDiff, NewLv: newLevel, LevelWeight: levelWeight, OwnbuyReturnType: ownbuyReturnType}
	var node = lvUser
	//关系链处理
	for _, v := range *userRelationShip {
		if v.Uid == 0 {
			continue
		}
		for _, v1 := range levelList {
			levelId := v.Level
			if v1.Id == levelId {
				levelWeight = v1.LevelWeight
			}
		}
		node.ParentUser = &comm_plan.LvUser{Uid: v.Uid, Lv: v.Level, Diff: v.Diff, OldDiff: v.OldDiff, LevelWeight: levelWeight}
		node = node.ParentUser
	}
	levelWeight1 := 0
	level1 := 0
	lvWeight, _ := db.UserLevelInIDescByWeightLowWithOne(eg)
	if lvWeight != nil {
		levelWeight1 = lvWeight.LevelWeight
		level1 = lvWeight.Id
	}
	if err := comm_plan.Fn[opt.Mode](opt, fee, integralFee, lvUser, pvd, sysFee, integralSysFee, level1, levelWeight1, eg); err != nil {
		fmt.Println("方案有问题1")
		return 0, 0, 0, nil, err
	}
	lvUser.Profit = commission
	lvUser.ProfitList = amountList
	return pvdFee, sysFee, subsidyFee, lvUser, nil
}
func getOwnSubsidy(opt *comm_plan.PlanOpt, level int, isShare bool, fee, sysFee, integralFee float64) map[string]float64 {
	var res = make(map[string]float64)
	if opt.UserRate[level].SubsidyEnable != 1 {
		return res
	}
	if len(opt.UserRate[level].SubsidyOwnBiliList) == 0 {
		var tmp = map[string]string{
			"coin_id":    "0",
			"self_bili":  zhios_order_relate_utils.Float64ToStrByPrec(opt.UserRate[level].SubsidySelfRate*100, 6),
			"share_bili": zhios_order_relate_utils.Float64ToStrByPrec(opt.UserRate[level].SubsidyShareRate*100, 6),
		}
		opt.UserRate[level].SubsidyOwnBiliList = append(opt.UserRate[level].SubsidyOwnBiliList, tmp)
	}
	for _, v := range opt.UserRate[level].SubsidyOwnBiliList {
		var newFee = fee
		if v["coin_id"] != "0" { //积分的
			newFee = integralFee
		}
		if v["coin_id"] == "0" && opt.UserRate[level].SubsidyMode == 1 { //  按佣金计算补贴
			newFee = sysFee
		}
		var subsidyFee float64 = 0
		subsidyFee = newFee * (zhios_order_relate_utils.StrToFloat64(v["self_bili"]) / 100)
		if isShare {
			subsidyFee = newFee * (zhios_order_relate_utils.StrToFloat64(v["share_bili"]) / 100)
		}
		subsidyFee = zhios_order_relate_utils.FloatFormat(subsidyFee, 6)
		res[v["coin_id"]] = subsidyFee
	}
	return res
}
func getTikTokOwnSubsidy(opt *comm_plan.PlanOpt, level int, isShare bool, rmd *md.CommissionParam) map[string]float64 {
	var res = make(map[string]float64)
	if zhios_order_relate_utils.StrToFloat64(rmd.TikTokTeamCommission) == 0 {
		return res
	}
	if opt.UserRate[level].SubsidyEnable != 1 {
		return res
	}
	if opt.UserRate[level].TikTokTeamSubsidyEnable != 1 {
		return res
	}
	if len(opt.UserRate[level].TikTokTeamSubsidyList) == 0 {
		return res
	}
	for _, v := range opt.UserRate[level].TikTokTeamSubsidyList {
		var newFee = zhios_order_relate_utils.StrToFloat64(rmd.TikTokTeamCommission)
		var subsidyFee float64 = 0
		subsidyFee = newFee * (zhios_order_relate_utils.StrToFloat64(v["self_bili"]) / 100)
		if isShare {
			subsidyFee = newFee * (zhios_order_relate_utils.StrToFloat64(v["share_bili"]) / 100)
		}
		subsidyFee = zhios_order_relate_utils.FloatFormat(subsidyFee, 6)
		res[v["coin_id"]] = subsidyFee
	}
	return res
}
func convertList2Map(a []*comm_plan.VirtualCoinCommission) (b map[string]float64) {
	b = make(map[string]float64)
	for _, i := range a {
		b[i.Cid] = i.Val
	}
	return b
}

// BatchGetPublicPlatoonRelateCommission 批量分佣 商品显示
func BatchGetPublicPlatoonRelateCommissionByGoods(engine *xorm.Engine, masterId string, PublicPlatoonRelateCommissionReqList []*md.PublicPlatoonRelateCommissionReq) (map[string]*md.PublicPlatoonRelateCommissionResp, error) {
	var resp = make(map[string]*md.PublicPlatoonRelateCommissionResp)
	for _, param := range PublicPlatoonRelateCommissionReqList {
		// 根据供应商
		rewardOpt, err := db.DbsPlanRewardByPvd(engine, param.Pvd)
		if err != nil {
			resp[param.Oid] = &md.PublicPlatoonRelateCommissionResp{}
			continue
		}
		if rewardOpt == nil {
			resp[param.Oid] = &md.PublicPlatoonRelateCommissionResp{}
			continue
		}
		if rewardOpt.State == 0 {
			resp[param.Oid] = &md.PublicPlatoonRelateCommissionResp{}
			continue
		}
		if rewardOpt.PlanCommissionId == 0 {
			resp[param.Oid] = &md.PublicPlatoonRelateCommissionResp{}
			continue
		}
		fmt.Println("抽成设置:", zhios_order_relate_utils.SerializeStr(rewardOpt))
		fmt.Println("commission id:", rewardOpt.PlanCommissionId)

		//TODO::计算共富收益
		commissionOpt, err := db.DbsPlanCommissionById(engine, rewardOpt.PlanCommissionId)
		if err != nil || commissionOpt == nil || commissionOpt.Id == 0 {
			resp[param.Oid] = &md.PublicPlatoonRelateCommissionResp{}
			continue
		}
		var subsidyTmp = make(map[string]*md.LvGrade)
		var tmp map[string]*md.LvGrade
		if err := json.Unmarshal([]byte(commissionOpt.Data), &tmp); err != nil {
			resp[param.Oid] = &md.PublicPlatoonRelateCommissionResp{}
			continue
		}
		for _, v := range tmp {
			subsidyTmp[zhios_order_relate_utils.IntToStr(v.Lv)] = v
		}
		lvGrade := subsidyTmp[param.UserLevel]
		var directPush = make(map[string]string)
		var selfRateList = make(map[string]string)
		var commonWealthSystem = make(map[string]string)
		if lvGrade != nil {
			for _, coinId := range lvGrade.ReturnType {
				var pendingAmount = param.PendingAmount
				if zhios_order_relate_utils.StrToInt(coinId) > 0 { //积分更换基数
					pendingAmount = param.PendingIntegral
				}
				selfRateList[coinId] = zhios_order_relate_utils.Float64ToStr(zhios_order_relate_utils.StrToFloat64(lvGrade.SelfRateList[coinId]) * zhios_order_relate_utils.StrToFloat64(pendingAmount) / 100)
				commonWealthSystem[coinId] = zhios_order_relate_utils.Float64ToStr(zhios_order_relate_utils.StrToFloat64(lvGrade.CommonWealthSystem[coinId]) * zhios_order_relate_utils.StrToFloat64(pendingAmount) / 100)
				directPush[coinId] = zhios_order_relate_utils.Float64ToStr(zhios_order_relate_utils.StrToFloat64(lvGrade.DirectPush[coinId]) * zhios_order_relate_utils.StrToFloat64(pendingAmount) / 100)
			}
		}

		resp[param.Oid] = &md.PublicPlatoonRelateCommissionResp{
			Uid:                param.Uid,
			PendingAmount:      param.PendingAmount,
			PendingIntegral:    param.PendingIntegral,
			Oid:                param.Oid,
			SelfRateList:       selfRateList,
			CommonWealthSystem: commonWealthSystem,
			DirectPush:         directPush,
		}
	}
	return resp, nil
}
