package svc

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"fmt"
	"github.com/shopspring/decimal"
	"strconv"
	"time"
	"xorm.io/xorm"
)

// DealUserAmount 处理给用户余额
func DealUserAmount(session *xorm.Session, req md.DealUserAmount) (err error) {
	if req.Amount < 0 {
		req.Amount = 0
	}
	//1、分布式锁阻拦
	requestIdPrefix := fmt.Sprintf(md.DealUserAmountRequestIdPrefix, req.Mid, req.Uid)
	cb, err := HandleBalanceDistributedLockForAmount(req.Mid, strconv.Itoa(req.Uid), requestIdPrefix)
	if err != nil {
		return err
	}
	if cb != nil {
		defer cb() // 释放锁
	}

	//2、计算&&组装数据
	now := time.Now()
	userAmount, err := GetUserAmount(session, req.Mid, req.Uid, true)
	if err != nil {
		return err
	}
	userAmountValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(userAmount))
	amountValue := decimal.NewFromFloat(req.Amount).RoundFloor(4)

	var finUserFlow = model.FinUserFlow{
		Uid:          req.Uid,
		Amount:       amountValue.String(),
		BeforeAmount: userAmount,
		OrdType:      req.OrderType,
		OrdId:        req.OrdId,
		OrdTitle:     req.Title,
		OrdAction:    req.OrdAction,
		OrdTime:      int(now.Unix()),
		State:        1,
		CreateAt:     now,
		UpdateAt:     now,
		Date:         now.Format("2006-01-02"),
	}
	if req.Kind == "add" {
		finUserFlow.Type = 0
		finUserFlow.AfterAmount = userAmountValue.Add(amountValue).RoundFloor(4).String()
	} else if req.Kind == "sub" {
		finUserFlow.Type = 1
		finUserFlow.AfterAmount = userAmountValue.Sub(amountValue).RoundFloor(4).String()
	} else {
		err = errors.New("错误的kind类型")
		return err
	}
	if zhios_order_relate_utils.StrToFloat64(finUserFlow.AfterAmount) < 0 {
		zhios_order_relate_utils.FilePutContents("user_amount_not", zhios_order_relate_utils.SerializeStr(map[string]interface{}{
			"uid":           finUserFlow.Uid,
			"amount":        finUserFlow.Amount,
			"before_amount": finUserFlow.BeforeAmount,
			"title":         finUserFlow.OrdTitle,
			"mid":           req.Mid,
		}))
		return errors.New("用户余额不足")
	}

	//3、插入 `fin_user_flow` 记录
	affected, err := session.Insert(&finUserFlow)
	if affected == 0 || err != nil {
		_ = zhios_order_relate_logx.Warn(err)
		return err
	}

	//4、修改 `user_profile`的fin_valid值 && 及缓存
	err = SetCacheUserAmount(session, req.Mid, finUserFlow.AfterAmount, req.Uid, true)
	if err != nil {
		return err
	}

	return nil
}

func DealUserAmountNew(session *xorm.Session, req md.DealUserAmount) (err error) {
	if req.Amount < 0 {
		req.Amount = 0
	}
	//1、分布式锁阻拦
	requestIdPrefix := fmt.Sprintf(md.DealUserAmountRequestIdPrefix, req.Mid, req.Uid)
	cb, err := HandleBalanceDistributedLockForAmount(req.Mid, strconv.Itoa(req.Uid), requestIdPrefix)
	if err != nil {
		return err
	}
	if cb != nil {
		defer cb() // 释放锁
	}

	//2、计算&&组装数据
	userAmount, err := GetUserAmount(session, req.Mid, req.Uid, false)
	if err != nil {
		return err
	}
	userAmountValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(userAmount))
	amountValue := decimal.NewFromFloat(req.Amount).RoundFloor(8)
	AfterAmount := ""
	if req.Kind == "add" {
		AfterAmount = userAmountValue.Add(amountValue).RoundFloor(8).String()
	} else if req.Kind == "sub" {
		AfterAmount = userAmountValue.Sub(amountValue).RoundFloor(8).String()
	} else {
		err = errors.New("错误的kind类型")
		return err
	}
	//4、修改 `user_profile`的fin_valid值 && 及缓存
	err = SetCacheUserAmount(session, req.Mid, AfterAmount, req.Uid, true)
	if err != nil {
		return err
	}

	return nil
}

func DealUserAmountFlow(session *xorm.Session, req md.DealUserAmount) (err error) {
	if req.Amount < 0 {
		req.Amount = 0
	}
	//2、计算&&组装数据
	now := time.Now()
	userAmount, err := GetUserAmount(session, req.Mid, req.Uid, false)
	if err != nil {
		return err
	}
	userAmountValue := decimal.NewFromFloat(zhios_order_relate_utils.StrToFloat64(userAmount))
	amountValue := decimal.NewFromFloat(req.Amount).RoundFloor(8)

	var finUserFlow = model.FinUserFlow{
		Uid:         req.Uid,
		Amount:      amountValue.String(),
		AfterAmount: userAmount,
		OrdType:     req.OrderType,
		OrdId:       req.OrdId,
		OrdTitle:    req.Title,
		OrdAction:   req.OrdAction,
		OrdTime:     int(now.Unix()),
		State:       1,
		CreateAt:    now,
		UpdateAt:    now,
		Memo:        req.Num,
		Date:        now.Format("2006-01-02"),
	}
	finUserFlow.Type = 0
	finUserFlow.BeforeAmount = userAmountValue.Sub(amountValue).RoundFloor(8).String()
	if zhios_order_relate_utils.StrToFloat64(finUserFlow.AfterAmount) < 0 {
		zhios_order_relate_utils.FilePutContents("user_amount_not", zhios_order_relate_utils.SerializeStr(map[string]interface{}{
			"uid":           finUserFlow.Uid,
			"amount":        finUserFlow.Amount,
			"before_amount": finUserFlow.BeforeAmount,
			"title":         finUserFlow.OrdTitle,
			"mid":           req.Mid,
		}))
		return errors.New("用户余额不足")
	}

	//3、插入 `fin_user_flow` 记录
	affected, err := session.Insert(&finUserFlow)
	if affected == 0 || err != nil {
		_ = zhios_order_relate_logx.Warn(err)
		return err
	}
	return nil
}

// GetUserAmount 获取用户余额
func GetUserAmount(session *xorm.Session, masterId string, uid int, isForceUpdate bool) (amount string, err error) {
	if isForceUpdate {
		userAmount, err1 := db.UserProfileFindByIDWithSession(session, uid)
		if err1 != nil {
			return amount, err1
		}
		if userAmount == nil {
			amount = "0"
		} else {
			amount = userAmount.FinValid
		}
		//将获取到的余额值缓存至redis
		_ = SetCacheUserAmount(session, masterId, amount, uid, false)
	} else {
		redisKey := fmt.Sprintf(md.UserAmountRedisKey, masterId, uid)
		amount, err = cache.GetString(redisKey)
		if err != nil && err.Error() != "redigo: nil returned" {
			return
		}
		userAmount, err1 := db.UserProfileFindByIDWithSession(session, uid)
		if err1 != nil {
			return amount, err1
		}
		if userAmount == nil {
			amount = "0"
		} else {
			amount = userAmount.FinValid
		}
		//将获取到的余额值缓存至redis
		_ = SetCacheUserAmount(session, masterId, amount, uid, false)
	}
	return amount, nil
}

// SetCacheUserAmount 设置缓存的用户余额
func SetCacheUserAmount(session *xorm.Session, masterId, amount string, uid int, isUpdateDb bool) error {
	redisKey := fmt.Sprintf(md.UserAmountRedisKey, masterId, uid)
	if isUpdateDb {
		_, err := session.Where("uid=?", uid).Update(model.UserProfile{
			Uid:      uid,
			FinValid: amount,
		})
		if err != nil {
			return err
		}
	}
	//_, err := cache.Set(redisKey, int64(utils.StrToFloat64(amount)))
	//TODO::默认缓存1小时 (先调整为 2 min)
	_, err := cache.SetEx(redisKey, zhios_order_relate_utils.StrToFloat64(amount), 60*0.5)
	if err != nil {
		return err
	}
	return nil
}
